#include "Board.h"
#include "tmc/ic/TMC4330/TMC4330.h"

#define TMC4330_MAX_VELOCITY      2147483647
#define TMC4330_MAX_ACCELERATION  16777215

#define MOTORS 1

static uint32 right(uint8 motor, int32 velocity);
static uint32 left(uint8 motor, int32 velocity);
static uint32 rotate(uint8 motor, int32 velocity);
static uint32 stop(uint8 motor);
static uint32 moveTo(uint8 motor, int32 position);
static uint32 moveBy(uint8 motor, int32 *ticks);
static uint32 GAP(uint8 type, uint8 motor, int32 *value);
static uint32 SAP(uint8 type, uint8 motor, int32 value);

static void readRegister(uint8 address, int32 *value);
static void writeRegister(uint8 address, int32 value);

static void periodicJob(uint32 tick);
static void checkErrors(uint32 tick);
static void deInit(void);
static uint32 userFunction(uint8 type, uint8 motor, int32 *value);
static uint8 reset();
static int32 discardVelocityDecimals(int32 value);

typedef struct
{
	IOPinTypeDef  *TARGET_REACHED;
	IOPinTypeDef  *NRST;
	IOPinTypeDef  *START;
	IOPinTypeDef  *HOME_REF;
	IOPinTypeDef  *STOP_R;
	IOPinTypeDef  * STOP_L;
	IOPinTypeDef  *INTR;
} PinsTypeDef;

static PinsTypeDef Pins;

SPIChannelTypeDef *TMC4330_SPIChannel;
TMC4330TypeDef TMC4330;
ConfigurationTypeDef *TMC4330_config;

void tmc4330_writeDatagram(uint8 address, uint8 x1, uint8 x2, uint8 x3, uint8 x4)
{
	int value;

	TMC4330.status = TMC4330_SPIChannel->readWrite(address|0x80, FALSE);
	TMC4330_SPIChannel->readWrite(x1, FALSE);
	TMC4330_SPIChannel->readWrite(x2, FALSE);
	TMC4330_SPIChannel->readWrite(x3, FALSE);
	TMC4330_SPIChannel->readWrite(x4, TRUE);

	value = x1;
	value <<= 8;
	value |= x2;
	value <<= 8;
	value |= x3;
	value <<= 8;
	value |= x4;


	TMC4330_config->shadowRegister[address & 0x7F] = value;
}

void tmc4330_writeInt(uint8 address, int value)
{
	tmc4330_writeDatagram(address, 0xFF & (value>>24), 0xFF & (value>>16), 0xFF & (value>>8), 0xFF & (value>>0));
}

int tmc4330_readInt(uint8 address)
{
	int value;

	address &= 0x7F;

	// If not readable, return the shadow register copy
	if(!IS_READABLE(TMC4330.registerAccess[address]))
		return TMC4330_config->shadowRegister[address];

	TMC4330_SPIChannel->readWrite(address, FALSE);
	TMC4330_SPIChannel->readWrite(0, FALSE);
	TMC4330_SPIChannel->readWrite(0, FALSE);
	TMC4330_SPIChannel->readWrite(0, FALSE);
	TMC4330_SPIChannel->readWrite(0, TRUE);

	TMC4330.status = TMC4330_SPIChannel->readWrite(address, FALSE);
	value = TMC4330_SPIChannel->readWrite(0, FALSE);
	value <<= 8;
	value |= TMC4330_SPIChannel->readWrite(0, FALSE);
	value <<= 8;
	value |= TMC4330_SPIChannel->readWrite(0, FALSE);
	value <<= 8;
	value |= TMC4330_SPIChannel->readWrite(0, TRUE);

	return value;
}

static int32 discardVelocityDecimals(int32 value)
{
	if(abs(value) > 8000000)
	{
		value = (value < 0) ? -8000000 : 8000000;
	}
	return value << 8;
}

static uint32 rotate(uint8 motor, int32 velocity)
{
	if(motor >= MOTORS)
		return ERROR_MOTOR;

	tmc4330_writeInt(TMC4330_RAMPMODE, ~TMC4330_RAMP_POSITION & tmc4330_readInt(TMC4330_RAMPMODE));
	tmc4330_writeInt(TMC4330_VMAX, discardVelocityDecimals(velocity));

	return ERROR_NONE;
}

static uint32 right(uint8 motor, int32 velocity)
{
	return rotate(motor, velocity);
}

static uint32 left(uint8 motor, int32 velocity)
{
	return rotate(motor, -velocity);
}

static uint32 stop(uint8 motor)
{
	return rotate(motor, 0);
}

static uint32 moveTo(uint8 motor, int32 position)
{
	if(motor >= MOTORS)
		return ERROR_MOTOR;

	tmc4330_writeInt(TMC4330_RAMPMODE, TMC4330_RAMP_POSITION | tmc4330_readInt(TMC4330_RAMPMODE));
	tmc4330_writeInt(TMC4330_X_TARGET, position);

	return ERROR_NONE;
}

static uint32 moveBy(uint8 motor, int32 *ticks)
{
	// determine actual position and add numbers of ticks to move
	*ticks = tmc4330_readInt(TMC4330_XACTUAL) + *ticks;

	return moveTo(motor, *ticks);
}

// \xml <?xml version="1.0" encoding="UTF-8" standalone="no"?>
// \xml <tmc:product_module author="?" xmlns:tmc="http://www.trinamic.com" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xsi:schemaLocation="http://www.trinamic.com https://www.trinamic.com/fileadmin/xml/TMC_Schema.xsd">
// \xml <name>TMC4330-EVAL</name>
// \xml <name_fw_version>4330</name_fw_version>
// \xml <hardware_version>1.0</hardware_version>
// \xml <software_version>3.00</software_version>
static uint32 handleParameter(u8 readWrite, u8 motor, u8 type, int32 *value)
{
	u32 errors = ERROR_NONE;

	if(motor >= MOTORS)
		return ERROR_MOTOR;

		// \xml <axis_parameters axis="0">
	switch(type)
	{
	case 0:
		// \xml <tmc:axis_parameter type="0" name="Target position" identifier="par::TargetPosition">
		// \xml   <desc>The desired target position in position mode.</desc>
		// \xml   <tmc:value_signed min="#s32_MIN#" max="#s32_MAX#" default="0" unit="[usteps]" access="RW"></tmc:value_signed>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			*value = tmc4330_readInt(TMC4330_X_TARGET);
		} else if(readWrite == WRITE) {
			tmc4330_writeInt( TMC4330_X_TARGET, *value);
		}
		break;
	case 1:
		// \xml <tmc:axis_parameter type="1" name="Actual position" identifier="par::ActualPosition">
		// \xml   <desc>The actual position of the motor. Stop the motor before overwriting it. Should normally only be overwritten for reference position setting.</desc>
		// \xml   <tmc:value_signed min="#s32_MIN#" max="#s32_MAX#" default="0" unit="[usteps]" access="RW"></tmc:value_signed>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			*value = tmc4330_readInt(TMC4330_XACTUAL);
		} else if(readWrite == WRITE) {
			tmc4330_writeInt( TMC4330_XACTUAL, *value);
		}
		break;
	case 2:
		// \xml <tmc:axis_parameter type="2" name="Target speed" identifier="par::TargetVelocity">
		// \xml   <desc>The desired speed in velocity mode. Not valid in position mode.</desc>
		// \xml   <tmc:value_signed min="#s32_MIN#" max="#s32_MAX#" default="0" unit="[pps]" access="RW"></tmc:value_signed>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			*value = tmc4330_readInt(TMC4330_VMAX) >> 8;
		} else if(readWrite == WRITE) {
			tmc4330_writeInt( TMC4330_VMAX, discardVelocityDecimals(*value));
		}
		break;
	case 3:
		// \xml <tmc:axis_parameter type="3" name="Actual speed" identifier="par::ActualVelocity">
		// \xml   <desc>The actual speed of the motor.</desc>
		// \xml   <tmc:value_signed min="#s32_MIN#" max="#s32_MAX#" default="0" unit="[pps]" access="R"></tmc:value_signed>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			*value = tmc4330_readInt(TMC4330_VACTUAL);
		} else if(readWrite == WRITE) {
			errors |= ERROR_TYPE;
		}
		break;
	case 4:
		// \xml <tmc:axis_parameter type="4" name="Maximum speed" identifier="par::MaxVelocity">
		// \xml   <desc>The maximum speed used for positioning ramps.</desc>
		// \xml   <tmc:value_signed min="#s32_MIN#" max="#s32_MAX#" default="150000" unit="[pps]" access="RW"></tmc:value_signed>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			*value = tmc4330_readInt(TMC4330_VMAX) >> 8;
		} else if(readWrite == WRITE) {
			tmc4330_writeInt( TMC4330_VMAX, discardVelocityDecimals(*value));
		}
		break;
	case 5:
		// \xml <tmc:axis_parameter type="5" name="Maximum acceleration" identifier="par::MaxAcceleration">
		// \xml   <desc>Maximum acceleration in positioning ramps. Acceleration and deceleration value in velocity mode.</desc>
		// \xml   <tmc:value_unsigned min="0" max="#TMC4330_MAX_ACCELERATION#" default="1000" unit="[pps/s]" access="RW"></tmc:value_unsigned>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			*value = tmc4330_readInt(TMC4330_AMAX)>>2;
		} else if(readWrite == WRITE) {
			if(*value & ~0x3FFFFF)
			{
				errors |= ERROR_VALUE;
			}
			else
			{
				tmc4330_writeInt( TMC4330_AMAX, *value<<2);
			}
		}
		break;
	case 8:
		// \xml <tmc:axis_parameter type="8" name="Position reached flag" identifier="par::PositionReachedFlag">
		// \xml   <desc>This flag is always set when target position and actual position are equal.</desc>
		// \xml   <tmc:value_bool desc_false="Target position not reached" desc_true="Target position reached" default="false" access="R"></tmc:value_bool>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			*value = (tmc4330_readInt(TMC4330_STATUS) & (1<<0))? 1:0;
		} else if(readWrite == WRITE)
			errors |= ERROR_TYPE;
		break;
	case 14:
		// \xml <tmc:axis_parameter type="14" name="Ramp type" identifier="par::RampType">
		// \xml   <desc>Actual type of ramp.</desc>
		// \xml   <tmc:value_bool desc_false="Trapezoidal ramp" desc_true="S-shaped ramp" default="false" access="RW"></tmc:value_bool>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			*value = tmc4330_readInt(TMC4330_RAMPMODE) << 1;
		} else if(readWrite == WRITE) {
			tmc4330_writeInt(TMC4330_RAMPMODE, (*value) ? TMC4330_RAMP_SSHAPE : TMC4330_RAMP_TRAPEZ);
		}
		break;
	case 15:
		// \xml <tmc:axis_parameter type="15" name="Velocity VSTART" identifier="par::StartVelocity">
		// \xml   <desc>Motor start velocity (in position mode only). Do not set VSTART higher than VSTOP.</desc>
		// \xml   <tmc:value_unsigned min="0" max="#TMC4330_MAX_VELOCITY#" default="1" unit="[pps]" access="RW"></tmc:value_unsigned>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			*value = tmc4330_readInt(TMC4330_VSTART);
		} else if(readWrite == WRITE) {
			tmc4330_writeInt( TMC4330_VSTART, *value);
		}
		break;
	case 16:
		// \xml <tmc:axis_parameter type="16" name="Acceleration AStart" identifier="par::AStart">
		// \xml   <desc>First acceleration between VSTART and V1 (in position mode only).</desc>
		// \xml   <tmc:value_unsigned min="0" max="#TMC4330_MAX_ACCELERATION#" default="100" unit="[pps/s]" access="RW"></tmc:value_unsigned>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			*value = tmc4330_readInt(TMC4330_ASTART) >> 2;
		} else if(readWrite == WRITE) {
			if(*value & ~0x3FFFFF)
			{
				errors |= ERROR_VALUE;
			}
			else
			{
				tmc4330_writeInt( TMC4330_ASTART, *value<<2);
			}
		}
		break;
	case 17:
		// \xml <tmc:axis_parameter type="17" name="Maximum Deceleration" identifier="par::MaxDeceleration">
		// \xml   <desc>Maximum deceleration in positioning ramps. Used to decelerate from maximum positioning speed (axis parameter 4) to velocity V1</desc>
		// \xml   <tmc:value_unsigned min="0" max="#TMC4330_MAX_ACCELERATION#" default="5" unit="[pps/s]" access="RW"></tmc:value_unsigned>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			*value = tmc4330_readInt(TMC4330_DMAX) >> 2;
		} else if(readWrite == WRITE) {
			if(*value & ~0x3FFFFF)
			{
				errors |= ERROR_VALUE;
			}
			else
			{
				tmc4330_writeInt( TMC4330_DMAX, *value<<2);
			}
		}
		break;
	case 18:
		// \xml <tmc:axis_parameter type="18" name="Velocity VBreak" identifier="par::VBreak">
		// \xml   <desc>First acceleration / deceleration phase target velocity (in position mode only). Setting this value to 0 turns off the first acceleration / deceleration phase, maximum acceleration (axis parameter 5) and maximum deceleration (axis parameter 17) are used only.</desc>
		// \xml   <tmc:value_unsigned min="0" max="#TMC4330_MAX_VELOCITY#" default="2" unit="[pps]" access="RW"></tmc:value_unsigned>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			*value = tmc4330_readInt(TMC4330_VBREAK);
		} else if(readWrite == WRITE) {
			tmc4330_writeInt( TMC4330_VBREAK, *value);
		}
		break;
	case 19:
		// \xml <tmc:axis_parameter type="19" name="Deceleration DFinal" identifier="par::DFinal">
		// \xml   <desc>Deceleration between V1 and VSTOP (in positioning mode only).</desc>
		// \xml   <tmc:value_unsigned min="0" max="#TMC4330_MAX_ACCELERATION#" default="200" unit="[pps/s]" access="RW"></tmc:value_unsigned>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			*value = tmc4330_readInt(TMC4330_DFINAL) >> 2;
		} else if(readWrite == WRITE) {
			if(*value & ~0x3FFFFF)
			{
				errors |= ERROR_VALUE;
			}
			else
			{
				tmc4330_writeInt( TMC4330_DFINAL, *value<<2);
			}
		}
		break;
	case 20:
		// \xml <tmc:axis_parameter type="20" name="Velocity VSTOP" identifier="par::StopVelocity">
		// \xml   <desc>Motor stop velocity (in position mode only).</desc>
		// \xml   <tmc:value_unsigned min="0" max="#TMC4330_MAX_VELOCITY#" default="2" unit="[pps]" access="RW"></tmc:value_unsigned>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			*value = tmc4330_readInt(TMC4330_VSTOP);
		} else if(readWrite == WRITE) {
			tmc4330_writeInt( TMC4330_VSTOP, *value);
		}
		break;
	case 21:
		// \xml <tmc:axis_parameter type="21" name="Deceleration DStop" identifier="par::DSTOP">
					// \xml   <desc>Deceleration between V1 and VSTOP (in positioning mode only).</desc>
		// \xml   <tmc:value_unsigned min="0" max="#TMC4330_MAX_ACCELERATION#" default="200" unit="[pps/s]" access="RW"></tmc:value_unsigned>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			*value = tmc4330_readInt(TMC4330_DSTOP);
		} else if(readWrite == WRITE) {
			if(*value & ~0x3FFFFF)
			{
				errors |= ERROR_VALUE;
			}
			else
			{
				tmc4330_writeInt( TMC4330_DSTOP, *value);
			}
		}
		break;
	case 22:
		// \xml <tmc:axis_parameter type="22" name="Bow 1" identifier="par::BOW1">
		// \xml   <desc>First bow value of a complete velocity ramp.</desc>
		// \xml   <tmc:value_unsigned min="0" max="#TMC4330_MAX_ACCELERATION#" default="200" unit="[pps/s]" access="RW"></tmc:value_unsigned>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			*value = tmc4330_readInt(TMC4330_BOW1);
		} else if(readWrite == WRITE) {
			tmc4330_writeInt( TMC4330_BOW1, *value);
		}
		break;
	case 23:
		// \xml <tmc:axis_parameter type="23" name="Bow 2" identifier="par::BOW2">
		// \xml   <desc>Second bow value of a complete velocity ramp.</desc>
		// \xml   <tmc:value_unsigned min="0" max="#TMC4330_MAX_ACCELERATION#" default="200" unit="[pps/s]" access="RW"></tmc:value_unsigned>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			*value = tmc4330_readInt(TMC4330_BOW2);
		} else if(readWrite == WRITE) {
			tmc4330_writeInt( TMC4330_BOW2, *value);
		}
		break;
	case 24:
		// \xml <tmc:axis_parameter type="24" name="Bow 3" identifier="par::BOW3">
		// \xml   <desc>Third bow value of a complete velocity ramp.</desc>
		// \xml   <tmc:value_unsigned min="0" max="#TMC4330_MAX_ACCELERATION#" default="200" unit="[pps/s]" access="RW"></tmc:value_unsigned>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			*value = tmc4330_readInt(TMC4330_BOW3);
		} else if(readWrite == WRITE) {
			tmc4330_writeInt( TMC4330_BOW3, *value);
		}
		break;
	case 25:
		// \xml <tmc:axis_parameter type="25" name="Bow 4" identifier="par::BOW4">
		// \xml   <desc>Fourth bow value of a complete velocity ramp.</desc>
		// \xml   <tmc:value_unsigned min="0" max="#TMC4330_MAX_ACCELERATION#" default="200" unit="[pps/s]" access="RW"></tmc:value_unsigned>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			*value = tmc4330_readInt(TMC4330_BOW4);
		} else if(readWrite == WRITE) {
			tmc4330_writeInt( TMC4330_BOW4, *value);
		}
		break;
	case 26:
		// \xml <tmc:axis_parameter type="26" name="Virtual stop left" identifier="par::VirtualStopLeft">
		// \xml   <desc>Virtual left stop that triggers a stop event when XACTUAL is below this parameter.</desc>
		// \xml   <tmc:value_signed min="#s32_MIN#" max="#s32_MAX#" default="0" unit="[usteps]" access="RW"></tmc:value_signed>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			*value = tmc4330_readInt(TMC4330_VIRT_STOP_LEFT);
		} else if(readWrite == WRITE) {
			tmc4330_writeInt( TMC4330_VIRT_STOP_LEFT, *value);
		}
		break;
	case 27:
		// \xml <tmc:axis_parameter type="27" name="Virtual stop right" identifier="par::VirtualStopRight">
		// \xml   <desc>Virtual right stop that triggers a stop event when XACTUAL is over this parameter.</desc>
		// \xml   <tmc:value_signed min="#s32_MIN#" max="#s32_MAX#" default="0" unit="[usteps]" access="RW"></tmc:value_signed>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			*value = tmc4330_readInt(TMC4330_VIRT_STOP_RIGHT);
		} else if(readWrite == WRITE) {
			tmc4330_writeInt( TMC4330_VIRT_STOP_RIGHT, *value);
		}
		break;
	case 214:
		// \xml <tmc:axis_parameter type="214" name="Power Down Delay" identifier="par::PowerDownDelay">
		// \xml   <desc>Standstill period before the current will be ramped down to standby current. The standard value is 200 (which means 2000ms).</desc>
		// \xml   <tmc:value_signed min="0" max="#u16_MAX#" default="0" unit="[10 ms]" access="RW"></tmc:value_signed>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			*value = tmc4330_readInt(TMC4330_STDBY_DELAY);
		} else if(readWrite == WRITE) {
			tmc4330_writeInt( TMC4330_STDBY_DELAY, *value*160000);
		}
		break;
	default:
		errors |= ERROR_TYPE;
		break;
	}
	// \xml </axis_parameters>
	return errors;
}
// \xml </tmc:product_module>

static uint32 SAP(uint8 type, uint8 motor, int32 value)
{
	return handleParameter(WRITE, motor, type, &value);
}

static uint32 GAP(uint8 type, uint8 motor, int32 *value)
{
	return handleParameter(READ, motor, type, value);
}

static void writeRegister(uint8 address, int32 value)
{
	tmc4330_writeInt(address, value);
}

static void readRegister(uint8 address, int32 *value)
{
	*value = tmc4330_readInt(address);
}

static void periodicJob(uint32 tick)
{
	tmc4330_periodicJob(tick, &TMC4330, TMC4330_config);
	if(TMC4330_config->state != CONFIG_READY)
		reset();
}

static void checkErrors(uint32 tick)
{
	UNUSED(tick);
	Evalboards.ch1.errors = 0;
}


static uint32 userFunction(uint8 type, uint8 motor, int32 *value)
{
	uint32 errors = 0;
	UNUSED(motor);

	switch(type)
	{
	case 0:	// simulate left/right reference switches, set high to support external ref swiches*/
		/*
		 * The the TMC4361 ref switch input is pulled high by external resistor an can be pulled low either by
		 * this µC or external signal. To use external signal make sure the signals from µC are high or floating.
		 */
		if(!(*value & ~3))
		{
			if(*value & (1<<0))
			{
				HAL.IOs->config->toInput(Pins.STOP_R); // pull up -> set it to floating causes high
			}
			else
			{
				HAL.IOs->config->toOutput(Pins.STOP_R);
				HAL.IOs->config->setLow(Pins.STOP_R);
			}

			if(*value & (1<<1))
			{
				HAL.IOs->config->toInput(Pins.STOP_L); // pull up -> set it to floating causes high
			}
			else
			{
				HAL.IOs->config->toOutput(Pins.STOP_L);
				HAL.IOs->config->setLow(Pins.STOP_L);
			}
		}
		else
		{
			errors |= ERROR_VALUE;
		}
		break;
	case 1:	// simulate reference switche HOME_REF, set high to support external ref swiches
		/*
		 * The the TMC43x1 ref switch input is pulled high by external resistor an can be pulled low either by
		 * this µC or external signal. To use external signal make sure the signals from µC are high or floating.
		 */

		if(*value)
		{
			HAL.IOs->config->toInput(Pins.HOME_REF); // pull up -> set it to floating causes high
		}
		else
		{
			HAL.IOs->config->toOutput(Pins.HOME_REF);
			HAL.IOs->config->setLow(Pins.HOME_REF);
		}
		break;
	case 255:
		Evalboards.ch2.config->reset();
		Evalboards.ch1.config->reset();
		break;
	default:
		errors |= ERROR_TYPE;
		break;
	}
	return errors;
}

static void deInit(void)
{
	HAL.IOs->config->setLow(Pins.NRST);

	HAL.IOs->config->reset(Pins.STOP_L);
	HAL.IOs->config->reset(Pins.STOP_R);
	HAL.IOs->config->reset(Pins.HOME_REF);
	HAL.IOs->config->reset(Pins.START);
	HAL.IOs->config->reset(Pins.INTR);
	HAL.IOs->config->reset(Pins.TARGET_REACHED);
	HAL.IOs->config->reset(Pins.NRST);

	HAL.SPI->ch2.reset();
}

static uint8 reset()
{
	if(TMC4330_config->state == CONFIG_READY)
	{
		HAL.IOs->config->setLow(Pins.NRST);
		TMC4330_config->state = CONFIG_RESET;
	}
	else
	{
		HAL.IOs->config->setHigh(Pins.NRST);

		// set STEP/DIR high length to 4 clock cycles
		tmc4330_writeInt(TMC4330_STP_LENGTH_ADD, 0x04);

		TMC4330_config->state = CONFIG_READY;
		Evalboards.ch2.config->reset();
	}
	return 1;
}

void TMC4330_init(void)
{
	tmc4330_initConfig(&TMC4330);

	Pins.INTR            = &HAL.IOs->pins->DIO5;
	Pins.STOP_L          = &HAL.IOs->pins->DIO12;
	Pins.STOP_R          = &HAL.IOs->pins->DIO13;
	Pins.HOME_REF        = &HAL.IOs->pins->DIO14;
	Pins.START           = &HAL.IOs->pins->DIO15;
	Pins.NRST            = &HAL.IOs->pins->DIO17;
	Pins.TARGET_REACHED  = &HAL.IOs->pins->DIO18;

	HAL.IOs->config->toOutput(Pins.NRST);
	HAL.IOs->config->toOutput(Pins.STOP_L);
	HAL.IOs->config->toOutput(Pins.STOP_R);
	HAL.IOs->config->toOutput(Pins.HOME_REF);
	HAL.IOs->config->toOutput(Pins.START);

	HAL.IOs->config->setHigh(Pins.NRST);
	HAL.IOs->config->setHigh(Pins.STOP_L);
	HAL.IOs->config->setHigh(Pins.STOP_R);
	HAL.IOs->config->setHigh(Pins.HOME_REF);
	HAL.IOs->config->setHigh(Pins.START);

	HAL.IOs->config->toInput(Pins.INTR);
	HAL.IOs->config->toInput(Pins.TARGET_REACHED);

	TMC4330_SPIChannel = &HAL.SPI->ch1;
	TMC4330_SPIChannel->CSN = &HAL.IOs->pins->SPI1_CSN;

	TMC4330_config = Evalboards.ch1.config;

	tmc4330_writeInt(TMC4330_RAMPMODE, TMC4330_RAMP_TRAPEZ);

	Evalboards.ch1.config->reset        = reset;
	Evalboards.ch1.config->configIndex  = 0;
	Evalboards.ch1.config->state        = CONFIG_RESET;

	Evalboards.ch1.rotate               = rotate;
	Evalboards.ch1.right                = right;
	Evalboards.ch1.left                 = left;
	Evalboards.ch1.stop                 = stop;
	Evalboards.ch1.GAP                  = GAP;
	Evalboards.ch1.SAP                  = SAP;
	Evalboards.ch1.moveTo               = moveTo;
	Evalboards.ch1.moveBy               = moveBy;
	Evalboards.ch1.writeRegister        = writeRegister;
	Evalboards.ch1.readRegister         = readRegister;
	Evalboards.ch1.periodicJob          = periodicJob;
	Evalboards.ch1.userFunction         = userFunction;
	Evalboards.ch1.checkErrors          = checkErrors;
	Evalboards.ch1.numberOfMotors       = MOTORS;
	Evalboards.ch1.deInit               = deInit;
};
