#include "Board.h"
#include "tmc/BoardAssignment.h"
#include "tmc/ic/TMC4361A/TMC4361A.h"

//todo CHECK 3: enter correct values (JE)
#define TMC4361A_MAX_VELOCITY     2147483647
#define TMC4361_MAX_ACCELERATION  16777215

#define MOTORS 1

static uint32 right(uint8 motor, int32 velocity);
static uint32 left(uint8 motor, int32 velocity);
static uint32 rotate(uint8 motor, int32 velocity);
static uint32 stop(uint8 motor);
static uint32 moveTo(uint8 motor, int32 position);
static uint32 moveBy(uint8 motor, int32 *ticks);
static uint32 GAP(uint8 type, uint8 motor, int32 *value);
static uint32 SAP(uint8 type, uint8 motor, int32 value);

static void readRegister(uint8 address, int32 *value);
static void writeRegister(uint8 address, int32 value);

static void periodicJob(uint32 tick);
static void checkErrors(uint32 tick);
static void deInit(void);
static uint32 userFunction(uint8 type, uint8 motor, int32 *value);
static uint8 reset();
static uint8 restore();
static int32 discardVelocityDecimals(int32 value);

typedef struct
{
	IOPinTypeDef  *TARGET_REACHED;
	IOPinTypeDef  *NRST;
	IOPinTypeDef  *FREEZE;
	IOPinTypeDef  *START;
	IOPinTypeDef  *HOME_REF;
	IOPinTypeDef  *STOP_R;
	IOPinTypeDef  *STOP_L;
	IOPinTypeDef  *INTR;
	IOPinTypeDef  *STANDBY_CLK;
} PinsTypeDef;

static PinsTypeDef Pins;

SPIChannelTypeDef *TMC4361A_SPIChannel;
TMC4361ATypeDef TMC4361A;
ConfigurationTypeDef *TMC4361A_config;

void tmc4361A_writeDatagram(uint8 address, uint8 x1, uint8 x2, uint8 x3, uint8 x4)
{
	int value;

	TMC4361A.status = TMC4361A_SPIChannel->readWrite(address|0x80, FALSE);
	TMC4361A_SPIChannel->readWrite(x1, FALSE);
	TMC4361A_SPIChannel->readWrite(x2, FALSE);
	TMC4361A_SPIChannel->readWrite(x3, FALSE);
	TMC4361A_SPIChannel->readWrite(x4, TRUE);

	value = (x1 << 24) | (x2 << 16) | (x3 << 8) | x4;

	TMC4361A_config->shadowRegister[address & 0x7F] = value;
}

void tmc4361A_writeInt(uint8 address, int value)
{
	tmc4361A_writeDatagram(address, value>>24, value>>16, value>>8, value);
}

int tmc4361A_readInt(uint8 address)
{
	int value;

	address &= 0x7F;

	if(!IS_READABLE(TMC4361A.registerAccess[address]))
		return TMC4361A_config->shadowRegister[address];

	TMC4361A_SPIChannel->readWrite(address, FALSE);
	TMC4361A_SPIChannel->readWrite(0, FALSE);
	TMC4361A_SPIChannel->readWrite(0, FALSE);
	TMC4361A_SPIChannel->readWrite(0, FALSE);
	TMC4361A_SPIChannel->readWrite(0, TRUE);

	TMC4361A.status = TMC4361A_SPIChannel->readWrite(address, FALSE);
	value = TMC4361A_SPIChannel->readWrite(0, FALSE);
	value <<= 8;
	value |= TMC4361A_SPIChannel->readWrite(0, FALSE);
	value <<= 8;
	value |= TMC4361A_SPIChannel->readWrite(0, FALSE);
	value <<= 8;
	value |= TMC4361A_SPIChannel->readWrite(0, TRUE);

	return value;
}

// The cover function emulates the SPI readWrite function
static uint8 tmc4361A_cover(uint8 data, uint8 lastTransfer)
{
	static uint64 coverIn = 0;     // read from squirrel
	static uint64 coverOut = 0;    // write to squirrel
	static uint8 coverLength = 0;  // data to be written

	uint8 out = 0; // return value of this function

	// buffer outgoing data
	coverOut <<= 8;    // shift left by one byte to make room for the next byte
	coverOut |= data;  // add new byte to be written
	coverLength++;     // count outgoing bytes

	// return read and buffered byte to be returned
	out = coverIn >> 56;  // output last received byte
	coverIn <<= 8;        // shift by one byte to read this next time

	if(lastTransfer)
	{
		/* Write data to cover register(s). The lower 4 bytes go into the cover low register,
		 * the higher 4 bytes, if present, go into the cover high register.
		 * The datagram needs to be sent twice, otherwise the read buffer will be delayed by
		 * one read/write datagram.
		 */

		// Send the buffered datagram & wait a bit before continuing so the 4361 can complete the datagram to the driver
		// measured delay between COVER_LOW transmission and COVER_DONE flag: ~90s -> 1 ms more than enough
		// todo CHECK 3: Delay measurement only done on TMC4361, not 4361A - make sure the required delay didnt change (LH) #1
		if(coverLength > 4)
			tmc4361A_writeInt(TMC4361A_COVER_HIGH_WR, coverOut >> 32);
		tmc4361A_writeInt(TMC4361A_COVER_LOW_WR, coverOut & 0xFFFFFFFF);
		wait(1);

		// Trigger a re-send by writing the low register again
		tmc4361A_writeInt(TMC4361A_COVER_LOW_WR, coverOut & 0xFFFFFFFF);

		// Read the reply
		coverIn = 0;
		if(coverLength > 4)
			coverIn |= (uint64) tmc4361A_readInt(TMC4361A_COVER_DRV_HIGH_RD) << 32;
		coverIn |= tmc4361A_readInt(TMC4361A_COVER_DRV_LOW_RD);
		coverIn <<= (8-coverLength) * 8; // Shift the highest byte of the reply to the highest byte of the buffer uint64

		// Clear write buffer
		coverOut = 0;
		coverLength=0;
	}

	return out; // return buffered read byte
}

static int32 discardVelocityDecimals(int32 value)
{
	if(abs(value) > 8000000)
	{
		value = (value < 0) ? -8000000 : 8000000;
	}
	return value << 8;
}

static uint32 rotate(uint8 motor, int32 velocity)
{
	if(motor >= MOTORS)
		return ERROR_MOTOR;

	tmc4361A_writeInt(TMC4361A_RAMPMODE, ~TMC4361A_RAMP_POSITION & tmc4361A_readInt(TMC4361A_RAMPMODE));
	tmc4361A_writeInt(TMC4361A_VMAX, discardVelocityDecimals(velocity));

	return ERROR_NONE;
}

static uint32 right(uint8 motor, int32 velocity)
{
	return rotate(motor, velocity);
}

static uint32 left(uint8 motor, int32 velocity)
{
	return rotate(motor, -velocity);
}

static uint32 stop(uint8 motor)
{
	return rotate(motor, 0);
}

static uint32 moveTo(uint8 motor, int32 position)
{
	if(motor >= MOTORS)
		return ERROR_MOTOR;

	tmc4361A_writeInt(TMC4361A_RAMPMODE, TMC4361A_RAMP_POSITION | tmc4361A_readInt(TMC4361A_RAMPMODE));
	tmc4361A_writeInt(TMC4361A_X_TARGET, position);

	return ERROR_NONE;
}

static uint32 moveBy(uint8 motor, int32 *ticks)
{
	// determine actual position and add numbers of ticks to move
	*ticks = tmc4361A_readInt(TMC4361A_XACTUAL) + *ticks;

	return moveTo(motor, *ticks);
}

// \xml <?xml version="1.0" encoding="UTF-8" standalone="no"?>
// \xml <tmc:product_module author="?" xmlns:tmc="http://www.trinamic.com" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xsi:schemaLocation="http://www.trinamic.com https://www.trinamic.com/fileadmin/xml/TMC_Schema.xsd">
// \xml <name>TMC4361-EVAL</name>
// \xml <name_fw_version>4361</name_fw_version>
// \xml <hardware_version>1.0</hardware_version>
// \xml <software_version>3.00</software_version>
static uint32 handleParameter(u8 readWrite, u8 motor, u8 type, int32 *value)
{
	u32 errors = ERROR_NONE;
	uint32 uvalue;

	if(motor >= MOTORS)
		return ERROR_MOTOR;

	// \xml <axis_parameters axis="0">
	switch(type)
	{
	case 0:
		// \xml <tmc:axis_parameter type="0" name="Target position" identifier="par::TargetPosition">
		// \xml   <desc>The desired target position in position mode.</desc>
		// \xml   <tmc:value_signed min="#s32_MIN#" max="#s32_MAX#" default="0" unit="[usteps]" access="RW"></tmc:value_signed>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			*value = tmc4361A_readInt(TMC4361A_X_TARGET);
		} else if(readWrite == WRITE) {
			tmc4361A_writeInt(TMC4361A_X_TARGET, *value);
		}
		break;
	case 1:
		// \xml <tmc:axis_parameter type="1" name="Actual position" identifier="par::ActualPosition">
		// \xml   <desc>The actual position of the motor. Stop the motor before overwriting it. Should normally only be overwritten for reference position setting.</desc>
		// \xml   <tmc:value_signed min="#s32_MIN#" max="#s32_MAX#" default="0" unit="[usteps]" access="RW"></tmc:value_signed>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			*value = tmc4361A_readInt(TMC4361A_XACTUAL);
		} else if(readWrite == WRITE) {
			tmc4361A_writeInt(TMC4361A_XACTUAL, *value);
		}
		break;
	case 2:
		// \xml <tmc:axis_parameter type="2" name="Target speed" identifier="par::TargetVelocity">
		// \xml   <desc>The desired speed in velocity mode. Not valid in position mode.</desc>
		// \xml   <tmc:value_signed min="#s32_MIN#" max="#s32_MAX#" default="0" unit="[pps]" access="RW"></tmc:value_signed>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			*value = tmc4361A_readInt(TMC4361A_VMAX) >> 8;
		} else if(readWrite == WRITE) {
			tmc4361A_writeInt(TMC4361A_VMAX, discardVelocityDecimals(*value));
		}
		break;
	case 3:
		// \xml <tmc:axis_parameter type="3" name="Actual speed" identifier="par::ActualVelocity">
		// \xml   <desc>The actual speed of the motor.</desc>
		// \xml   <tmc:value_signed min="#s32_MIN#" max="#s32_MAX#" default="0" unit="[pps]" access="R"></tmc:value_signed>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			*value = tmc4361A_readInt(TMC4361A_VACTUAL);
		} else if(readWrite == WRITE) {
			errors |= ERROR_TYPE;
		}
		break;
	case 4:
		// \xml <tmc:axis_parameter type="4" name="Maximum speed" identifier="par::MaxVelocity">
		// \xml   <desc>The maximum speed used for positioning ramps.</desc>
		// \xml   <tmc:value_signed min="#s32_MIN#" max="#s32_MAX#" default="150000" unit="[pps]" access="RW"></tmc:value_signed>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			*value = tmc4361A_readInt(TMC4361A_VMAX) >> 8;
		} else if(readWrite == WRITE) {
			tmc4361A_writeInt(TMC4361A_VMAX, discardVelocityDecimals(*value));
		}
		break;
	case 5:
		// \xml <tmc:axis_parameter type="5" name="Maximum acceleration" identifier="par::MaxAcceleration">
		// \xml   <desc>Maximum acceleration in positioning ramps. Acceleration and deceleration value in velocity mode.</desc>
		// \xml   <tmc:value_unsigned min="0" max="#TMC4361_MAX_ACCELERATION#" default="1000" unit="[pps/s]" access="RW"></tmc:value_unsigned>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			*value = tmc4361A_readInt(TMC4361A_AMAX)>>2;
		} else if(readWrite == WRITE) {
			if(*value & ~0x3FFFFF)
			{
				errors |= ERROR_VALUE;
			}
			else
			{
				tmc4361A_writeInt(TMC4361A_AMAX, *value<<2);
			}
		}
		break;
	case 8:
		// \xml <tmc:axis_parameter type="8" name="Position reached flag" identifier="par::PositionReachedFlag">
		// \xml   <desc>This flag is always set when target position and actual position are equal.</desc>
		// \xml   <tmc:value_bool desc_false="Target position not reached" desc_true="Target position reached" default="false" access="R"></tmc:value_bool>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			*value = (tmc4361A_readInt(TMC4361A_STATUS) & (1<<0))? 1:0;
		} else if(readWrite == WRITE)
			errors |= ERROR_TYPE;
		break;
	case 14:
		// \xml <tmc:axis_parameter type="14" name="Ramp type" identifier="par::RampType">
		// \xml   <desc>Actual type of ramp.</desc>
		// \xml   <tmc:value_bool desc_false="Trapezoidal ramp" desc_true="S-shaped ramp" default="false" access="RW"></tmc:value_bool>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			*value = tmc4361A_readInt(TMC4361A_RAMPMODE)<<1;
		} else if(readWrite == WRITE) {
			tmc4361A_writeInt(TMC4361A_RAMPMODE, (*value) ? TMC4361A_RAMP_SSHAPE : TMC4361A_RAMP_TRAPEZ);
		}
		break;
	case 15:
		// \xml <tmc:axis_parameter type="15" name="Velocity VSTART" identifier="par::StartVelocity">
		// \xml   <desc>Motor start velocity (in position mode only). Do not set VSTART higher than VSTOP.</desc>
		// \xml   <tmc:value_unsigned min="0" max="#TMC4361A_MAX_VELOCITY#" default="1" unit="[pps]" access="RW"></tmc:value_unsigned>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			*value = tmc4361A_readInt(TMC4361A_VSTART);
		} else if(readWrite == WRITE) {
			tmc4361A_writeInt(TMC4361A_VSTART, *value);
		}
		break;
	case 16:
		// \xml <tmc:axis_parameter type="16" name="Acceleration AStart" identifier="par::AStart">
		// \xml   <desc>First acceleration between VSTART and V1 (in position mode only).</desc>
		// \xml   <tmc:value_unsigned min="0" max="#TMC4361_MAX_ACCELERATION#" default="100" unit="[pps/s]" access="RW"></tmc:value_unsigned>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			*value = tmc4361A_readInt(TMC4361A_ASTART)>>2;
		} else if(readWrite == WRITE) {
			if(*value & ~0x3FFFFF)
			{
				errors |= ERROR_VALUE;
			}
			else
			{
				tmc4361A_writeInt(TMC4361A_ASTART, *value<<2);
			}
		}
		break;
	case 17:
		// \xml <tmc:axis_parameter type="17" name="Maximum Deceleration" identifier="par::MaxDeceleration">
		// \xml   <desc>Maximum deceleration in positioning ramps. Used to decelerate from maximum positioning speed (axis parameter 4) to velocity V1</desc>
		// \xml   <tmc:value_unsigned min="0" max="#TMC4361_MAX_ACCELERATION#" default="5" unit="[pps/s]" access="RW"></tmc:value_unsigned>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			*value = tmc4361A_readInt(TMC4361A_DMAX)>>2;
		} else if(readWrite == WRITE) {
			if(*value & ~0x3FFFFF)
			{
				errors |= ERROR_VALUE;
			}
			else
			{
				tmc4361A_writeInt(TMC4361A_DMAX, *value<<2);
			}
		}
		break;
	case 18:
		// \xml <tmc:axis_parameter type="18" name="Velocity VBreak" identifier="par::VBreak">
		// \xml   <desc>First acceleration / deceleration phase target velocity (in position mode only). Setting this value to 0 turns off the first acceleration / deceleration phase, maximum acceleration (axis parameter 5) and maximum deceleration (axis parameter 17) are used only.</desc>
		// \xml   <tmc:value_unsigned min="0" max="#TMC4361A_MAX_VELOCITY#" default="2" unit="[pps]" access="RW"></tmc:value_unsigned>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			*value = tmc4361A_readInt(TMC4361A_VBREAK);
		} else if(readWrite == WRITE) {
			tmc4361A_writeInt(TMC4361A_VBREAK, *value);
		}
		break;
	case 19:
		// \xml <tmc:axis_parameter type="19" name="Deceleration DFinal" identifier="par::DFinal">
		// \xml   <desc>Deceleration between V1 and VSTOP (in positioning mode only).</desc>
		// \xml   <tmc:value_unsigned min="0" max="#TMC4361_MAX_ACCELERATION#" default="200" unit="[pps/s]" access="RW"></tmc:value_unsigned>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			*value = tmc4361A_readInt(TMC4361A_DFINAL) >> 2;
		} else if(readWrite == WRITE) {
			if(*value & ~0x3FFFFF)
			{
				errors |= ERROR_VALUE;
			}
			else
			{
				tmc4361A_writeInt(TMC4361A_DFINAL, *value<<2);
			}
		}
		break;
	case 20:
		// \xml <tmc:axis_parameter type="20" name="Velocity VSTOP" identifier="par::StopVelocity">
		// \xml   <desc>Motor stop velocity (in position mode only).</desc>
		// \xml   <tmc:value_unsigned min="0" max="#TMC4361A_MAX_VELOCITY#" default="2" unit="[pps]" access="RW"></tmc:value_unsigned>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			*value = tmc4361A_readInt(TMC4361A_VSTOP);
		} else if(readWrite == WRITE) {
			tmc4361A_writeInt(TMC4361A_VSTOP, *value);
		}
		break;
	case 21:
		// \xml <tmc:axis_parameter type="21" name="Deceleration DStop" identifier="par::DSTOP">
		// \xml   <desc>Deceleration between V1 and VSTOP (in positioning mode only).</desc>
		// \xml   <tmc:value_unsigned min="0" max="#TMC4361_MAX_ACCELERATION#" default="200" unit="[pps/s]" access="RW"></tmc:value_unsigned>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			*value = tmc4361A_readInt(TMC4361A_DSTOP);
		} else if(readWrite == WRITE) {
			if(*value & ~0x3FFFFF)
			{
				errors |= ERROR_VALUE;
			}
			else
			{
				tmc4361A_writeInt(TMC4361A_DSTOP, *value);
			}
		}
		break;
	case 22:
		// \xml <tmc:axis_parameter type="22" name="Bow 1" identifier="par::BOW1">
		// \xml   <desc>First bow value of a complete velocity ramp.</desc>
		// \xml   <tmc:value_unsigned min="0" max="#TMC4361_MAX_ACCELERATION#" default="200" unit="[pps/s]" access="RW"></tmc:value_unsigned>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			*value = tmc4361A_readInt(TMC4361A_BOW1);
		} else if(readWrite == WRITE) {
			tmc4361A_writeInt(TMC4361A_BOW1, *value);
		}
		break;
	case 23:
		// \xml <tmc:axis_parameter type="23" name="Bow 2" identifier="par::BOW2">
		// \xml   <desc>Second bow value of a complete velocity ramp.</desc>
		// \xml   <tmc:value_unsigned min="0" max="#TMC4361_MAX_ACCELERATION#" default="200" unit="[pps/s]" access="RW"></tmc:value_unsigned>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			*value = tmc4361A_readInt(TMC4361A_BOW2);
		} else if(readWrite == WRITE) {
			tmc4361A_writeInt(TMC4361A_BOW2, *value);
		}
		break;
	case 24:
		// \xml <tmc:axis_parameter type="24" name="Bow 3" identifier="par::BOW3">
		// \xml   <desc>Third bow value of a complete velocity ramp.</desc>
		// \xml   <tmc:value_unsigned min="0" max="#TMC4361_MAX_ACCELERATION#" default="200" unit="[pps/s]" access="RW"></tmc:value_unsigned>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			*value = tmc4361A_readInt(TMC4361A_BOW3);
		} else if(readWrite == WRITE) {
			tmc4361A_writeInt(TMC4361A_BOW3, *value);
		}
		break;
	case 25:
		// \xml <tmc:axis_parameter type="25" name="Bow 4" identifier="par::BOW4">
		// \xml   <desc>Fourth bow value of a complete velocity ramp.</desc>
		// \xml   <tmc:value_unsigned min="0" max="#TMC4361_MAX_ACCELERATION#" default="200" unit="[pps/s]" access="RW"></tmc:value_unsigned>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			*value = tmc4361A_readInt(TMC4361A_BOW4);
		} else if(readWrite == WRITE) {
			tmc4361A_writeInt(TMC4361A_BOW4, *value);
		}
		break;
	case 26:
		// \xml <tmc:axis_parameter type="26" name="Virtual stop left" identifier="par::VirtualStopLeft">
		// \xml   <desc>Virtual left stop that triggers a stop event when XACTUAL is below this parameter.</desc>
		// \xml   <tmc:value_signed min="#s32_MIN#" max="#s32_MAX#" default="0" unit="[usteps]" access="RW"></tmc:value_signed>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			*value = tmc4361A_readInt(TMC4361A_VIRT_STOP_LEFT);
		} else if(readWrite == WRITE) {
			tmc4361A_writeInt(TMC4361A_VIRT_STOP_LEFT, *value);
		}
		break;
	case 27:
		// \xml <tmc:axis_parameter type="27" name="Virtual stop right" identifier="par::VirtualStopRight">
		// \xml   <desc>Virtual right stop that triggers a stop event when XACTUAL is over this parameter.</desc>
		// \xml   <tmc:value_signed min="#s32_MIN#" max="#s32_MAX#" default="0" unit="[usteps]" access="RW"></tmc:value_signed>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			*value = tmc4361A_readInt(TMC4361A_VIRT_STOP_RIGHT);
		} else if(readWrite == WRITE) {
			tmc4361A_writeInt(TMC4361A_VIRT_STOP_RIGHT, *value);
		}
		break;
	case 108:
		// \xml <tmc:axis_parameter type="108" name="CL Gamma VMin" identifier="par::CLGammaVMin">
		// \xml   <desc>At this velocity the gamma compensation will start.</desc>
		// \xml   <tmc:value_unsigned min="0" max="16777215" default="1" unit="[pps]" access="RW"></tmc:value_unsigned>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			*value = tmc4361A_readInt(TMC4361A_CL_VMIN_EMF_WR);		// read from shadow register
		} else if(readWrite == WRITE) {
			tmc4361A_writeInt(TMC4361A_CL_VMIN_EMF_WR, *value);
		}
		break;
	case 109:
		// \xml <tmc:axis_parameter type="109" name="CL Gamma VMax" identifier="par::CLGammaVMax">
		// \xml   <desc>For higher velocities the influence of the back EMF may be compensated. At the start velocity (CL Gamma VMin) plus this velocity the compensation will reach its maximum.</desc>
		// \xml   <tmc:value_unsigned min="0" max="16777215" default="1" unit="[pps]" access="RW"></tmc:value_unsigned>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			*value = tmc4361A_readInt(TMC4361A_CL_VADD_EMF); 	// read from shadow register
		} else if(readWrite == WRITE) {
			tmc4361A_writeInt(TMC4361A_CL_VADD_EMF, *value);
		}
		break;
	case 110:
		// \xml <tmc:axis_parameter type="110" name="CL maximum Gamma" identifier="par::CLMaxGamma">
		// \xml   <desc>For higher velocities the influence of the back EMF may be compensated. This is the compensation factor that will be added scaled using the current velocity.</desc>
		// \xml   <tmc:value_unsigned min="0" max="255" default="0" unit="" access="RW"></tmc:value_unsigned>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			*value = tmc4361A_readInt(TMC4361A_CL_BETA) >> 16;
		} else if(readWrite == WRITE) {
			uvalue = tmc4361A_readInt( TMC4361A_CL_BETA) & 0x000001FF;
			tmc4361A_writeInt(TMC4361A_CL_BETA, uvalue | (*value<<16));
		}
		break;
	case 111:
		// \xml <tmc:axis_parameter type="111" name="CL beta" identifier="par::CLBeta">
		// \xml   <desc>Maximum commutation angle that can be used to compensate for an evaluated position deviation.</desc>
		// \xml   <tmc:value_unsigned min="0" max="511" default="0" unit="" access="RW"></tmc:value_unsigned>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			*value = tmc4361A_readInt(TMC4361A_CL_BETA) & 0xFF;
		} else if(readWrite == WRITE) {
			uvalue = tmc4361A_readInt( TMC4361A_CL_BETA) & 0x00FF0000;
			tmc4361A_writeInt(TMC4361A_CL_BETA, uvalue | (*value & 0x1FF));
		}
		break;
	case 112:
		// \xml <tmc:axis_parameter type="112" name="CL offset" identifier="par::CLOffset">
		// \xml   <desc>Offset for closed loop operation. Measured during closed loop initialization.</desc>
		// \xml   <tmc:value_signed min="#s32_MIN#" max="#s32_MAX#" default="0" unit="" access="RW"></tmc:value_signed>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			*value = tmc4361A_readInt(TMC4361A_CL_OFFSET);
		} else if(readWrite == WRITE) {
			tmc4361A_writeInt(TMC4361A_CL_OFFSET, *value);
		}
		break;
	case 113:
		// \xml <tmc:axis_parameter type="113" name="CL current minimum" identifier="par::CLCurrentMin">
		// \xml   <desc>Minimum current setting in closed loop operation.</desc>
		// \xml   <tmc:value_unsigned min="0" max="255" default="0" unit="" access="RW"></tmc:value_unsigned>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			*value = (tmc4361A_readInt(TMC4361A_SCALE_VALUES) >> 0) & 0xFF;
		} else if(readWrite == WRITE) {
			uvalue = tmc4361A_readInt(TMC4361A_SCALE_VALUES) & ~(0xFF<<0);
			uvalue |= (*value & 0xFF) << 0;
			tmc4361A_writeInt(TMC4361A_SCALE_VALUES, uvalue);
		}
		break;
	case 114:
		// \xml <tmc:axis_parameter type="114" name="CL current maximum" identifier="par::CLCurrentMax">
		// \xml   <desc>Maximum current setting in closed loop operation.</desc>
		// \xml   <tmc:value_unsigned min="0" max="255" default="0" unit="" access="RW"></tmc:value_unsigned>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			*value = (tmc4361A_readInt(TMC4361A_SCALE_VALUES) >> 8) & 0xFF;
		} else if(readWrite == WRITE) {
			uvalue = tmc4361A_readInt(TMC4361A_SCALE_VALUES) & ~(0xFF<<8);
			uvalue |= (*value & 0xFF) << 8;
			tmc4361A_writeInt(TMC4361A_SCALE_VALUES, uvalue);
		}
		break;
	case 115:
		// \xml <tmc:axis_parameter type="115" name="CL correction velocity P" identifier="par::CLCorrectionVelocityP">
		// \xml   <desc>Parameter P of the PI regulator which controls the maximum velocity during closed loop regulation.</desc>
		// \xml   <tmc:value_unsigned min="0" max="16777215" default="0" unit="" access="RW"></tmc:value_unsigned>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			*value = TMC4361A_config->shadowRegister[TMC4361A_CL_VMAX_CALC_P_WR];
		} else if(readWrite == WRITE) {
			tmc4361A_writeInt(TMC4361A_CL_VMAX_CALC_P_WR, *value);
		}
		break;
	case 116:
		// \xml <tmc:axis_parameter type="116" name="CL correction velocity I" identifier="par::CLCorrectionVelocityI">
		// \xml   <desc>Parameter I of the PI regulator which controls the maximum velocity during closed loop regulation.</desc>
		// \xml   <tmc:value_unsigned min="0" max="16777215" default="0" unit="" access="RW"></tmc:value_unsigned>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			*value = TMC4361A_config->shadowRegister[TMC4361A_CL_VMAX_CALC_I_WR];
		} else if(readWrite == WRITE) {
			tmc4361A_writeInt(TMC4361A_CL_VMAX_CALC_I_WR, *value);
		}
		break;
	case 117:
		// \xml <tmc:axis_parameter type="117" name="CL correction velocity I clipping" identifier="par::CLCorrectionVelIClip">
		// \xml   <desc></desc>
		// \xml   <tmc:value_unsigned min="0" max="1" default="0" unit="" access="RW"></tmc:value_unsigned>
		// \xml </tmc:axis_parameter>
		// todo AP 3: same register as in AP 116 and 115? (BS) #1
		// (JE) : probably register TMC4361_PID_I_CLIP_WR ?
		if(readWrite == READ) {
			*value = TMC4361A_config->shadowRegister[TMC4361A_PID_I_WR] >> 0;
			*value &= 0x7FFF;
		} else if(readWrite == WRITE) {
			uvalue = TMC4361A_config->shadowRegister[TMC4361A_PID_I_WR];
			uvalue &= ~(0x7FFF << 0);
			uvalue |= (*value & 0x7FFF) << 0;
			tmc4361A_writeInt(TMC4361A_PID_I_WR, uvalue);
		}
		break;
	case 118:
		// \xml <tmc:axis_parameter type="118" name="CL correction velocity DV clock" identifier="par::CL_CorrectionVelDVClock">
		// \xml   <desc></desc>
		// \xml   <tmc:value_unsigned min="0" max="1" default="0" unit="" access="RW"></tmc:value_unsigned>
		// \xml </tmc:axis_parameter>
		// todo AP 3: same register as in AP 116 and 115? (BS) #2
		// (JE) probably register TMC4361_PID_I_CLIP_WR?
		if(readWrite == READ) {
			*value = TMC4361A_config->shadowRegister[TMC4361A_PID_I_WR] >> 16;
			*value &= 0xFF;
		} else if(readWrite == WRITE) {
			uvalue = TMC4361A_config->shadowRegister[TMC4361A_PID_I_WR];
			uvalue &= ~(0xFF << 16);
			uvalue |= (*value & 0xFF) << 16;
			tmc4361A_writeInt(TMC4361A_PID_I_WR, uvalue);
		}
		break;
	case 119:
		// \xml <tmc:axis_parameter type="119" name="CL correction velocity DV clipping" identifier="par::CL_Correction_VelDVClip">
		// \xml   <desc>Limit maximum velocity deviation above the maximum velocity.</desc>
		// \xml   <tmc:value_unsigned min="0" max="#u32_MAX#" default="0" unit="" access="RW"></tmc:value_unsigned>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			*value = tmc4361A_readInt(TMC4361A_PID_DV_CLIP_WR);
		} else if(readWrite == WRITE) {
			tmc4361A_writeInt(TMC4361A_PID_DV_CLIP_WR, *value);
		}
		break;
	case 120:
		// \xml <tmc:axis_parameter type="120" name="CL upscale delay" identifier="par::CLUpscaleDelay">
		// \xml   <desc>Delay when increasing motor current during closed loop operation.</desc>
		// \xml   <tmc:value_unsigned min="0" max="16777215" default="0" unit="" access="RW"></tmc:value_unsigned>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			*value = tmc4361A_readInt(TMC4361A_CL_UPSCALE_DELAY);
		} else if(readWrite == WRITE) {
			tmc4361A_writeInt(TMC4361A_CL_UPSCALE_DELAY, *value);
		}
		break;
	case 121:
		// \xml <tmc:axis_parameter type="121" name="CL Downscale delay" identifier="par::CLDownscaleDelay">
		// \xml   <desc>Delay when decreasing motor current during closed loop operation.</desc>
		// \xml   <tmc:value_unsigned min="0" max="16777215" default="0" unit="" access="RW"></tmc:value_unsigned>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			*value = tmc4361A_readInt(TMC4361A_CL_DOWNSCALE_DELAY);
		} else if(readWrite == WRITE) {
			tmc4361A_writeInt(TMC4361A_CL_DOWNSCALE_DELAY, *value);
		}
		break;
	case 124:
		// \xml <tmc:axis_parameter type="124" name="CL Correction Position P" identifier="par::CLCorrectionPositionP">
		// \xml   <desc>Proportional controller P parameter for compensating a detected position deviation. Resulting P parameter is this parameter divided by 65536.</desc>
		// \xml   <tmc:value_unsigned min="0" max="16777215" default="0" unit="" access="RW"></tmc:value_unsigned>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			*value = tmc4361A_readInt(TMC4361A_CL_DELTA_P_WR);
		} else if(readWrite == WRITE) {
			tmc4361A_writeInt(TMC4361A_CL_DELTA_P_WR, *value);
		}
		break;
	case 125:
		// \xml <tmc:axis_parameter type="125" name="CL max. correction tolerance" identifier="par::CLMaxCorrectionTolerance">
		// \xml   <desc>Tolerance range for position deviation.</desc>
		// \xml   <tmc:value_unsigned min="0" max="255" default="0" unit="" access="RW"></tmc:value_unsigned>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			*value = tmc4361A_readInt(TMC4361A_CL_TOLERANCE_WR);
		} else if(readWrite == WRITE) {
			tmc4361A_writeInt(TMC4361A_CL_TOLERANCE_WR, *value);
		}
		break;
	case 126:
		// \xml <tmc:axis_parameter type="126" name="CL start up" identifier="par::CLStartUp">
		// \xml   <desc>Tolerance range for position deviation. If position deviation is within the CL start up window the current will not ramp up.</desc>
		// \xml   <tmc:value_unsigned min="0" max="255" default="0" unit="" access="RW"></tmc:value_unsigned>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			*value = (tmc4361A_readInt(TMC4361A_SCALE_VALUES) >> 16) & 0xFF;
		} else if(readWrite == WRITE) {
			uvalue = tmc4361A_readInt(TMC4361A_SCALE_VALUES) & ~(0xFF<<16);
			uvalue |= (*value & 0xFF) << 16;
			tmc4361A_writeInt(TMC4361A_SCALE_VALUES, uvalue);
		}
		break;
	case 129: // todo AP 2: merge AP 129 with AP 133? #1
		// \xml <tmc:axis_parameter type="129" name="Closed Loop Flag" identifier="par::CLFlag">
		// \xml   <desc>Closed loop enabled flag. After switching on closed loop, please wait until CL Init Flag is 1 before executing any motion commands.</desc>
		// \xml   <tmc:value_bool desc_false="Closed Loop Off" desc_true="Closed Loop On" default="false" access="RW"></tmc:value_bool>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			// Read for closed loop flag is implemented as AP 133
		} else if(readWrite == WRITE) {
			//Closed loop on/off
			if(*value)
			{
				*value = tmc4361A_calibrateClosedLoop(1);
				if(!*value)
					errors |= ERROR_NOT_DONE;
			}
			else
			{
				uvalue 	= tmc4361A_readInt(TMC4361A_ENC_IN_CONF);
				uvalue 	&= ~(1<<22); // closed loop
				tmc4361A_writeInt(TMC4361A_ENC_IN_CONF, uvalue);
			}
		}
		break;
	case 132:
		// \xml <tmc:axis_parameter type="132" name="Measured Encoder Speed" identifier="par::MeasuredEncoderSpeed">
		// \xml   <desc>Current speed value measured from encoder.</desc>
		// \xml   <tmc:value_signed min="#s32_MIN#" max="#s32_MAX#" default="0" unit="" access="R"></tmc:value_signed>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			*value = tmc4361A_readInt(TMC4361A_V_ENC_RD);
		} else if(readWrite == WRITE)
			errors |= ERROR_TYPE;
		break;
	case 133: // todo AP 2: merge AP 129 with AP 133? #2
		// \xml <tmc:axis_parameter type="133" name="Closed Loop Init Flag" identifier="par::CLInitFlag">
		// \xml   <desc>Closed Loop initialization flag.</desc>
		// \xml   <tmc:value_bool desc_false="Initialization not finished yet" desc_true="Closed loop initialization finished" default="false" access="RW"></tmc:value_bool>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			uvalue 	= tmc4361A_readInt(TMC4361A_ENC_IN_CONF);
			*value = (((uvalue >> 22) & 3) == 1) ? 1 : 0;
		} else if(readWrite == WRITE) {
			// Write for closed loop flag is implemented as AP 129
		}
		break;
	case 134:
		// todo AP XML 2: Split this into read-only Encoder deviation and write-only CL_TR_TOLERANCE? Having this as one axis parameter is not that intuitive (LH) #1
		// \xml <tmc:axis_parameter type="134" name="Encoder deviation" identifier="par::EncoderDeviation">
		// \xml   <desc>Deviation between the actual position (parameter 1) and the encoder position (parameter 209). A.k.a.: Positioning window (Maximum deviation for target reached flag to be set).</desc>
		// \xml   <tmc:value_unsigned min="0" max="#u32_MAX#" default="0" unit="[steps]" access="R"></tmc:value_unsigned>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			*value = tmc4361A_readInt(TMC4361A_ENC_POS_DEV_RD);
		} else if(readWrite == WRITE) {
			tmc4361A_writeInt(TMC4361A_CL_TR_TOLERANCE_WR, *value);
		}
		break;
	case 136:
		// \xml <tmc:axis_parameter type="136" name="Encoder Velocity Delay" identifier="par::EncVelDelay">
		// \xml   <desc>Delay period before the next current encoder velocity value becomes considered for mean encoder velocity calculation.</desc>
		// \xml   <tmc:value_unsigned min="0" max="255" default="0" unit="[clock cycles]" access="RW"></tmc:value_unsigned>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			*value = (TMC4361A_config->shadowRegister[TMC4361A_ENC_VMEAN_WAIT_WR] >> 0) &  0xFF;
		} else if(readWrite == WRITE) {
			uvalue = TMC4361A_config->shadowRegister[TMC4361A_ENC_VMEAN_WAIT_WR];
			uvalue &= ~(0xFF << 0);
			uvalue |= (*value & 0x0F) << 0;
			tmc4361A_writeInt(TMC4361A_ENC_VMEAN_WAIT_WR, uvalue);
		}
		break;
	case 137:
		// \xml <tmc:axis_parameter type="137" name="Encoder Velocity Filter" identifier="par::EncVelFilter">
		// \xml   <desc>Filter exponent to calculate mean encoder velocity.</desc>
		// \xml   <tmc:value_unsigned min="0" max="16" default="0" unit="" access="RW"></tmc:value_unsigned>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			*value = (TMC4361A_config->shadowRegister[TMC4361A_ENC_VMEAN_WAIT_WR] >> 8) &  0xF;
		} else if(readWrite == WRITE) {
			uvalue = TMC4361A_config->shadowRegister[TMC4361A_ENC_VMEAN_WAIT_WR];
			uvalue &= ~(0xF << 8);
			uvalue |= (*value & 0x0F) << 8;
			tmc4361A_writeInt(TMC4361A_ENC_VMEAN_WAIT_WR, uvalue);
		}
		break;
	case 138:
		// \xml <tmc:axis_parameter type="138" name="Filter Update Time" identifier="par::FilterUpdateTime">
		// \xml   <desc>Filter update time in clock cycles.</desc>
		// \xml   <tmc:value_unsigned min="0" max="#u16_MAX#" default="0" unit="[clock cycles]" access="RW"></tmc:value_unsigned>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			*value = (TMC4361A_config->shadowRegister[TMC4361A_ENC_VMEAN_WAIT_WR] >> 16) &  0xFF;
		} else if(readWrite == WRITE) {
			uvalue = TMC4361A_config->shadowRegister[TMC4361A_ENC_VMEAN_WAIT_WR];
			uvalue &= ~(0xFF << 16);
			uvalue |= (*value & 0x0FF) << 16;
			tmc4361A_writeInt(TMC4361A_ENC_VMEAN_WAIT_WR, uvalue);
		}
		break;
	case 200:
		// \xml <tmc:axis_parameter type="200" name="Boost current" identifier="par::BoostCurrent">
		// \xml   <desc>Current used for acceleration and deceleration phases. If set to 0 the same current as set by axis parameter 6 will be used. Same scaling as with axis parameter 6.</desc>
		// \xml   <tmc:value_unsigned min="0" max="255" default="0" unit="" access="RW"></tmc:value_unsigned>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			*value = (tmc4361A_readInt(TMC4361A_SCALE_VALUES) >> 0) & 0xFF;
		} else if(readWrite == WRITE) {
			uvalue = tmc4361A_readInt(TMC4361A_SCALE_VALUES) & ~(0xFF<<0);
			uvalue |= (*value & 0xFF) << 0;
			tmc4361A_writeInt(TMC4361A_SCALE_VALUES, uvalue);
		}
		break;
	case 209:
		// \xml <tmc:axis_parameter type="209" name="Encoder position" identifier="par::EncoderPosition">
		// \xml   <desc>The current encoder position</desc>
		// \xml   <tmc:value_signed min="#s32_MIN#" max="#s32_MAX#" default="0" unit="[usteps]" access="RW"></tmc:value_signed>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			*value = tmc4361A_readInt(TMC4361A_ENC_POS);
		} else if(readWrite == WRITE) {
			tmc4361A_writeInt(TMC4361A_ENC_POS, *value);
		}
		break;
	case 212:
		// \xml <tmc:axis_parameter type="212" name="Maximum encoder deviation" identifier="par::MaxEncoderDeviation">
		// \xml   <desc>When the actual position (parameter 1) and the encoder position (parameter 209) differ more than set here the motor will be stopped. This function is switched off when the maximum deviation is set to zero.</desc>
		// \xml   <tmc:value_unsigned min="0" max="#u32_MAX#" default="0" unit="[encoder steps]" access="RW"></tmc:value_unsigned>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			*value = TMC4361A_config->shadowRegister[TMC4361A_SCALE_VALUES]; // todo CHECK 3: shouldn't this register be TMC4361A_ENC_POS_DEV_TOL_WR like below? (BS) #1
		} else if(readWrite == WRITE) {
			tmc4361A_writeInt(TMC4361A_ENC_POS_DEV_TOL_WR, *value);
		}
		break;
	case 214:
		// \xml <tmc:axis_parameter type="214" name="Power Down Delay" identifier="par::PowerDownDelay">
		// \xml   <desc>Standstill period before the current will be ramped down to standby current. The standard value is 200 (which means 2000ms).</desc>
		// \xml   <tmc:value_signed min="0" max="#u16_MAX#" default="0" unit="[10 ms]" access="RW"></tmc:value_signed>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			*value = tmc4361A_readInt(TMC4361A_STDBY_DELAY);
		} else if(readWrite == WRITE) {
			tmc4361A_writeInt(TMC4361A_STDBY_DELAY, *value*160000);
		}
		break;
	default:
		errors |= ERROR_TYPE;
		break;
	}
	// \xml </axis_parameters>
	return errors;
}
// \xml </tmc:product_module>

static uint32 SAP(uint8 type, uint8 motor, int32 value)
{
	return handleParameter(WRITE, motor, type, &value);
}

static uint32 GAP(uint8 type, uint8 motor, int32 *value)
{
	return handleParameter(READ, motor, type, value);
}

static void writeRegister(uint8 address, int32 value)
{
	tmc4361A_writeInt(address, value);
}

static void readRegister(uint8 address, int32 *value)
{
	*value	= tmc4361A_readInt(address);
}

static void periodicJob(uint32 tick)
{
	ConfigState lastState = TMC4361A_config->state;
	uint8_t driver, dataLength;
	uint32_t value;

	tmc4361A_periodicJob(tick, &TMC4361A, TMC4361A_config);

	// Config reset/restore complete
	if(lastState != CONFIG_READY && TMC4361A_config->state == CONFIG_READY)
	{
		// Setup SPI
		switch(Evalboards.ch2.id)
		{
		case ID_TMC2130:
			driver = 0b1101;
			dataLength = 0;
			break;
		case ID_TMC2660:
			driver = 0b1011;
			dataLength = 0;
			break;
		default:
			driver = 0b1111;
			dataLength = 40;
			break;
		}
		value = 0x44400040 | (dataLength << 13) | (driver << 0);
		tmc4361A_writeInt(TMC4361A_SPIOUT_CONF, value);

		// Reset/Restore driver
		if(lastState == CONFIG_RESET)
			Evalboards.ch2.config->reset();
		else
			Evalboards.ch2.config->restore();
	}
}

static void checkErrors(uint32 tick)
{
	UNUSED(tick);
	Evalboards.ch1.errors = 0;
}

static uint32 userFunction(uint8 type, uint8 motor, int32 *value)
{
	uint32 errors = 0;
	UNUSED(motor);

	switch(type)
	{
	case 0:	// simulate left/right reference switches, set high to support external ref swiches
		/*
		 * The the TMC4361 ref switch input is pulled high by external resistor an can be pulled low either by
		 * this µC or external signal. To use external signal make sure the signals from µC are high or floating.
		 */
		if(!(*value & ~3))
		{
			if(*value & (1<<0))
			{
				HAL.IOs->config->toInput(Pins.STOP_R); // pull up -> set it to floating causes high
			}
			else
			{
				HAL.IOs->config->toOutput(Pins.STOP_R);
				HAL.IOs->config->setLow(Pins.STOP_R);
			}

			if(*value & (1<<1))
			{
				HAL.IOs->config->toInput(Pins.STOP_L); // pull up -> set it to floating causes high
			}
			else
			{
				HAL.IOs->config->toOutput(Pins.STOP_L);
				HAL.IOs->config->setLow(Pins.STOP_L);
			}
		}
		//else TMCL.reply->Status = REPLY_INVALID_VALUE;
		break;
	case 1:	// simulate reference switche HOME_REF, set high to support external ref swiches
		/*
		 * The the TMC43x1 ref switch input is pulled high by external resistor an can be pulled low either by
		 * this µC or external signal. To use external signal make sure the signals from µC are high or floating.
		 */
		if(*value)
		{
			HAL.IOs->config->toInput(Pins.HOME_REF); // pull up -> set it to floating causes high
		}
		else
		{
			HAL.IOs->config->toOutput(Pins.HOME_REF);
			HAL.IOs->config->setLow(Pins.HOME_REF);
		}
		break;
	case 2:	// simulate reference switche FREEZE, set high to support external ref swiches
		/*
		 * The the TMC43x1 ref switch input is pulled high by external resistor an can be pulled low either by
		 * this µC or external signal. To use external signal make sure the signals from µC are high or floating.
		 */

		if(*value)
		{
			HAL.IOs->config->toInput(Pins.FREEZE); // pull up -> set it to floating causes high
		}
		else
		{
			HAL.IOs->config->toOutput(Pins.FREEZE);
			HAL.IOs->config->setLow(Pins.FREEZE);
		}
		break;
	case 3:
		*value = tmc4361A_calibrateClosedLoop(1);
		if(!*value)
			errors |= ERROR_NOT_DONE;
		break;
	case 255:
		Evalboards.ch2.config->reset();
		Evalboards.ch1.config->reset();
		break;
	default:
		errors |= ERROR_TYPE;
		break;
	}
	return errors;
}

static void deInit(void)
{
	HAL.IOs->config->setLow(Pins.NRST);

	HAL.IOs->config->reset(Pins.STOP_L);
	HAL.IOs->config->reset(Pins.STOP_R);
	HAL.IOs->config->reset(Pins.HOME_REF);
	HAL.IOs->config->reset(Pins.START);
	HAL.IOs->config->reset(Pins.FREEZE);
	HAL.IOs->config->reset(Pins.STANDBY_CLK);
	HAL.IOs->config->reset(Pins.INTR);
	HAL.IOs->config->reset(Pins.TARGET_REACHED);
	HAL.IOs->config->reset(Pins.NRST);

	HAL.SPI->ch2.reset();
}

static uint8 reset()
{
	// Pulse the low-active hardware reset pin
	HAL.IOs->config->setLow(Pins.NRST);
	wait(1);
	HAL.IOs->config->setHigh(Pins.NRST);

	tmc4361A_reset(TMC4361A_config);

	return 1;
}

static uint8 restore()
{
	// Pulse the low-active hardware reset pin
	HAL.IOs->config->setLow(Pins.NRST);
	wait(1);
	HAL.IOs->config->setHigh(Pins.NRST);

	tmc4361A_restore(TMC4361A_config);

	return 1;
}

void TMC4361A_init(void)
{
	tmc4361A_initConfig(&TMC4361A);

	Pins.STANDBY_CLK     = &HAL.IOs->pins->DIO4;
	Pins.INTR            = &HAL.IOs->pins->DIO5;
	Pins.STOP_L          = &HAL.IOs->pins->DIO12;
	Pins.STOP_R          = &HAL.IOs->pins->DIO13;
	Pins.HOME_REF        = &HAL.IOs->pins->DIO14;
	Pins.START           = &HAL.IOs->pins->DIO15;
	Pins.FREEZE          = &HAL.IOs->pins->DIO16;
	Pins.NRST            = &HAL.IOs->pins->DIO17;
	Pins.TARGET_REACHED  = &HAL.IOs->pins->DIO18;

	HAL.IOs->config->toOutput(Pins.NRST);
	HAL.IOs->config->toOutput(Pins.STOP_L);
	HAL.IOs->config->toOutput(Pins.STOP_R);
	HAL.IOs->config->toOutput(Pins.HOME_REF);
	HAL.IOs->config->toOutput(Pins.START);
	HAL.IOs->config->toOutput(Pins.FREEZE);

	HAL.IOs->config->setHigh(Pins.NRST);

	HAL.IOs->config->setHigh(Pins.STOP_L);
	HAL.IOs->config->setHigh(Pins.STOP_R);
	HAL.IOs->config->setHigh(Pins.HOME_REF);
	HAL.IOs->config->setHigh(Pins.START);
	HAL.IOs->config->setHigh(Pins.FREEZE);

	HAL.IOs->config->toInput(Pins.STANDBY_CLK);
	HAL.IOs->config->toInput(Pins.INTR);
	HAL.IOs->config->toInput(Pins.TARGET_REACHED);

	TMC4361A_SPIChannel = &HAL.SPI->ch1;
	TMC4361A_SPIChannel->CSN = &HAL.IOs->pins->SPI1_CSN;

	TMC4361A_config = Evalboards.ch1.config;

	tmc4361A_writeInt(TMC4361A_RAMPMODE, TMC4361A_RAMP_TRAPEZ);

	Evalboards.ch1.config->state        = CONFIG_RESET;
	Evalboards.ch1.config->configIndex  = 0;
	Evalboards.ch1.config->reset        = reset;
	Evalboards.ch1.config->restore      = restore;

	Evalboards.ch1.cover                = tmc4361A_cover;
	Evalboards.ch1.rotate               = rotate;
	Evalboards.ch1.right                = right;
	Evalboards.ch1.left                 = left;
	Evalboards.ch1.stop                 = stop;
	Evalboards.ch1.GAP                  = GAP;
	Evalboards.ch1.SAP                  = SAP;
	Evalboards.ch1.moveTo               = moveTo;
	Evalboards.ch1.moveBy               = moveBy;
	Evalboards.ch1.writeRegister        = writeRegister;
	Evalboards.ch1.readRegister         = readRegister;
	Evalboards.ch1.periodicJob          = periodicJob;
	Evalboards.ch1.userFunction         = userFunction;
	Evalboards.ch1.checkErrors          = checkErrors;
	Evalboards.ch1.numberOfMotors       = MOTORS;
	Evalboards.ch1.deInit               = deInit;
};
