#include "Board.h"
#include "tmc/ic/TMC4670/TMC4670.h"
#include "../hal/SPI.h"

#define MOTORS         1  // number of motors for this board
#define DEFAULT_MOTOR  0

IOPinTypeDef *PIN_DRV_ENN;
ConfigurationTypeDef *TMC4670_config;

typedef struct
{
	u32  startVoltage;
	u16  initWaitTime;
	u16  actualInitWaitTime;
	u8   initState;
	u8   initMode;
	u16  torqueMeasurementFactor; // u8.u8
	u8   motionMode;
} TMinimalMotorConfig;

TMinimalMotorConfig motorConfig[MOTORS];

// measurement
s32 ramMeasureType = 0;

static u32 mainLoopCount = 0;

// => SPI wrapper (extend the SPI functions to allow motor selection in your application)
int tmc4670_readInt(u8 motor, uint8 address)
{
	UNUSED(motor);
	return spi_ch1_readInt(address);
}

void tmc4670_writeInt(u8 motor, uint8 address, int value)
{
	UNUSED(motor);
	spi_ch1_writeInt(address, value);
}

u16 tmc4670_readRegister16BitValue(u8 motor, u8 address, u8 channel)
{
	UNUSED(motor);
	return spi_ch1_readRegister16BitValue(address, channel);
}

void tmc4670_writeRegister16BitValue(u8 motor, u8 address, u8 channel, u16 value)
{
	UNUSED(motor);
	spi_ch1_writeRegister16BitValue(address, channel, value);
}
// <= SPI wrapper

static uint32 rotate(uint8 motor, int32 velocity)
{
	if(motor >= MOTORS)
		return ERROR_MOTOR;

	tmc4670_setTargetVelocity(motor, velocity);

	return ERROR_NONE;
}

static uint32 right(uint8 motor, int32 velocity)
{
	return rotate(motor, velocity);
}

static uint32 left(uint8 motor, int32 velocity)
{
	return rotate(motor, -velocity);
}

static uint32 stop(uint8 motor)
{
	return rotate(motor, 0);
}

static uint32 moveTo(uint8 motor, int32 position)
{
	if(motor >= MOTORS)
		return ERROR_MOTOR;

	tmc4670_setAbsolutTargetPosition(motor, position);

	return ERROR_NONE;
}

static uint32 moveBy(uint8 motor, int32 *ticks)
{
	if(motor >= MOTORS)
		return ERROR_MOTOR;

	tmc4670_setRelativeTargetPosition(motor, *ticks);

	return ERROR_NONE;
}

// \xml <?xml version="1.0" encoding="UTF-8" standalone="no"?>
// \xml <tmc:product_module author="ED" xmlns:tmc="http://www.trinamic.com" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xsi:schemaLocation="http://www.trinamic.com https://www.trinamic.com/fileadmin/xml/TMC_Schema.xsd">
// \xml <name>TMC4670-EVAL</name>
// \xml <name_fw_version>4670</name_fw_version>
// \xml <hardware_version>1.0</hardware_version>
// \xml <software_version>2.91</software_version>
static uint32 handleParameter(u8 readWrite, u8 motor, u8 type, int32 *value)
{
	u32 errors = ERROR_NONE;

	if(motor >= MOTORS)
		return ERROR_MOTOR;

	// \xml <axis_parameters axis="0">
	switch(type)
	{
	case 4:
		// \xml <tmc:axis_parameter type="4" name="Maximum speed" identifier="par::MaxVelocity">
		// \xml <desc>Max. absolute speed for velocity and positioning mode.</desc>
		// \xml <tmc:value_unsigned min="0" max="100000" default="4000" unit="[rpm]" access="RW"></tmc:value_unsigned>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ)
			*value = (u32)tmc4670_readInt(motor, TMC4670_PID_VELOCITY_LIMIT);
		else if(readWrite == WRITE)
			tmc4670_writeInt(motor, TMC4670_PID_VELOCITY_LIMIT, *value);
		break;
	case 11:
		// \xml <tmc:axis_parameter type="11" name="acceleration" identifier="par::Acceleration">
		// \xml <desc>Acceleration parameter for ROL, ROR, and the velocity ramp of MVP.</desc>
		// \xml <tmc:value_unsigned min="0" max="100000" default="2000" unit="[rpm/s]" access="RW"></tmc:value_unsigned>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) // todo ADD 3: Implement this (LH) #1
		{
			*value = 0;
		} else if(readWrite == WRITE) {

		}
		break;
	case 12:
		// \xml <tmc:axis_parameter type="12" name="enable velocity ramp" identifier="par::EnableRamp">
		// \xml <desc>An activated ramp allows a defined acceleration for velocity and position mode.</desc>
		// \xml <tmc:value_bool desc_false="Deactivate velocity ramp generator." desc_true="Activate velocity ramp generator." default="true" access="RW"></tmc:value_bool>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) // todo ADD 3: Implement this (LH) #2
		{
			*value = 0;
		} else if(readWrite == WRITE) {

		}
		break;
	case 14:
		// \xml <tmc:axis_parameter type="14" name="clear on null" identifier="par::ClearOnNull">
		// \xml <desc>Clear the position counter on encoder N channel.</desc>
		// \xml <tmc:value_bool desc_false="do not clear position counter at next N channel event" desc_true="set position counter to zero at next N channel event" default="false" access="RW"></tmc:value_bool>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) // todo ADD 3: Implement this (LH) #3
		{
			*value = 0;
		} else if(readWrite == WRITE) {

		}
		break;
	// ADC offsets
	case 50:
		// \xml <tmc:axis_parameter type="50" name="ADC_I1_OFFSET" identifier="par::ADC_I1_OFFSET">
		// \xml   <desc>Offset for current ADC channel 1.</desc>
		// \xml   <tmc:value_signed min="#s16_MIN#" max="#s16_MAX#" default="0" unit="" access="RW"></tmc:value_signed>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ)
			*value = (s16) tmc4670_readRegister16BitValue(motor, TMC4670_ADC_I1_SCALE_OFFSET, BIT_0_TO_15);
		else if(readWrite == WRITE)
			tmc4670_writeRegister16BitValue(motor, TMC4670_ADC_I1_SCALE_OFFSET, BIT_0_TO_15, *value);
		break;
	case 51:
		// \xml <tmc:axis_parameter type="51" name="ADC_I0_OFFSET" identifier="par::ADC_I0_OFFSET">
		// \xml   <desc>Offset for current ADC channel 0.</desc>
		// \xml   <tmc:value_signed min="#s16_MIN#" max="#s16_MAX#" default="0" unit="" access="RW"></tmc:value_signed>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ)
			*value = (s16) tmc4670_readRegister16BitValue(motor, TMC4670_ADC_I0_SCALE_OFFSET, BIT_0_TO_15);
		else if(readWrite == WRITE)
			tmc4670_writeRegister16BitValue(motor, TMC4670_ADC_I0_SCALE_OFFSET, BIT_0_TO_15, *value);
		break;
		// ADC scaler values
	case 53:
		// \xml <tmc:axis_parameter type="53" name="ADC_I1_SCALE" identifier="par::ADC_I1_SCALE">
		// \xml   <desc>Scaling factor for current ADC channel 1.</desc>
		// \xml   <tmc:value_signed min="#s16_MIN#" max="#s16_MAX#" default="0" unit="" access="RW"></tmc:value_signed>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ)
			*value = (s16) tmc4670_readRegister16BitValue(motor, TMC4670_ADC_I1_SCALE_OFFSET, BIT_16_TO_31);
		else if(readWrite == WRITE)
			tmc4670_writeRegister16BitValue(motor, TMC4670_ADC_I1_SCALE_OFFSET, BIT_16_TO_31, *value);
		break;
	case 54:
		// \xml <tmc:axis_parameter type="54" name="ADC_I0_SCALE" identifier="par::ADC_I0_SCALE">
		// \xml   <desc>Scaling factor for current ADC channel 0.</desc>
		// \xml   <tmc:value_signed min="#s16_MIN#" max="#s16_MAX#" default="0" unit="" access="RW"></tmc:value_signed>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ)
			*value = (s16) tmc4670_readRegister16BitValue(motor, TMC4670_ADC_I0_SCALE_OFFSET, BIT_16_TO_31);
		else if(readWrite == WRITE)
			tmc4670_writeRegister16BitValue(motor, TMC4670_ADC_I0_SCALE_OFFSET, BIT_16_TO_31, *value);
		break;
	case 100:
		// \xml <tmc:axis_parameter type="100" name="ADC_I_U_RAW" identifier="par::ADC_I_U_RAW">
		// \xml <desc>Raw phase current U.</desc>
		// \xml <tmc:value_unsigned min="0" max="#u16_MAX#" default="0" unit="" access="R"></tmc:value_unsigned>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			tmc4670_writeInt(motor, TMC4670_ADC_RAW_ADDR, 1);
			*value = (u16) tmc4670_readRegister16BitValue(motor, TMC4670_ADC_RAW_DATA, BIT_0_TO_15);
		} else if(readWrite == WRITE) {
			errors |= ERROR_TYPE;
		}
		break;
	case 101:
		// \xml <tmc:axis_parameter type="101" name="ADC_I_V_RAW" identifier="par::ADC_I_V_RAW">
		// \xml <desc>Raw phase current V.</desc>
		// \xml <tmc:value_unsigned min="0" max="#u16_MAX#" default="0" unit="" access="R"></tmc:value_unsigned>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			tmc4670_writeInt(motor, TMC4670_ADC_RAW_ADDR, 1);
			*value = (u16) tmc4670_readRegister16BitValue(motor, TMC4670_ADC_RAW_DATA, BIT_16_TO_31);
		} else if(readWrite == WRITE) {
			errors |= ERROR_TYPE;
		}
		break;
	case 102:
		// ADC_IB_RAW
		// \xml <tmc:axis_parameter type="102" name="ADC_I_B_RAW" identifier="par::ADC_I_B_RAW">
		// \xml <desc>Raw phase current Bottom.</desc>
		// \xml <tmc:value_unsigned min="0" max="#u16_MAX#" default="0" unit="" access="R"></tmc:value_unsigned>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			tmc4670_writeInt(motor, TMC4670_ADC_RAW_ADDR, 2);
			*value = (u16) tmc4670_readRegister16BitValue(motor, TMC4670_ADC_RAW_DATA, BIT_0_TO_15);
		} else if(readWrite == WRITE) {
			errors |= ERROR_TYPE;
		}
		break;
	case 103:
		// \xml <tmc:axis_parameter type="103" name="ADC_I_UX_RAW" identifier="par::ADC_I_UX_RAW">
		// \xml <desc>Raw phase current U resp. X.</desc>
		// \xml <tmc:value_unsigned min="0" max="#u16_MAX#" default="0" unit="" access="R"></tmc:value_unsigned>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			tmc4670_writeInt(motor, TMC4670_ADC_RAW_ADDR, 0);
			*value = (u16) tmc4670_readRegister16BitValue(motor, TMC4670_ADC_RAW_DATA, BIT_0_TO_15);
		} else if(readWrite == WRITE) {
			errors |= ERROR_TYPE;
		}
		break;
	case 104:
		// \xml <tmc:axis_parameter type="104" name="ADC_I_WY_RAW" identifier="par::ADC_I_WY_RAW">
		// \xml <desc>Raw phase current W resp. Y.</desc>
		// \xml <tmc:value_unsigned min="0" max="#u16_MAX#" default="0" unit="" access="R"></tmc:value_unsigned>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			tmc4670_writeInt(motor, TMC4670_ADC_RAW_ADDR, 0);
			*value = (u16) tmc4670_readRegister16BitValue(motor, TMC4670_ADC_RAW_DATA, BIT_16_TO_31);
		} else if(readWrite == WRITE) {
			errors |= ERROR_TYPE;
		}
		break;
	case 105:
		// \xml <tmc:axis_parameter type="105" name="ADC_VM_RAW" identifier="par::ADC_VM_RAW">
		// \xml <desc>Raw supply voltage.</desc>
		// \xml <tmc:value_unsigned min="0" max="#u16_MAX#" default="0" unit="" access="R"></tmc:value_unsigned>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			tmc4670_writeInt(motor, TMC4670_ADC_RAW_ADDR, 3);
			*value = (u16) tmc4670_readRegister16BitValue(motor, TMC4670_ADC_RAW_DATA, BIT_0_TO_15);
		} else if(readWrite == WRITE) {
			errors |= ERROR_TYPE;
		}
		break;
	case 106:
		// \xml <tmc:axis_parameter type="106" name="ADC_T_MOSFETS_RAW" identifier="par::ADC_T_MOSFETS_RAW">
		// \xml <desc>Raw mosfet temperature.</desc>
		// \xml <tmc:value_unsigned min="0" max="#u16_MAX#" default="0" unit="" access="R"></tmc:value_unsigned>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			tmc4670_writeInt(motor, TMC4670_ADC_RAW_ADDR, 4);
			*value = (u16) tmc4670_readRegister16BitValue(motor, TMC4670_ADC_RAW_DATA, BIT_0_TO_15);
		} else if(readWrite == WRITE) {
			errors |= ERROR_TYPE;
		}
		break;
	case 107:
		// \xml <tmc:axis_parameter type="107" name="ADC_T_MOTOR_RAW" identifier="par::ADC_T_MOTOR_RAW">
		// \xml <desc>Raw motor temperature.</desc>
		// \xml <tmc:value_unsigned min="0" max="#u16_MAX#" default="0" unit="" access="R"></tmc:value_unsigned>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			tmc4670_writeInt(motor, TMC4670_ADC_RAW_ADDR, 4);
			*value = (u16) tmc4670_readRegister16BitValue(motor, TMC4670_ADC_RAW_DATA, BIT_16_TO_31);
		} else if(readWrite == WRITE) {
			errors |= ERROR_TYPE;
		}
		break;
	case 108:
		// \xml <tmc:axis_parameter type="108" name="ADC_U_UX_RAW" identifier="par::ADC_U_UX_RAW">
		// \xml <desc>Raw voltage terminal U resp. X.</desc>
		// \xml <tmc:value_unsigned min="0" max="#u16_MAX#" default="0" unit="" access="R"></tmc:value_unsigned>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			tmc4670_writeInt(motor, TMC4670_ADC_RAW_ADDR, 5);
			*value = (u16) tmc4670_readRegister16BitValue(motor, TMC4670_ADC_RAW_DATA, BIT_0_TO_15);
		} else if(readWrite == WRITE) {
			errors |= ERROR_TYPE;
		}
		break;
	case 109:
		// \xml <tmc:axis_parameter type="109" name="ADC_U_WY_RAW" identifier="par::ADC_U_WY_RAW">
		// \xml <desc>Raw voltage terminal W resp. Y.</desc>
		// \xml <tmc:value_unsigned min="0" max="#u16_MAX#" default="0" unit="" access="R"></tmc:value_unsigned>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			tmc4670_writeInt(motor, TMC4670_ADC_RAW_ADDR, 5);
			*value = (u16) tmc4670_readRegister16BitValue(motor, TMC4670_ADC_RAW_DATA, BIT_16_TO_31);
		} else if(readWrite == WRITE) {
			errors |= ERROR_TYPE;
		}
		break;
	case 110:
		// \xml <tmc:axis_parameter type="110" name="ADC_U_V_RAW" identifier="par::ADC_U_V_RAW">
		// \xml <desc>Raw voltage terminal V.</desc>
		// \xml <tmc:value_unsigned min="0" max="#u16_MAX#" default="0" unit="" access="R"></tmc:value_unsigned>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			tmc4670_writeInt(motor, TMC4670_ADC_RAW_ADDR, 6);
			*value = (u16) tmc4670_readRegister16BitValue(motor, TMC4670_ADC_RAW_DATA, BIT_0_TO_15);
		} else if(readWrite == WRITE) {
			errors |= ERROR_TYPE;
		}
		break;
	case 111:
		// \xml <tmc:axis_parameter type="111" name="AENC_UX_RAW" identifier="par::AENC_UX_RAW">
		// \xml <desc>Raw analog encoder voltage U resp. X.</desc>
		// \xml <tmc:value_unsigned min="0" max="#u16_MAX#" default="0" unit="" access="R"></tmc:value_unsigned>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			tmc4670_writeInt(motor, TMC4670_ADC_RAW_ADDR, 7);
			*value = (u16) tmc4670_readRegister16BitValue(motor, TMC4670_ADC_RAW_DATA, BIT_0_TO_15);
		} else if(readWrite == WRITE) {
			errors |= ERROR_TYPE;
		}
		break;
	case 112:
		// \xml <tmc:axis_parameter type="112" name="AENC_WY_RAW" identifier="par::AENC_WY_RAW">
		// \xml <desc>Raw analog encoder voltage W resp. Y.</desc>
		// \xml <tmc:value_unsigned min="0" max="#u16_MAX#" default="0" unit="" access="R"></tmc:value_unsigned>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			tmc4670_writeInt(motor, TMC4670_ADC_RAW_ADDR, 7);
			*value = (u16) tmc4670_readRegister16BitValue(motor, TMC4670_ADC_RAW_DATA, BIT_16_TO_31);
		} else if(readWrite == WRITE) {
			errors |= ERROR_TYPE;
		}
		break;
	case 113:
		// \xml <tmc:axis_parameter type="113" name="AENC_V_RAW" identifier="par::AENC_V_RAW">
		// \xml <desc>Raw analog encoder voltage V.</desc>
		// \xml <tmc:value_unsigned min="0" max="#u16_MAX#" default="0" unit="" access="R"></tmc:value_unsigned>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			tmc4670_writeInt(motor, TMC4670_ADC_RAW_ADDR, 8);
			*value = (u16) tmc4670_readRegister16BitValue(motor, TMC4670_ADC_RAW_DATA, BIT_0_TO_15);
		} else if(readWrite == WRITE) {
			errors |= ERROR_TYPE;
		}
		break;
	case 114:
		// \xml <tmc:axis_parameter type="114" name="AENC_N_RAW" identifier="par::AENC_N_RAW">
		// \xml <desc>Raw analog encoder zero position voltage N.</desc>
		// \xml <tmc:value_unsigned min="0" max="#u16_MAX#" default="0" unit="" access="R"></tmc:value_unsigned>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			tmc4670_writeInt(motor, TMC4670_ADC_RAW_ADDR, 8);
			*value = (u16) tmc4670_readRegister16BitValue(motor, TMC4670_ADC_RAW_DATA, BIT_16_TO_31);
		} else if(readWrite == WRITE) {
			errors |= ERROR_TYPE;
		}
		break;
	case 115:
		// \xml <tmc:axis_parameter type="115" name="ANALOG_GPI_RAW" identifier="par::ANALOG_GPI_RAW">
		// \xml <desc>Raw analog input voltage.</desc>
		// \xml <tmc:value_unsigned min="0" max="#u16_MAX#" default="0" unit="" access="R"></tmc:value_unsigned>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			tmc4670_writeInt(motor, TMC4670_ADC_RAW_ADDR, 9);
			*value = (u16) tmc4670_readRegister16BitValue(motor, TMC4670_ADC_RAW_DATA, BIT_0_TO_15);
		} else if(readWrite == WRITE) {
			errors |= ERROR_TYPE;
		}
		break;
	case 116:
		// \xml <tmc:axis_parameter type="116" name="ADC_I0_scaled" identifier="par::ADC_I0_scaled">
		// \xml <desc>Register of scaled current ADC_I0 value including signed added offset as input for the FOC.</desc>
		// \xml <tmc:value_signed min="#s16_MIN#" max="#s16_MAX#" default="0" unit="" access="R"></tmc:value_signed>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			*value = (s16) tmc4670_readRegister16BitValue(motor, TMC4670_ADC_IWY_IUX, BIT_0_TO_15);
		} else if(readWrite == WRITE) {
			errors |= ERROR_TYPE;
		}
		break;
	case 117:
		// \xml <tmc:axis_parameter type="117" name="ADC_I2_scaled" identifier="par::ADC_I2_scaled">
		// \xml <desc>Register of scaled current ADC_I2 value including signed added offset as input for the FOC.</desc>
		// \xml <tmc:value_signed min="#s16_MIN#" max="#s16_MAX#" default="0" unit="" access="R"></tmc:value_signed>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			*value = (s16) tmc4670_readRegister16BitValue(motor, TMC4670_ADC_IWY_IUX, BIT_16_TO_31);
		} else if(readWrite == WRITE) {
			errors |= ERROR_TYPE;
		}
		break;
	case 118:
		// \xml <tmc:axis_parameter type="118" name="ADC_I1_scaled" identifier="par::ADC_I1_scaled">
		// \xml <desc>Register of scaled current ADC_I1 value including signed added offset as input for the FOC.</desc>
		// \xml <tmc:value_signed min="#s16_MIN#" max="#s16_MAX#" default="0" unit="" access="R"></tmc:value_signed>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			*value = (s16) tmc4670_readRegister16BitValue(motor, TMC4670_ADC_IV, BIT_0_TO_15);
		} else if(readWrite == WRITE) {
			errors |= ERROR_TYPE;
		}
		break;
	case 120:
		// \xml <tmc:axis_parameter type="120" name="AENC_UX" identifier="par::AENC_UX">
		// \xml <desc>Register of scaled analog encoder value including signed added offset as input for the interpolator.</desc>
		// \xml <tmc:value_signed min="#s16_MIN#" max="#s16_MAX#" default="0" unit="" access="R"></tmc:value_signed>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			*value = (s16) tmc4670_readRegister16BitValue(motor, TMC4670_AENC_WY_UX, BIT_0_TO_15);
		} else if(readWrite == WRITE) {
			errors |= ERROR_TYPE;
		}
		break;
	case 121:
		// \xml <tmc:axis_parameter type="121" name="AENC_WY" identifier="par::AENC_WY">
		// \xml <desc>Register of scaled analog encoder value including signed added offset as input for the interpolator.</desc>
		// \xml <tmc:value_signed min="#s16_MIN#" max="#s16_MAX#" default="0" unit="" access="R"></tmc:value_signed>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			*value = (s16) tmc4670_readRegister16BitValue(motor, TMC4670_AENC_WY_UX, BIT_16_TO_31);
		} else if(readWrite == WRITE) {
			errors |= ERROR_TYPE;
		}
		break;
	case 122:
		// \xml <tmc:axis_parameter type="122" name="AENC_VN" identifier="par::AENC_VN">
		// \xml <desc>Register of scaled analog encoder value including signed added offset as input for the interpolator.</desc>
		// \xml <tmc:value_signed min="#s16_MIN#" max="#s16_MAX#" default="0" unit="" access="R"></tmc:value_signed>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			*value = (s16) tmc4670_readRegister16BitValue(motor, TMC4670_AENC_N_VN, BIT_0_TO_15);
		} else if(readWrite == WRITE) {
			errors |= ERROR_TYPE;
		}
		break;
	case 123:
		// \xml <tmc:axis_parameter type="123" name="AENC_N" identifier="par::AENC_N">
		// \xml <desc>Register of scaled analog encoder value including signed added offset of analog N pulse.</desc>
		// \xml <tmc:value_signed min="#s16_MIN#" max="#s16_MAX#" default="0" unit="" access="R"></tmc:value_signed>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			*value = (s16) tmc4670_readRegister16BitValue(motor, TMC4670_AENC_N_VN, BIT_16_TO_31);
		} else if(readWrite == WRITE) {
			errors |= ERROR_TYPE;
		}
		break;
	case 124:
		// \xml <tmc:axis_parameter type="124" name="ADCSD_I0_RAW" identifier="par::ADCSD_I0_RAW">
		// \xml <desc>Sigma Delta ADC_I0 current.</desc>
		// \xml <tmc:value_signed min="#s16_MIN#" max="#s16_MAX#" default="0" unit="" access="R"></tmc:value_signed>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			tmc4670_writeInt(motor, TMC4670_ADC_RAW_ADDR, 10);
			*value = (s16) tmc4670_readRegister16BitValue(motor, TMC4670_ADC_RAW_DATA, BIT_0_TO_15);
		} else if(readWrite == WRITE) {
			errors |= ERROR_TYPE;
		}
		break;
	case 125:
		// \xml <tmc:axis_parameter type="125" name="ADCSD_I1_RAW" identifier="par::ADCSD_I1_RAW">
		// \xml <desc>Sigma Delta ADC_I1 current.</desc>
		// \xml <tmc:value_signed min="#s16_MIN#" max="#s16_MAX#" default="0" unit="" access="R"></tmc:value_signed>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			tmc4670_writeInt(motor, TMC4670_ADC_RAW_ADDR, 10);
			*value = (s16) tmc4670_readRegister16BitValue(motor, TMC4670_ADC_RAW_DATA, BIT_16_TO_31);
		} else if(readWrite == WRITE) {
			errors |= ERROR_TYPE;
		}
		break;
	case 126:
		// \xml <tmc:axis_parameter type="126" name="ADCSD_I_B_RAW" identifier="par::ADCSD_I_B_RAW">
		// \xml <desc>Sigma Delta ADC current at bottom.</desc>
		// \xml <tmc:value_signed min="#s16_MIN#" max="#s16_MAX#" default="0" unit="" access="R"></tmc:value_signed>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			tmc4670_writeInt(motor, TMC4670_ADC_RAW_ADDR, 11);
			*value = (s16) tmc4670_readRegister16BitValue(motor, TMC4670_ADC_RAW_DATA, BIT_0_TO_15);
		} else if(readWrite == WRITE) {
			errors |= ERROR_TYPE;
		}
		break;
	case 127:
		// \xml <tmc:axis_parameter type="127" name="ADC_I0_EXT" identifier="par::ADC_I0_EXT">
		// \xml <desc>Register for write of ADC_I0 value from external source (e.g. CPU).</desc>
		// \xml <tmc:value_unsigned min="0" max="#u16_MAX#" default="0" unit="" access="R"></tmc:value_unsigned>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			*value = (u16) tmc4670_readRegister16BitValue(motor, TMC4670_ADC_I1_I0_EXT, BIT_0_TO_15);
		} else if(readWrite == WRITE) {
			errors |= ERROR_TYPE;
		}
		break;
	case 128:
		// \xml <tmc:axis_parameter type="128" name="ADC_I1_EXT" identifier="par::ADC_I1_EXT">
		// \xml <desc>Register for write of ADC_I1 value from external source (e.g. CPU).</desc>
		// \xml <tmc:value_unsigned min="0" max="#u16_MAX#" default="0" unit="" access="R"></tmc:value_unsigned>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			*value = (u16) tmc4670_readRegister16BitValue(motor, TMC4670_ADC_I1_I0_EXT, BIT_16_TO_31);
		} else if(readWrite == WRITE) {
			errors |= ERROR_TYPE;
		}
		break;
	case 132:
		// \xml <tmc:axis_parameter type="132" name="AENC_DECODER_COUNT" identifier="par::AENC_DECODER_COUNT">
		// \xml <desc>Decoder position, raw unscaled.</desc>
		// \xml <tmc:value_signed min="#s32_MIN#" max="#s32_MAX#" default="0" unit="" access="R"></tmc:value_signed>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			*value = (s32) tmc4670_readInt(motor, TMC4670_AENC_DECODER_COUNT);
		} else if(readWrite == WRITE) {
			errors |= ERROR_TYPE;
		}
		break;
	case 133:
		// \xml <tmc:axis_parameter type="133" name="AENC_DECODER_COUNT_N" identifier="par::AENC_DECODER_COUNT_N">
		// \xml <desc>Latched decoder position on analog N pulse event.</desc>
		// \xml <tmc:value_signed min="#s32_MIN#" max="#s32_MAX#" default="0" unit="" access="R"></tmc:value_signed>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			*value = (s32) tmc4670_readInt(motor, TMC4670_AENC_DECODER_COUNT_N);
		} else if(readWrite == WRITE) {
			errors |= ERROR_TYPE;
		}
		break;
	case 134:
		// \xml <tmc:axis_parameter type="134" name="AENC_DECODER_PHI_E" identifier="par::AENC_DECODER_PHI_E">
		// \xml <desc>Resulting phi_e available for the FOC.</desc>
		// \xml <tmc:value_signed min="#s16_MIN#" max="#s16_MAX#" default="0" unit="" access="R"></tmc:value_signed>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			*value = (s16) tmc4670_readRegister16BitValue(motor, TMC4670_AENC_DECODER_PHI_E_PHI_M, BIT_16_TO_31);
		} else if(readWrite == WRITE) {
			errors |= ERROR_TYPE;
		}
		break;
	case 135:
		// \xml <tmc:axis_parameter type="135" name="AENC_DECODER_PHI_M" identifier="par::AENC_DECODER_PHI_M">
		// \xml <desc>Resulting phi_m available for the FOC.</desc>
		// \xml <tmc:value_signed min="#s16_MIN#" max="#s16_MAX#" default="0" unit="" access="R"></tmc:value_signed>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			*value = (s16) tmc4670_readRegister16BitValue(motor, TMC4670_AENC_DECODER_PHI_E_PHI_M, BIT_0_TO_15);
		} else if(readWrite == WRITE) {
			errors |= ERROR_TYPE;
		}
		break;
	case 136:
		// \xml <tmc:axis_parameter type="136" name="AENC_DECODER_POSITION" identifier="par::AENC_DECODER_POSITION">
		// \xml <desc>Multi-turn position.</desc>
		// \xml <tmc:value_signed min="#s32_MIN#" max="#s32_MAX#" default="0" unit="" access="R"></tmc:value_signed>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			*value = (s32) tmc4670_readInt(motor, TMC4670_AENC_DECODER_POSITION);
		} else if(readWrite == WRITE) {
			errors |= ERROR_TYPE;
		}
		break;
	case 140:
		// \xml <tmc:axis_parameter type="140" name="OPENLOOP_VELOCITY_TARGET" identifier="par::OPENLOOP_VELOCITY_TARGET">
		// \xml <desc>Target velocity of open loop phi.</desc>
		// \xml <tmc:value_signed min="#s32_MIN#" max="#s32_MAX#" default="0" unit="[rpm]" access="R"></tmc:value_signed>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			*value = (s32) tmc4670_readInt(motor, TMC4670_OPENLOOP_VELOCITY_TARGET);
		} else if(readWrite == WRITE) {
			errors |= ERROR_TYPE;
		}
		break;
	case 141:
		// \xml <tmc:axis_parameter type="141" name="OPENLOOP_VELOCITY_ACTUAL" identifier="par::OPENLOOP_VELOCITY_ACTUAL">
		// \xml <desc>Actual velocity of open loop generator.</desc>
		// \xml <tmc:value_signed min="#s32_MIN#" max="#s32_MAX#" default="0" unit="[rpm]" access="R"></tmc:value_signed>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			*value = (s32) tmc4670_readInt(motor, TMC4670_OPENLOOP_VELOCITY_ACTUAL);
		} else if(readWrite == WRITE) {
			errors |= ERROR_TYPE;
		}
		break;
	case 142:
		// \xml <tmc:axis_parameter type="142" name="OPENLOOP_PHI" identifier="par::OPENLOOP_PHI">
		// \xml   <desc>Angle phi open loop (either mapped to electrical angel phi_e or mechanical angle phi_m).</desc>
		// \xml   <tmc:value_signed min="#s16_MIN#" max="#s16_MAX#" default="0" unit="" access="R"></tmc:value_signed>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			*value = (s16) tmc4670_readRegister16BitValue(motor, TMC4670_OPENLOOP_PHI, BIT_0_TO_15);
		} else if(readWrite == WRITE) {
			errors |= ERROR_TYPE;
		}
		break;
	case 150:
		// \xml <tmc:axis_parameter type="150" name="ABN_DECODER_COUNT" identifier="par::ABN_DECODER_COUNT">
		// \xml <desc>Raw decoder count; the digital decoder engine counts modulo (decoder_ppr).</desc>
		// \xml <tmc:value_unsigned min="0" max="#u24_MAX#" default="0" unit="" access="R"></tmc:value_unsigned>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			*value = tmc4670_readInt(motor, TMC4670_ABN_DECODER_COUNT) & 0x00FFFFFF;
		} else if(readWrite == WRITE) {
			errors |= ERROR_TYPE;
		}
		break;
	case 151:
		// \xml <tmc:axis_parameter type="151" name="ABN_DECODER_COUNT_N" identifier="par::ABN_DECODER_COUNT_N">
		// \xml <desc>Decoder count latched on N pulse, when N pulse clears decoder_count also decoder_count_n is 0.</desc>
		// \xml <tmc:value_unsigned min="0" max="#u24_MAX#" default="0" unit="" access="R"></tmc:value_unsigned>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			*value = tmc4670_readInt(motor, TMC4670_ABN_DECODER_COUNT_N) & 0x00FFFFFF;
		} else if(readWrite == WRITE) {
			errors |= ERROR_TYPE;
		}
		break;
	case 152:
		// \xml <tmc:axis_parameter type="152" name="ABN_DECODER_PHI_E" identifier="par::ABN_DECODER_PHI_E">
		// \xml   <desc></desc>
		// \xml   <tmc:value_signed min="#s16_MIN#" max="#s16_MAX#" default="0" unit="" access="R"></tmc:value_signed>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			*value = (s16) tmc4670_readRegister16BitValue(motor, TMC4670_ABN_DECODER_PHI_E_PHI_M, BIT_16_TO_31);
		} else if(readWrite == WRITE) {
			errors |= ERROR_TYPE;
		}
		break;
	case 153:
		// \xml <tmc:axis_parameter type="153" name="ABN_DECODER_PHI_M" identifier="par::ABN_DECODER_PHI_M">
		// \xml   <desc></desc>
		// \xml   <tmc:value_signed min="#s16_MIN#" max="#s16_MAX#" default="0" unit="" access="R"></tmc:value_signed>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			*value = (s16) tmc4670_readRegister16BitValue(motor, TMC4670_ABN_DECODER_PHI_E_PHI_M, BIT_0_TO_15);
		} else if(readWrite == WRITE) {
			errors |= ERROR_TYPE;
		}
		break;
	case 160:
		// \xml <tmc:axis_parameter type="160" name="HALL_PHI_E" identifier="par::HALL_PHI_E">
		// \xml <desc></desc>
		// \xml <tmc:value_signed min="#s16_MIN#" max="#s16_MAX#" default="0" unit="" access="R"></tmc:value_signed>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			*value = (s16) tmc4670_readRegister16BitValue(motor, TMC4670_HALL_PHI_E_INTERPOLATED_PHI_E, BIT_0_TO_15);
		} else if(readWrite == WRITE) {
			errors |= ERROR_TYPE;
		}
		break;
	case 161:
		// \xml <tmc:axis_parameter type="161" name="HALL_PHI_E_INTERPOLATED" identifier="par::HALL_PHI_E_INTERPOLATED">
		// \xml <desc></desc>
		// \xml <tmc:value_signed min="#s16_MIN#" max="#s16_MAX#" default="0" unit="" access="R"></tmc:value_signed>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			*value = (s16) tmc4670_readRegister16BitValue(motor, TMC4670_HALL_PHI_E_INTERPOLATED_PHI_E, BIT_16_TO_31);
		} else if(readWrite == WRITE) {
			errors |= ERROR_TYPE;
		}
		break;
	case 162:
		// \xml <tmc:axis_parameter type="162" name="AENC_DECODER_PHI_A_RAW" identifier="par::AENC_DECODER_PHI_A_RAW">
		// \xml <desc>Raw analog angle phi calculated from analog AENC inputs (analog hall, analog SinCos, ...).</desc>
		// \xml <tmc:value_signed min="#s16_MIN#" max="#s16_MAX#" default="0" unit="" access="R"></tmc:value_signed>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			*value = (s16) tmc4670_readRegister16BitValue(motor, TMC4670_AENC_DECODER_PHI_A_RAW, BIT_0_TO_15);
		} else if(readWrite == WRITE) {
			errors |= ERROR_TYPE;
		}
		break;
	case 163:
		// \xml <tmc:axis_parameter type="163" name="AENC_DECODER_PHI_A" identifier="par::AENC_DECODER_PHI_A">
		// \xml <desc>Resulting phi available for the FOC.</desc>
		// \xml <tmc:value_signed min="#s16_MIN#" max="#s16_MAX#" default="0" unit="" access="R"></tmc:value_signed>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			*value = (s16) tmc4670_readRegister16BitValue(motor, TMC4670_AENC_DECODER_PHI_A, BIT_0_TO_15);
		} else if(readWrite == WRITE) {
			errors |= ERROR_TYPE;
		}
		break;
	case 164:
		// \xml <tmc:axis_parameter type="164" name="HALL_PHI_M" identifier="par::HALL_PHI_M">
		// \xml <desc></desc>
		// \xml <tmc:value_signed min="#s16_MIN#" max="#s16_MAX#" default="0" unit="" access="R"></tmc:value_signed>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			*value = (s16) tmc4670_readRegister16BitValue(motor, TMC4670_HALL_PHI_M, BIT_0_TO_15);
		} else if(readWrite == WRITE) {
			errors |= ERROR_TYPE;
		}
		break;
	case 170:
		// \xml <tmc:axis_parameter type="170" name="PHI_E" identifier="par::PHI_E">
		// \xml <desc></desc>
		// \xml <tmc:value_signed min="#s16_MIN#" max="#s16_MAX#" default="0" unit="" access="R"></tmc:value_signed>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			*value = (s16) tmc4670_readRegister16BitValue(motor, TMC4670_PHI_E, BIT_0_TO_15);
		} else if(readWrite == WRITE) {
			errors |= ERROR_TYPE;
		}
		break;
	case 171:
		// \xml <tmc:axis_parameter type="171" name="PID_TORQUE_TARGET" identifier="par::TargetTorque">
		// \xml <desc></desc>
		// \xml <tmc:value_signed min="#s16_MIN#" max="#s16_MAX#" default="0" unit="" access="RW"></tmc:value_signed>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ)
			*value = (s16) tmc4670_readRegister16BitValue(motor, TMC4670_PID_TORQUE_FLUX_TARGET, BIT_16_TO_31);
		else if(readWrite == WRITE)
			tmc4670_writeRegister16BitValue(motor, TMC4670_PID_TORQUE_FLUX_TARGET, BIT_16_TO_31, *value);
		break;
	case 172:
		// \xml <tmc:axis_parameter type="172" name="PID_FLUX_TARGET" identifier="par::PID_FLUX_TARGET">
		// \xml <desc></desc>
		// \xml <tmc:value_signed min="#s16_MIN#" max="#s16_MAX#" default="0" unit="" access="RW"></tmc:value_signed>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ)
			*value = (s16) tmc4670_readRegister16BitValue(motor, TMC4670_PID_TORQUE_FLUX_TARGET, BIT_0_TO_15);
		else if(readWrite == WRITE)
			tmc4670_writeRegister16BitValue(motor, TMC4670_PID_TORQUE_FLUX_TARGET, BIT_0_TO_15, *value);
		break;
	case 173:
		// \xml <tmc:axis_parameter type="173" name="PID_VELOCITY_TARGET" identifier="par::PID_VELOCITY_TARGET">
		// \xml <desc></desc>
		// \xml <tmc:value_signed min="#s32_MIN#" max="#s32_MAX#" default="0" unit="" access="R"></tmc:value_signed>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			*value = (s32) tmc4670_readInt(motor, TMC4670_PID_VELOCITY_TARGET);
		} else if(readWrite == WRITE) {
			errors |= ERROR_TYPE;
		}
		break;
	case 174:
		// \xml <tmc:axis_parameter type="174" name="PID_POSITION_TARGET" identifier="par::TargetPosition">
		// \xml <desc></desc>
		// \xml <tmc:value_signed min="#s32_MIN#" max="#s32_MAX#" default="0" unit="" access="RW"></tmc:value_signed>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			*value = (s32) tmc4670_readInt(motor, TMC4670_PID_POSITION_TARGET);
		} else if(readWrite == WRITE) {
			tmc4670_setAbsolutTargetPosition(motor, *value);
		}
		break;
	case 175:
		// \xml <tmc:axis_parameter type="175" name="PID_TORQUE_ACTUAL" identifier="par::ActualTorqueRaw">
		// \xml <desc>Actual torque.</desc>
		// \xml <tmc:value_signed min="#s16_MIN#" max="#s16_MAX#" default="0" unit="" access="R"></tmc:value_signed>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			*value = (s16) tmc4670_readRegister16BitValue(motor, TMC4670_PID_TORQUE_FLUX_ACTUAL, BIT_16_TO_31);
		} else if(readWrite == WRITE) {
			errors |= ERROR_TYPE;
		}
		break;
	case 176:
		// \xml <tmc:axis_parameter type="176" name="PID_TORQUE_ACTUAL_mA" identifier="par::ActualTorque">
		// \xml <desc>Actual scaled torque [mA].</desc>
		// \xml <tmc:value_signed min="#s16_MIN#" max="#s16_MAX#" default="0" unit="[mA]" access="R"></tmc:value_signed>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			*value = tmc4670_getActualTorque_mA(motor, motorConfig[motor].torqueMeasurementFactor);
		} else if(readWrite == WRITE) {
			errors |= ERROR_TYPE;
		}
		break;
	case 177:
		// \xml <tmc:axis_parameter type="177" name="PID_FLUX_ACTUAL" identifier="par::PID_FLUX_ACTUAL">
		// \xml <desc>Actual flux.</desc>
		// \xml <tmc:value_signed min="#s16_MIN#" max="#s16_MAX#" default="0" unit="" access="R"></tmc:value_signed>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			*value = (s16) tmc4670_readRegister16BitValue(motor, TMC4670_PID_TORQUE_FLUX_ACTUAL, BIT_0_TO_15);
		} else if(readWrite == WRITE) {
			errors |= ERROR_TYPE;
		}
		break;
	case 178:
		// \xml <tmc:axis_parameter type="178" name="PID_VELOCITY_ACTUAL" identifier="par::ActualVelocity">
		// \xml <desc>Actual velocity.</desc>
		// \xml <tmc:value_signed min="#s32_MIN#" max="#s32_MAX#" default="0" unit="[rpm]" access="R"></tmc:value_signed>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			*value = tmc4670_getActualVelocity(motor);
		} else if(readWrite == WRITE) {
			errors |= ERROR_TYPE;
		}
		break;
	case 179:
		// \xml <tmc:axis_parameter type="179" name="PID_POSITION_ACTUAL" identifier="par::ActualPosition">
		// \xml <desc>Actual multi turn position for positioning.</desc>
		// \xml <tmc:value_signed min="#s32_MIN#" max="#s32_MAX#" default="0" unit="" access="RW"></tmc:value_signed>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ)
			*value = tmc4670_getActualPosition(motor);
		else if(readWrite == WRITE)
			tmc4670_setActualPosition(motor, *value);
		break;
	case 180:
		// \xml <tmc:axis_parameter type="180" name="PID_TORQUE_ERROR" identifier="par::PID_TORQUE_ERROR">
		// \xml <desc></desc>
		// \xml <tmc:value_signed min="#s32_MIN#" max="#s32_MAX#" default="0" unit="" access="R"></tmc:value_signed>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			tmc4670_writeInt(motor, TMC4670_PID_ERROR_ADDR, 0);
			*value = tmc4670_readInt(motor, TMC4670_PID_ERROR_DATA);
		} else if(readWrite == WRITE) {
			errors |= ERROR_TYPE;
		}
		break;
	case 181:
		// \xml <tmc:axis_parameter type="181" name="PID_FLUX_ERROR" identifier="par::PID_FLUX_ERROR">
		// \xml <desc></desc>
		// \xml <tmc:value_signed min="#s32_MIN#" max="#s32_MAX#" default="0" unit="" access="R"></tmc:value_signed>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			tmc4670_writeInt(motor, TMC4670_PID_ERROR_ADDR, 1);
			*value = tmc4670_readInt(motor, TMC4670_PID_ERROR_DATA);
		} else if(readWrite == WRITE) {
			errors |= ERROR_TYPE;
		}
		break;
	case 182:
		// \xml <tmc:axis_parameter type="182" name="PID_VELOCITY_ERROR" identifier="par::PID_VELOCITY_ERROR">
		// \xml <desc></desc>
		// \xml <tmc:value_signed min="#s32_MIN#" max="#s32_MAX#" default="0" unit="" access="R"></tmc:value_signed>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			tmc4670_writeInt(motor, TMC4670_PID_ERROR_ADDR, 2);
			*value = tmc4670_readInt(motor, TMC4670_PID_ERROR_DATA);
		} else if(readWrite == WRITE) {
			errors |= ERROR_TYPE;
		}
		break;
	case 183:
		// \xml <tmc:axis_parameter type="183" name="PID_POSITION_ERROR" identifier="par::PID_POSITION_ERROR">
		// \xml <desc></desc>
		// \xml <tmc:value_signed min="#s32_MIN#" max="#s32_MAX#" default="0" unit="" access="R"></tmc:value_signed>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			tmc4670_writeInt(motor, TMC4670_PID_ERROR_ADDR, 3);
			*value = tmc4670_readInt(motor, TMC4670_PID_ERROR_DATA);
		} else if(readWrite == WRITE) {
			errors |= ERROR_TYPE;
		}
		break;
	case 184:
		// \xml <tmc:axis_parameter type="184" name="PID_TORQUE_ERROR_SUM" identifier="par::PID_TORQUE_ERROR_SUM">
		// \xml <desc></desc>
		// \xml <tmc:value_signed min="#s32_MIN#" max="#s32_MAX#" default="0" unit="" access="R"></tmc:value_signed>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			tmc4670_writeInt(motor, TMC4670_PID_ERROR_ADDR, 4);
			*value = tmc4670_readInt(motor, TMC4670_PID_ERROR_DATA);
		} else if(readWrite == WRITE) {
			errors |= ERROR_TYPE;
		}
		break;
	case 185:
		// \xml <tmc:axis_parameter type="185" name="PID_FLUX_ERROR_SUM" identifier="par::PID_FLUX_ERROR_SUM">
		// \xml <desc></desc>
		// \xml <tmc:value_signed min="#s32_MIN#" max="#s32_MAX#" default="0" unit="" access="R"></tmc:value_signed>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			tmc4670_writeInt(motor, TMC4670_PID_ERROR_ADDR, 5);
			*value = tmc4670_readInt(motor, TMC4670_PID_ERROR_DATA);
		} else if(readWrite == WRITE) {
			errors |= ERROR_TYPE;
		}
		break;
	case 186:
		// \xml <tmc:axis_parameter type="186" name="PID_VELOCITY_ERROR_SUM" identifier="par::PID_VELOCITY_ERROR_SUM">
		// \xml <desc></desc>
		// \xml <tmc:value_signed min="#s32_MIN#" max="#s32_MAX#" default="0" unit="" access="R"></tmc:value_signed>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			tmc4670_writeInt(motor, TMC4670_PID_ERROR_ADDR, 6);
			*value = tmc4670_readInt(motor, TMC4670_PID_ERROR_DATA);
		} else if(readWrite == WRITE) {
			errors |= ERROR_TYPE;
		}
		break;
	case 187:
		// \xml <tmc:axis_parameter type="187" name="PID_POSITION_ERROR_SUM" identifier="par::PID_POSITION_ERROR_SUM">
		// \xml <desc></desc>
		// \xml <tmc:value_signed min="#s32_MIN#" max="#s32_MAX#" default="0" unit="" access="R"></tmc:value_signed>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			tmc4670_writeInt(motor, TMC4670_PID_ERROR_ADDR, 7);
			*value = tmc4670_readInt(motor, TMC4670_PID_ERROR_DATA);
		} else if(readWrite == WRITE) {
			errors |= ERROR_TYPE;
		}
		break;
	case 189:
		// \xml <tmc:axis_parameter type="189" name="PIDIN_TARGET_TORQUE" identifier="par::TargetTorqueRaw">
		// \xml <desc></desc>
		// \xml <tmc:value_signed min="#s32_MIN#" max="#s32_MAX#" default="0" unit="" access="RW"></tmc:value_signed>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			*value = tmc4670_getTargetTorque_raw(motor);
		} else if(readWrite == WRITE) {
			tmc4670_setTargetTorque_raw(motor, *value);
		}
		break;
	case 190:
		// \xml <tmc:axis_parameter type="190" name="PIDIN_TARGET_TORQUE_mA" identifier="par::TargetTorque">
		// \xml <desc>Actual scaled target torque [mA].</desc>
		// \xml <tmc:value_signed min="#s32_MIN#" max="#s32_MAX#" default="0" unit="[mA]" access="RW"></tmc:value_signed>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			*value = tmc4670_getTargetTorque_mA(motor, motorConfig[motor].torqueMeasurementFactor);
		} else if(readWrite == WRITE) {
			tmc4670_setTargetTorque_mA(motor, motorConfig[motor].torqueMeasurementFactor, *value);
		}
		break;
	case 191:
		// \xml <tmc:axis_parameter type="191" name="PIDIN_TARGET_FLUX" identifier="par::PIDIN_TARGET_FLUX">
		// \xml <desc></desc>
		// \xml <tmc:value_signed min="#s32_MIN#" max="#s32_MAX#" default="0" unit="" access="R"></tmc:value_signed>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			tmc4670_writeInt(motor, TMC4670_INTERIM_ADDR, 1);
			*value = tmc4670_readInt(motor, TMC4670_INTERIM_DATA);
		} else if(readWrite == WRITE) {
			errors |= ERROR_TYPE;
		}
		break;
	case 192:
		// \xml <tmc:axis_parameter type="192" name="PIDIN_TARGET_VELOCITY" identifier="par::TargetVelocity">
		// \xml <desc></desc>
		// \xml <tmc:value_signed min="#s32_MIN#" max="#s32_MAX#" default="0" unit="[rpm]" access="RW"></tmc:value_signed>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			*value = tmc4670_getTargetVelocity(motor);
		} else if(readWrite == WRITE) {
			tmc4670_setTargetVelocity(motor, *value);
		}
		break;
	case 193:
		// \xml <tmc:axis_parameter type="193" name="PIDIN_TARGET_POSITION" identifier="par::PIDIN_TARGET_POSITION">
		// \xml <desc></desc>
		// \xml <tmc:value_signed min="#s32_MIN#" max="#s32_MAX#" default="0" unit="" access="R"></tmc:value_signed>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			tmc4670_writeInt(motor, TMC4670_INTERIM_ADDR, 3);
			*value = tmc4670_readInt(motor, TMC4670_INTERIM_DATA);
		} else if(readWrite == WRITE) {
			errors |= ERROR_TYPE;
		}
		break;
	case 194:
		// \xml <tmc:axis_parameter type="194" name="PIDOUT_TARGET_TORQUE" identifier="par::PIDOUT_TARGET_TORQUE">
		// \xml <desc></desc>
		// \xml <tmc:value_signed min="#s32_MIN#" max="#s32_MAX#" default="0" unit="" access="R"></tmc:value_signed>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			tmc4670_writeInt(motor, TMC4670_INTERIM_ADDR, 4);
			*value = tmc4670_readInt(motor, TMC4670_INTERIM_DATA);
		} else if(readWrite == WRITE) {
			errors |= ERROR_TYPE;
		}
		break;
	case 195:
		// \xml <tmc:axis_parameter type="195" name="PIDOUT_TARGET_FLUX" identifier="par::PIDOUT_TARGET_FLUX">
		// \xml <desc></desc>
		// \xml <tmc:value_signed min="#s32_MIN#" max="#s32_MAX#" default="0" unit="" access="R"></tmc:value_signed>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			tmc4670_writeInt(motor, TMC4670_INTERIM_ADDR, 5);
			*value = tmc4670_readInt(motor, TMC4670_INTERIM_DATA);
		} else if(readWrite == WRITE) {
			errors |= ERROR_TYPE;
		}
		break;
	case 196:
		// \xml <tmc:axis_parameter type="196" name="PIDOUT_TARGET_VELOCITY" identifier="par::PIDOUT_TARGET_VELOCITY">
		// \xml <desc></desc>
		// \xml <tmc:value_signed min="#s32_MIN#" max="#s32_MAX#" default="0" unit="" access="R"></tmc:value_signed>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			tmc4670_writeInt(motor, TMC4670_INTERIM_ADDR, 6);
			*value = tmc4670_readInt(motor, TMC4670_INTERIM_DATA);
		} else if(readWrite == WRITE) {
			errors |= ERROR_TYPE;
		}
		break;
	case 197:
		// \xml <tmc:axis_parameter type="197" name="PIDOUT_TARGET_POSITION" identifier="par::PIDOUT_TARGET_POSITION">
		// \xml <desc></desc>
		// \xml <tmc:value_signed min="#s32_MIN#" max="#s32_MAX#" default="0" unit="" access="R"></tmc:value_signed>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			tmc4670_writeInt(motor, TMC4670_INTERIM_ADDR, 7);
			*value = tmc4670_readInt(motor, TMC4670_INTERIM_DATA);
		} else if(readWrite == WRITE) {
			errors |= ERROR_TYPE;
		}
		break;
	case 198:
		// \xml <tmc:axis_parameter type="198" name="FOC_IUX" identifier="par::FOC_IUX">
		// \xml <desc></desc>
		// \xml <tmc:value_signed min="#s16_MIN#" max="#s16_MAX#" default="0" unit="" access="R"></tmc:value_signed>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			tmc4670_writeInt(motor, TMC4670_INTERIM_ADDR, 8);
			*value = (s16) tmc4670_readRegister16BitValue(motor, TMC4670_INTERIM_DATA, BIT_0_TO_15);
		} else if(readWrite == WRITE) {
			errors |= ERROR_TYPE;
		}
		break;
	case 199:
		// \xml <tmc:axis_parameter type="199" name="FOC_IWY" identifier="par::FOC_IWY">
		// \xml <desc></desc>
		// \xml <tmc:value_signed min="#s16_MIN#" max="#s16_MAX#" default="0" unit="" access="R"></tmc:value_signed>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			tmc4670_writeInt(motor, TMC4670_INTERIM_ADDR, 8);
			*value = (s16) tmc4670_readRegister16BitValue(motor, TMC4670_INTERIM_DATA, BIT_16_TO_31);
		} else if(readWrite == WRITE) {
			errors |= ERROR_TYPE;
		}
		break;
	case 200:
		// \xml <tmc:axis_parameter type="200" name="FOC_IV" identifier="par::FOC_IV">
		// \xml <desc></desc>
		// \xml <tmc:value_signed min="#s16_MIN#" max="#s16_MAX#" default="0" unit="" access="R"></tmc:value_signed>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			tmc4670_writeInt(motor, TMC4670_INTERIM_ADDR, 9);
			*value = (s16) tmc4670_readRegister16BitValue(motor, TMC4670_INTERIM_DATA, BIT_0_TO_15);
		} else if(readWrite == WRITE) {
			errors |= ERROR_TYPE;
		}
		break;
	case 201:
		// \xml <tmc:axis_parameter type="201" name="FOC_IA" identifier="par::FOC_IA">
		// \xml <desc></desc>
		// \xml <tmc:value_signed min="#s16_MIN#" max="#s16_MAX#" default="0" unit="" access="R"></tmc:value_signed>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			tmc4670_writeInt(motor, TMC4670_INTERIM_ADDR, 10);
			*value = (s16) tmc4670_readRegister16BitValue(motor, TMC4670_INTERIM_DATA, BIT_0_TO_15);
		} else if(readWrite == WRITE) {
			errors |= ERROR_TYPE;
		}
		break;
	case 202:
		// \xml <tmc:axis_parameter type="202" name="FOC_IB" identifier="par::FOC_IB">
		// \xml <desc></desc>
		// \xml <tmc:value_signed min="#s16_MIN#" max="#s16_MAX#" default="0" unit="" access="R"></tmc:value_signed>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			tmc4670_writeInt(motor, TMC4670_INTERIM_ADDR, 10);
			*value = (s16) tmc4670_readRegister16BitValue(motor, TMC4670_INTERIM_DATA, BIT_16_TO_31);
		} else if(readWrite == WRITE) {
			errors |= ERROR_TYPE;
		}
		break;
	case 203:
		// \xml <tmc:axis_parameter type="203" name="FOC_ID" identifier="par::FOC_ID">
		// \xml <desc></desc>
		// \xml <tmc:value_signed min="#s16_MIN#" max="#s16_MAX#" default="0" unit="" access="R"></tmc:value_signed>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			tmc4670_writeInt(motor, TMC4670_INTERIM_ADDR, 11);
			*value = (s16) tmc4670_readRegister16BitValue(motor, TMC4670_INTERIM_DATA, BIT_0_TO_15);
		} else if(readWrite == WRITE) {
			errors |= ERROR_TYPE;
		}
		break;
	case 204:
		// \xml <tmc:axis_parameter type="204" name="FOC_IQ" identifier="par::FOC_IQ">
		// \xml <desc></desc>
		// \xml <tmc:value_signed min="#s16_MIN#" max="#s16_MAX#" default="0" unit="" access="R"></tmc:value_signed>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			tmc4670_writeInt(motor, TMC4670_INTERIM_ADDR, 11);
			*value = (s16) tmc4670_readRegister16BitValue(motor, TMC4670_INTERIM_DATA, BIT_16_TO_31);
		} else if(readWrite == WRITE) {
			errors |= ERROR_TYPE;
		}
		break;
	case 205:
		// \xml <tmc:axis_parameter type="205" name="FOC_UD" identifier="par::FOC_UD">
		// \xml <desc></desc>
		// \xml <tmc:value_signed min="#s16_MIN#" max="#s16_MAX#" default="0" unit="" access="R"></tmc:value_signed>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			tmc4670_writeInt(motor, TMC4670_INTERIM_ADDR, 12);
			*value = (s16) tmc4670_readRegister16BitValue(motor, TMC4670_INTERIM_DATA, BIT_0_TO_15);
		} else if(readWrite == WRITE) {
			errors |= ERROR_TYPE;
		}
		break;
	case 206:
		// \xml <tmc:axis_parameter type="206" name="FOC_UQ" identifier="par::FOC_UQ">
		// \xml <desc></desc>
		// \xml <tmc:value_signed min="#s16_MIN#" max="#s16_MAX#" default="0" unit="" access="R"></tmc:value_signed>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			tmc4670_writeInt(motor, TMC4670_INTERIM_ADDR, 12);
			*value = (s16) tmc4670_readRegister16BitValue(motor, TMC4670_INTERIM_DATA, BIT_16_TO_31);
		} else if(readWrite == WRITE) {
			errors |= ERROR_TYPE;
		}
		break;
	case 207:
		// \xml <tmc:axis_parameter type="207" name="FOC_UD_LIMITED" identifier="par::FOC_UD_LIMITED">
		// \xml <desc></desc>
		// \xml <tmc:value_signed min="#s16_MIN#" max="#s16_MAX#" default="0" unit="" access="R"></tmc:value_signed>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			tmc4670_writeInt(motor, TMC4670_INTERIM_ADDR, 13);
			*value = (s16) tmc4670_readRegister16BitValue(motor, TMC4670_INTERIM_DATA, BIT_0_TO_15);
		} else if(readWrite == WRITE) {
			errors |= ERROR_TYPE;
		}
		break;
	case 208:
		// \xml <tmc:axis_parameter type="208" name="FOC_UQ_LIMITED" identifier="par::FOC_UQ_LIMITED">
		// \xml <desc></desc>
		// \xml <tmc:value_signed min="#s16_MIN#" max="#s16_MAX#" default="0" unit="" access="R"></tmc:value_signed>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			tmc4670_writeInt(motor, TMC4670_INTERIM_ADDR, 13);
			*value = (s16) tmc4670_readRegister16BitValue(motor, TMC4670_INTERIM_DATA, BIT_16_TO_31);
		} else if(readWrite == WRITE) {
			errors |= ERROR_TYPE;
		}
		break;
	case 209:
		// \xml <tmc:axis_parameter type="209" name="FOC_UA" identifier="par::FOC_UA">
		// \xml <desc></desc>
		// \xml <tmc:value_signed min="#s16_MIN#" max="#s16_MAX#" default="0" unit="" access="R"></tmc:value_signed>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			tmc4670_writeInt(motor, TMC4670_INTERIM_ADDR, 14);
			*value = (s16) tmc4670_readRegister16BitValue(motor, TMC4670_INTERIM_DATA, BIT_0_TO_15);
		} else if(readWrite == WRITE) {
			errors |= ERROR_TYPE;
		}
		break;
	case 210:
		// \xml <tmc:axis_parameter type="210" name="FOC_UB" identifier="par::FOC_UB">
		// \xml <desc></desc>
		// \xml <tmc:value_signed min="#s16_MIN#" max="#s16_MAX#" default="0" unit="" access="R"></tmc:value_signed>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			tmc4670_writeInt(motor, TMC4670_INTERIM_ADDR, 14);
			*value = (s16) tmc4670_readRegister16BitValue(motor, TMC4670_INTERIM_DATA, BIT_16_TO_31);
		} else if(readWrite == WRITE) {
			errors |= ERROR_TYPE;
		}
		break;
	case 211:
		// \xml <tmc:axis_parameter type="211" name="FOC_UUX" identifier="par::FOC_UUX">
		// \xml <desc></desc>
		// \xml <tmc:value_signed min="#s16_MIN#" max="#s16_MAX#" default="0" unit="" access="R"></tmc:value_signed>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			tmc4670_writeInt(motor, TMC4670_INTERIM_ADDR, 15);
			*value = (s16) tmc4670_readRegister16BitValue(motor, TMC4670_INTERIM_DATA, BIT_0_TO_15);
		} else if(readWrite == WRITE) {
			errors |= ERROR_TYPE;
		}
		break;
	case 212:
		// \xml <tmc:axis_parameter type="212" name="FOC_UWY" identifier="par::FOC_UWY">
		// \xml <desc></desc>
		// \xml <tmc:value_signed min="#s16_MIN#" max="#s16_MAX#" default="0" unit="" access="R"></tmc:value_signed>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			tmc4670_writeInt(motor, TMC4670_INTERIM_ADDR, 15);
			*value = (s16) tmc4670_readRegister16BitValue(motor, TMC4670_INTERIM_DATA, BIT_16_TO_31);
		} else if(readWrite == WRITE) {
			errors |= ERROR_TYPE;
		}
		break;
	case 213:
		// \xml <tmc:axis_parameter type="213" name="FOC_UV" identifier="par::FOC_UV">
		// \xml <desc></desc>
		// \xml <tmc:value_signed min="#s16_MIN#" max="#s16_MAX#" default="0" unit="" access="R"></tmc:value_signed>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			tmc4670_writeInt(motor, TMC4670_INTERIM_ADDR, 16);
			*value = (s16) tmc4670_readRegister16BitValue(motor, TMC4670_INTERIM_DATA, BIT_0_TO_15);
		} else if(readWrite == WRITE) {
			errors |= ERROR_TYPE;
		}
		break;
	case 214:
		// \xml <tmc:axis_parameter type="214" name="PWM_UX" identifier="par::PWM_UX">
		// \xml <desc></desc>
		// \xml <tmc:value_signed min="#s16_MIN#" max="#s16_MAX#" default="0" unit="" access="R"></tmc:value_signed>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			tmc4670_writeInt(motor, TMC4670_INTERIM_ADDR, 17);
			*value = (s16) tmc4670_readRegister16BitValue(motor, TMC4670_INTERIM_DATA, BIT_0_TO_15);
		} else if(readWrite == WRITE) {
			errors |= ERROR_TYPE;
		}
		break;
	case 215:
		// \xml <tmc:axis_parameter type="215" name="PWM_WY" identifier="par::PWM_WY">
		// \xml <desc></desc>
		// \xml <tmc:value_signed min="#s16_MIN#" max="#s16_MAX#" default="0" unit="" access="R"></tmc:value_signed>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			tmc4670_writeInt(motor, TMC4670_INTERIM_ADDR, 17);
			*value = (s16) tmc4670_readRegister16BitValue(motor, TMC4670_INTERIM_DATA, BIT_16_TO_31);
		} else if(readWrite == WRITE) {
			errors |= ERROR_TYPE;
		}
		break;
	case 216:
		// \xml <tmc:axis_parameter type="216" name="PWM_UV" identifier="par::PWM_UV">
		// \xml <desc></desc>
		// \xml <tmc:value_signed min="#s16_MIN#" max="#s16_MAX#" default="0" unit="" access="R"></tmc:value_signed>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			tmc4670_writeInt(motor, TMC4670_INTERIM_ADDR, 18);
			*value = (s16) tmc4670_readRegister16BitValue(motor, TMC4670_INTERIM_DATA, BIT_0_TO_15);
		} else if(readWrite == WRITE) {
			errors |= ERROR_TYPE;
		}
		break;
	case 217:
		// \xml <tmc:axis_parameter type="217" name="ADC_I_0" identifier="par::ADC_I_0">
		// \xml <desc></desc>
		// \xml <tmc:value_unsigned min="0" max="#u16_MAX#" default="0" unit="" access="R"></tmc:value_unsigned>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			tmc4670_writeInt(motor, TMC4670_INTERIM_ADDR, 19);
			*value = (u16) tmc4670_readRegister16BitValue(motor, TMC4670_INTERIM_DATA, BIT_0_TO_15);
		} else if(readWrite == WRITE) {
			errors |= ERROR_TYPE;
		}
		break;
	case 218:
		// \xml <tmc:axis_parameter type="218" name="ADC_I_1" identifier="par::ADC_I_1">
		// \xml <desc></desc>
		// \xml <tmc:value_unsigned min="0" max="#u16_MAX#" default="0" unit="" access="R"></tmc:value_unsigned>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			tmc4670_writeInt(motor, TMC4670_INTERIM_ADDR, 19);
			*value = (u16) tmc4670_readRegister16BitValue(motor, TMC4670_INTERIM_DATA, BIT_16_TO_31);
		} else if(readWrite == WRITE) {
			errors |= ERROR_TYPE;
		}
		break;
	case 230:
		// \xml <tmc:axis_parameter type="230" name="DEBUG_VALUE_0" identifier="par::DEBUG_VALUE_0">
		// \xml <desc></desc>
		// \xml <tmc:value_signed min="#s16_MIN#" max="#s16_MAX#" default="0" unit="" access="R"></tmc:value_signed>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			tmc4670_writeInt(motor, TMC4670_INTERIM_ADDR, 192);
			*value = (s16) tmc4670_readRegister16BitValue(motor, TMC4670_INTERIM_DATA, BIT_0_TO_15);
		} else if(readWrite == WRITE) {
			errors |= ERROR_TYPE;
		}
		break;
	case 231:
		// \xml <tmc:axis_parameter type="231" name="DEBUG_VALUE_1" identifier="par::DEBUG_VALUE_1">
		// \xml <desc></desc>
		// \xml <tmc:value_signed min="#s16_MIN#" max="#s16_MAX#" default="0" unit="" access="R"></tmc:value_signed>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			tmc4670_writeInt(motor, TMC4670_INTERIM_ADDR, 192);
			*value = (s16) tmc4670_readRegister16BitValue(motor, TMC4670_INTERIM_DATA, BIT_16_TO_31);
		} else if(readWrite == WRITE) {
			errors |= ERROR_TYPE;
		}
		break;
	case 232:
		// \xml <tmc:axis_parameter type="232" name="DEBUG_VALUE_2" identifier="par::DEBUG_VALUE_2">
		// \xml <desc></desc>
		// \xml <tmc:value_signed min="#s16_MIN#" max="#s16_MAX#" default="0" unit="" access="R"></tmc:value_signed>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			tmc4670_writeInt(motor, TMC4670_INTERIM_ADDR, 193);
			*value = (s16) tmc4670_readRegister16BitValue(motor, TMC4670_INTERIM_DATA, BIT_0_TO_15);
		} else if(readWrite == WRITE) {
			errors |= ERROR_TYPE;
		}
		break;
	case 233:
		// \xml <tmc:axis_parameter type="233" name="DEBUG_VALUE_3" identifier="par::DEBUG_VALUE_3">
		// \xml <desc></desc>
		// \xml <tmc:value_signed min="#s16_MIN#" max="#s16_MAX#" default="0" unit="" access="R"></tmc:value_signed>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			tmc4670_writeInt(motor, TMC4670_INTERIM_ADDR, 193);
			*value = (s16) tmc4670_readRegister16BitValue(motor, TMC4670_INTERIM_DATA, BIT_16_TO_31);
		} else if(readWrite == WRITE) {
			errors |= ERROR_TYPE;
		}
		break;
	case 234:
		// \xml <tmc:axis_parameter type="234" name="DEBUG_VALUE_4" identifier="par::DEBUG_VALUE_4">
		// \xml <desc></desc>
		// \xml <tmc:value_signed min="#s16_MIN#" max="#s16_MAX#" default="0" unit="" access="R"></tmc:value_signed>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			tmc4670_writeInt(motor, TMC4670_INTERIM_ADDR, 194);
			*value = (s16) tmc4670_readRegister16BitValue(motor, TMC4670_INTERIM_DATA, BIT_0_TO_15);
		} else if(readWrite == WRITE) {
			errors |= ERROR_TYPE;
		}
		break;
	case 235:
		// \xml <tmc:axis_parameter type="235" name="DEBUG_VALUE_5" identifier="par::DEBUG_VALUE_5">
		// \xml <desc></desc>
		// \xml <tmc:value_signed min="#s16_MIN#" max="#s16_MAX#" default="0" unit="" access="R"></tmc:value_signed>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			tmc4670_writeInt(motor, TMC4670_INTERIM_ADDR, 194);
			*value = (s16) tmc4670_readRegister16BitValue(motor, TMC4670_INTERIM_DATA, BIT_16_TO_31);
		} else if(readWrite == WRITE) {
			errors |= ERROR_TYPE;
		}
		break;
	case 236:
		// \xml <tmc:axis_parameter type="236" name="DEBUG_VALUE_6" identifier="par::DEBUG_VALUE_6">
		// \xml <desc></desc>
		// \xml <tmc:value_signed min="#s16_MIN#" max="#s16_MAX#" default="0" unit="" access="R"></tmc:value_signed>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			tmc4670_writeInt(motor, TMC4670_INTERIM_ADDR, 195);
			*value = (s16) tmc4670_readRegister16BitValue(motor, TMC4670_INTERIM_DATA, BIT_0_TO_15);
		} else if(readWrite == WRITE) {
			errors |= ERROR_TYPE;
		}
		break;
	case 237:
		// \xml <tmc:axis_parameter type="237" name="DEBUG_VALUE_7" identifier="par::DEBUG_VALUE_7">
		// \xml <desc></desc>
		// \xml <tmc:value_signed min="#s16_MIN#" max="#s16_MAX#" default="0" unit="" access="R"></tmc:value_signed>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			tmc4670_writeInt(motor, TMC4670_INTERIM_ADDR, 195);
			*value = (s16) tmc4670_readRegister16BitValue(motor, TMC4670_INTERIM_DATA, BIT_16_TO_31);
		} else if(readWrite == WRITE) {
			errors |= ERROR_TYPE;
		}
		break;
	case 238:
		// \xml <tmc:axis_parameter type="238" name="DEBUG_VALUE_8" identifier="par::DEBUG_VALUE_8">
		// \xml <desc></desc>
		// \xml <tmc:value_unsigned min="0" max="#u16_MAX#" default="0" unit="" access="R"></tmc:value_unsigned>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			tmc4670_writeInt(motor, TMC4670_INTERIM_ADDR, 196);
			*value = (u16) tmc4670_readRegister16BitValue(motor, TMC4670_INTERIM_DATA, BIT_0_TO_15);
		} else if(readWrite == WRITE) {
			errors |= ERROR_TYPE;
		}
		break;
	case 239:
		// \xml <tmc:axis_parameter type="239" name="DEBUG_VALUE_9" identifier="par::DEBUG_VALUE_9">
		// \xml <desc></desc>
		// \xml <tmc:value_unsigned min="0" max="#u16_MAX#" default="0" unit="" access="R"></tmc:value_unsigned>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			tmc4670_writeInt(motor, TMC4670_INTERIM_ADDR, 196);
			*value = (u16) tmc4670_readRegister16BitValue(motor, TMC4670_INTERIM_DATA, BIT_16_TO_31);
		} else if(readWrite == WRITE) {
			errors |= ERROR_TYPE;
		}
		break;
	case 240:
		// \xml <tmc:axis_parameter type="240" name="DEBUG_VALUE_10" identifier="par::DEBUG_VALUE_10">
		// \xml <desc></desc>
		// \xml <tmc:value_unsigned min="0" max="#u16_MAX#" default="0" unit="" access="R"></tmc:value_unsigned>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			tmc4670_writeInt(motor, TMC4670_INTERIM_ADDR, 197);
			*value = (u16) tmc4670_readRegister16BitValue(motor, TMC4670_INTERIM_DATA, BIT_0_TO_15);
		} else if(readWrite == WRITE) {
			errors |= ERROR_TYPE;
		}
		break;
	case 241:
		// \xml <tmc:axis_parameter type="241" name="DEBUG_VALUE_11" identifier="par::DEBUG_VALUE_11">
		// \xml <desc></desc>
		// \xml <tmc:value_unsigned min="0" max="#u16_MAX#" default="0" unit="" access="R"></tmc:value_unsigned>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			tmc4670_writeInt(motor, TMC4670_INTERIM_ADDR, 197);
			*value = (u16) tmc4670_readRegister16BitValue(motor, TMC4670_INTERIM_DATA, BIT_16_TO_31);
		} else if(readWrite == WRITE) {
			errors |= ERROR_TYPE;
		}
		break;
	case 242:
		// \xml <tmc:axis_parameter type="242" name="DEBUG_VALUE_12" identifier="par::DEBUG_VALUE_12">
		// \xml <desc></desc>
		// \xml <tmc:value_unsigned min="0" max="#u16_MAX#" default="0" unit="" access="R"></tmc:value_unsigned>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			tmc4670_writeInt(motor, TMC4670_INTERIM_ADDR, 198);
			*value = (u16) tmc4670_readRegister16BitValue(motor, TMC4670_INTERIM_DATA, BIT_0_TO_15);
		} else if(readWrite == WRITE) {
			errors |= ERROR_TYPE;
		}
		break;
	case 243:
		// \xml <tmc:axis_parameter type="243" name="DEBUG_VALUE_13" identifier="par::DEBUG_VALUE_13">
		// \xml <desc></desc>
		// \xml <tmc:value_unsigned min="0" max="#u16_MAX#" default="0" unit="" access="R"></tmc:value_unsigned>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			tmc4670_writeInt(motor, TMC4670_INTERIM_ADDR, 198);
			*value = (u16) tmc4670_readRegister16BitValue(motor, TMC4670_INTERIM_DATA, BIT_16_TO_31);
		} else if(readWrite == WRITE) {
			errors |= ERROR_TYPE;
		}
		break;
	case 244:
		// \xml <tmc:axis_parameter type="244" name="DEBUG_VALUE_14" identifier="par::DEBUG_VALUE_14">
		// \xml <desc></desc>
		// \xml <tmc:value_unsigned min="0" max="#u16_MAX#" default="0" unit="" access="R"></tmc:value_unsigned>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			tmc4670_writeInt(motor, TMC4670_INTERIM_ADDR, 199);
			*value = (u16) tmc4670_readRegister16BitValue(motor, TMC4670_INTERIM_DATA, BIT_0_TO_15);
		} else if(readWrite == WRITE) {
			errors |= ERROR_TYPE;
		}
		break;
	case 245:
		// \xml <tmc:axis_parameter type="245" name="DEBUG_VALUE_15" identifier="par::DEBUG_VALUE_15">
		// \xml <desc></desc>
		// \xml <tmc:value_unsigned min="0" max="#u16_MAX#" default="0" unit="" access="R"></tmc:value_unsigned>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			tmc4670_writeInt(motor, TMC4670_INTERIM_ADDR, 199);
			*value = (u16) tmc4670_readRegister16BitValue(motor, TMC4670_INTERIM_DATA, BIT_16_TO_31);
		} else if(readWrite == WRITE) {
			errors |= ERROR_TYPE;
		}
		break;
	case 246:
		// \xml <tmc:axis_parameter type="246" name="DEBUG_VALUE_16" identifier="par::DEBUG_VALUE_16">
		// \xml <desc></desc>
		// \xml <tmc:value_signed min="#s32_MIN#" max="#s32_MAX#" default="0" unit="" access="R"></tmc:value_signed>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			tmc4670_writeInt(motor, TMC4670_INTERIM_ADDR, 200);
			*value =  (s32)tmc4670_readInt(motor, TMC4670_INTERIM_DATA);
		} else if(readWrite == WRITE) {
			errors |= ERROR_TYPE;
		}
		break;
	case 247:
		// \xml <tmc:axis_parameter type="247" name="DEBUG_VALUE_17" identifier="par::DEBUG_VALUE_17">
		// \xml <desc></desc>
		// \xml <tmc:value_signed min="#s32_MIN#" max="#s32_MAX#" default="0" unit="" access="R"></tmc:value_signed>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			tmc4670_writeInt(motor, TMC4670_INTERIM_ADDR, 201);
			*value =  (s32)tmc4670_readInt(motor, TMC4670_INTERIM_DATA);
		} else if(readWrite == WRITE) {
			errors |= ERROR_TYPE;
		}
		break;
	case 248:
		// \xml <tmc:axis_parameter type="248" name="DEBUG_VALUE_18" identifier="par::DEBUG_VALUE_18">
		// \xml <desc></desc>
		// \xml <tmc:value_signed min="#s32_MIN#" max="#s32_MAX#" default="0" unit="" access="R"></tmc:value_signed>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			tmc4670_writeInt(motor, TMC4670_INTERIM_ADDR, 202);
			*value =  (s32) tmc4670_readInt(motor, TMC4670_INTERIM_DATA);
		} else if(readWrite == WRITE) {
			errors |= ERROR_TYPE;
		}
		break;
	case 249:
		// \xml <tmc:axis_parameter type="249" name="DEBUG_VALUE_19" identifier="par::DEBUG_VALUE_19">
		// \xml <desc></desc>
		// \xml <tmc:value_signed min="#s32_MIN#" max="#s32_MAX#" default="0" unit="" access="R"></tmc:value_signed>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			tmc4670_writeInt(motor, TMC4670_INTERIM_ADDR, 203);
			*value =  (s32) tmc4670_readInt(motor, TMC4670_INTERIM_DATA);
		} else if(readWrite == WRITE) {
			errors |= ERROR_TYPE;
		}
		break;
	case 250:
		// \xml <tmc:axis_parameter type="250" name="Ram measure type" identifier="par::ramMeasureType">
		// \xml <desc></desc>
		// \xml <tmc:value_signed min="0" max="#s16_MAX#" default="0" unit="" access="RWX"></tmc:value_signed>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			*value = ramMeasureType;
		} else if(readWrite == WRITE) {
			ramMeasureType = *value;
		}
		break;
	case 251:
		// \xml <tmc:axis_parameter type="251" name="Torque measurement factor" identifier="par::torqueMeasurementFactor">
		// \xml <desc>Manually adjust the dual shunt current measurement factor (s24.s8).</desc>
		// \xml <tmc:value_unsigned min="0" max="#s32_MAX#" default="256" unit="" access="RW"></tmc:value_unsigned>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			*value = motorConfig[motor].torqueMeasurementFactor;
		} else if(readWrite == WRITE) {
			motorConfig[motor].torqueMeasurementFactor = *value;
		}
		break;
	case 252:
		// \xml <tmc:axis_parameter type="252" name="Start encoder initialization" identifier="par::StartEncoderInitialization">
		// \xml <desc></desc>
		// \xml <tmc:value_unsigned min="0" max="2" default="0" unit="" access="RW"></tmc:value_unsigned>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			*value = motorConfig[motor].initMode;
		} else if(readWrite == WRITE) {
			tmc4670_startEncoderInitialization(*value, &motorConfig[motor].initMode, &motorConfig[motor].initState);
		}
		break;
	case 253:
		// \xml <tmc:axis_parameter type="253" name="Encoder init state" identifier="par::EncoderInitState">
		// \xml <desc></desc>
		// \xml <tmc:value_unsigned min="0" max="2" default="0" unit="" access="RW"></tmc:value_unsigned>
		// \xml </tmc:axis_parameter>	// todo CHECK AP 2: is this RW? If yes, implement, if its R, uncomment the errors line (LH) #1
		if(readWrite == READ) {
			*value = motorConfig[motor].initState;
		} else if(readWrite == WRITE) {
			//errors |= ERROR_TYPE;
		}
		break;
	case 254:
		// \xml <tmc:axis_parameter type="254" name="Actual encoder wait time" identifier="par::ActualEncoderWaitTime">
		// \xml <desc></desc>
		// \xml <tmc:value_unsigned min="0" max="#u16_MAX#" default="0" unit="" access="R"></tmc:value_unsigned>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			*value = motorConfig[motor].actualInitWaitTime;
		} else if(readWrite == WRITE) {
			errors |= ERROR_TYPE;
		}
		break;
	case 255:
		// \xml <tmc:axis_parameter type="255" name="Main loop counter" identifier="par::MainLoopCounter">
		// \xml    <desc></desc>
		// \xml    <tmc:value_unsigned min="0" max="#u32_MAX#" default="0" unit="" access="R"></tmc:value_unsigned>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			*value = mainLoopCount;
		} else if(readWrite == WRITE) {
			errors |= ERROR_TYPE;
		}
		break;
	default:
		errors |= ERROR_TYPE;
		break;
	}
	// \xml </axis_parameters>
	return errors;
}
// \xml </tmc:product_module>

static uint32 getMeasuredSpeed(uint8 motor, int32 *value)
{
	if(motor >= MOTORS)
		return ERROR_MOTOR;

	*value = tmc4670_getActualVelocity(motor);

	return ERROR_NONE;
}

static void periodicJob(uint32 actualSystick)
{
	static u32 loopCounterCheckTime;
	static u32 loopCounter = 0;
	RamBufferElement element = { 0 };

	if((actualSystick - loopCounterCheckTime) >= 10)
	{
		mainLoopCount = loopCounter*100;
		loopCounter = 0;

		loopCounterCheckTime = actualSystick;
	}
	loopCounter++;

	// do encoder initialization if necessary
	int motor;
	for(motor = 0; motor < MOTORS; motor++)
	{
		tmc4670_periodicJob(motor, actualSystick, motorConfig[motor].initMode,
				&(motorConfig[motor].initState), motorConfig[motor].initWaitTime,
				&(motorConfig[motor].actualInitWaitTime), motorConfig[motor].startVoltage);
	}

	// is ram debug waiting for trigger?
	if(debug_isWaitingForTrigger())
		debug_checkTriggerCondition(0, 0, 0);

	// ram debug measurement
	if(debug_isRamDebuggingEnabled())
	{
		switch (debug_getRamMeasureMode())
		{
		case RAM_DEBUG_MEASURE_POSITION:
			element.item[0] = tmc4670_getTargetPosition(DEFAULT_MOTOR);	// target position
			element.item[1] = tmc4670_getActualPosition(DEFAULT_MOTOR);	// actual position
			debug_addRamDebugElement(element);
			break;
		case RAM_DEBUG_MEASURE_VELOCITY:
			element.item[0] = tmc4670_getTargetVelocity(DEFAULT_MOTOR);	// target velocity
			element.item[1] = tmc4670_getActualVelocity(DEFAULT_MOTOR);	// actual velocity
			debug_addRamDebugElement(element);
			break;
		case RAM_DEBUG_MEASURE_TORQUE:
			element.item[0] = tmc4670_getTargetTorque_mA(DEFAULT_MOTOR, motorConfig[DEFAULT_MOTOR].torqueMeasurementFactor);  // target torque_mA
			element.item[1] = tmc4670_getActualTorque_mA(DEFAULT_MOTOR, motorConfig[DEFAULT_MOTOR].torqueMeasurementFactor);  // actual torque_mA
			debug_addRamDebugElement(element);
			break;
		case RAM_DEBUG_MEASURE_DEFINED:
			switch (ramMeasureType) // todo REM 2: kann weg? (LH)
			{
			case 0:
				// do nothing
				element.item[0] = 10;
				element.item[1] = 20;
				element.item[2] = 30;
				element.item[3] = 40;
				break;
			case 1:
				// do nothing
				element.item[0] = 100;
				element.item[1] = 200;
				element.item[2] = 300;
				element.item[3] = 400;
				break;
			}
			debug_addRamDebugElement(element);
			break;
		}
	}
}

static void writeRegister(uint8 address, int32 value)
{
	tmc4670_writeInt(DEFAULT_MOTOR, address, value);
}

static void readRegister(uint8 address, int32 *value)
{
	*value = tmc4670_readInt(DEFAULT_MOTOR, address);
}

static uint32 SAP(uint8 type, uint8 motor, int32 value)
{
	return handleParameter(WRITE, motor, type, &value);
}

static uint32 GAP(uint8 type, uint8 motor, int32 *value)
{
	return handleParameter(READ, motor, type, value);
}

static uint32 userFunction(uint8 type, uint8 motor, int32 *value)
{
	UNUSED(type);
	UNUSED(motor);
	UNUSED(value);
	return 0;
}

static void enableDriver(DriverState state)
{
	if(state == DRIVER_USE_GLOBAL_ENABLE)
		state = Evalboards.driverEnable;

	if(state == DRIVER_DISABLE)
		HAL.IOs->config->setLow(PIN_DRV_ENN);
	else if((state == DRIVER_ENABLE) && (Evalboards.driverEnable == DRIVER_ENABLE))
		HAL.IOs->config->setHigh(PIN_DRV_ENN);
}

static void deInit(void)
{
	enableDriver(DRIVER_DISABLE);
	HAL.IOs->config->setLow(PIN_DRV_ENN);
};

static uint8 reset()
{
	return 1;
}

static uint8 restore()
{
	return 1;
}

static void checkErrors(uint32 tick)
{
	UNUSED(tick);
	Evalboards.ch1.errors = 0;
}

void TMC4670_init(void)
{
	// configure ENABLE-PIN for TMC4670
	PIN_DRV_ENN = &HAL.IOs->pins->DIO0;
	HAL.IOs->config->toOutput(PIN_DRV_ENN);
	HAL.IOs->config->setHigh(PIN_DRV_ENN);

	TMC4670_config = Evalboards.ch1.config;

	// connect evalboard functions
	Evalboards.ch1.config->reset        = reset;
	Evalboards.ch1.config->restore      = restore;
	Evalboards.ch1.config->state        = CONFIG_READY;
	Evalboards.ch1.config->configIndex  = 0;

	Evalboards.ch1.rotate               = rotate;
	Evalboards.ch1.right                = right;
	Evalboards.ch1.left                 = left;
	Evalboards.ch1.stop                 = stop;
	Evalboards.ch1.getMeasuredSpeed     = getMeasuredSpeed;
	Evalboards.ch1.GAP                  = GAP;
	Evalboards.ch1.SAP                  = SAP;
	Evalboards.ch1.moveTo               = moveTo;
	Evalboards.ch1.moveBy               = moveBy;
	Evalboards.ch1.writeRegister        = writeRegister;
	Evalboards.ch1.readRegister         = readRegister;
	Evalboards.ch1.periodicJob          = periodicJob;
	Evalboards.ch1.userFunction         = userFunction;
	Evalboards.ch1.enableDriver         = enableDriver;
	Evalboards.ch1.checkErrors          = checkErrors;
	Evalboards.ch1.numberOfMotors       = MOTORS;
	Evalboards.ch1.deInit               = deInit;
	Evalboards.ch1.VMMin                = 50;
	Evalboards.ch1.VMMax                = 650;

	// init motor config
	for(int i = 0; i < MOTORS; i++)
	{
		motorConfig[i].initWaitTime             = 1000;
		motorConfig[i].startVoltage             = 6000;
		motorConfig[i].initMode                 = 0;
		motorConfig[i].torqueMeasurementFactor  = 256;
	}

	// enable the driver
	enableDriver(DRIVER_ENABLE);
};
