/*
 * TMC8690_eval.c
 *
 *  Created on: 18.07.2016
 *      Author: bs, ed
 */

#include "Board.h"
#include "tmc/ic/TMC8690/TMC8690.h"

#include <math.h> // For M_PI

#define VM_MIN  50   // VM[V/10] min
#define VM_MAX  280  // VM[V/10] max +10%

#define MOTORS 1 // number of motors for this board

IOPinTypeDef *PIN_DRV_ENN;
ConfigurationTypeDef *TMC8690_config;

static u32 mainLoopCount = 0;

// => SPI wrapper
int tmc8690_readInt(u8 motor, uint8 address)
{
	UNUSED(motor);
	return spi_ch1_readInt(address);
}

void tmc8690_writeInt(u8 motor, uint8 address, int value)
{
	UNUSED(motor);
	spi_ch1_writeInt(address, value);
}

u16 tmc8690_spi_readRegister16BitValue(u8 address, u8 channel)
{
	return spi_ch1_readRegister16BitValue(address, channel);
}

void tmc8690_spi_writeRegister16BitValue(u8 address, u8 channel, u16 value)
{
	spi_ch1_writeRegister16BitValue(address, channel, value);
}
// <= SPI wrapper

// Efficiency Measurement
typedef union {
	int i;
	float f;
} f32;

uint32 uSupply;
int iSupplyOffset;
f32 iSupplyGain;
int torqueOffset;
f32 torqueGain;
f32 efficiencyInPercent;

void TMC8690_calcEfficiency()
{
	//			 Pout		 Pmech		AngularSpeed * Torque
	// Eff	=	______	=	_______	=	_____________________
	//			 Pin		 Pelec		  Vsupply * Isupply
	//
	// AngularSpeed = 2 * PI * MeasuredSpeed[rpm] / 60
	//
	// Torque = (TorqueADCValue - TorqueOffset) * TorqueGain
	// ISupply = (ISupplyADCValue - ISupplyOffset) * ISupplyGain
	//
	f32 torque, iSupply, omega;

	iSupply.f = ((s32) tmc8690_readInt(0, ADC_I_SUPPLY_FILT) - iSupplyOffset) * iSupplyGain.f;

	if(iSupply.f == 0 || uSupply == 0) { // avoid division by 0
		efficiencyInPercent.f = 0;
		return;
	}

	torque.f = abs(tmc8690_spi_readRegister16BitValue(ADC_TORQUE, BIT_0_TO_15) - torqueOffset) * torqueGain.f;
	omega.f	= 2.0 * M_PI * (float) abs(tmc8690_readInt(0, VELOCITY_IN_RPM_ACTUAL_PT1)) / 65536 / 60; // 65536 = 2^12 // todo CHECK 3: Tippfehler oder Wertefehler? (2^12 = 4096, nicht 65536) (LH)
	efficiencyInPercent.f = torque.f * omega.f / iSupply.f / (uSupply / 10); // USupply in [0,100 V] must be divided by 10
}

// \xml <?xml version="1.0" encoding="UTF-8" standalone="no"?>
// \xml <tmc:product_module author="?" xmlns:tmc="http://www.trinamic.com" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xsi:schemaLocation="http://www.trinamic.com https://www.trinamic.com/fileadmin/xml/TMC_Schema.xsd">
// \xml <name>TMC8690-EVAL</name>
// \xml <name_fw_version>8690</name_fw_version>
// \xml <hardware_version>1.0</hardware_version>
// \xml <software_version>1.0</software_version>

static uint32 handleParameter(u8 readWrite, u8 motor, u8 type, int32 *value)
{
	u32 errors = ERROR_NONE;

	if(motor >= MOTORS)
		return ERROR_MOTOR;

	// \xml <axis_parameters axis="0">
	switch(type)
	{
	// Efficiency Measurement
	case 20:
		// \xml <tmc:axis_parameter type="20" name="U_Supply" identifier="par::U_Supply">
		// \xml   <desc>Temporal U_Supply variable.</desc>
		// \xml   <tmc:value_unsigned min="0" max="#s32_MAX#" default="0" unit="[100mV]" access="RW"></tmc:value_unsigned>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ)
			*value = uSupply;
		else if(readWrite == WRITE)
			uSupply = *value;
		break;
	case 21:
		// \xml <tmc:axis_parameter type="21" name="I_Supply_Offset" identifier="par::I_Supply_Offset">
		// \xml   <desc>Temporal I_Supply_Offset variable.</desc>
		// \xml   <tmc:value_signed min="#s32_MIN#" max="#s32_MAX#" default="0" unit="" access="RW"></tmc:value_signed>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ)
			*value = iSupplyOffset;
		else if(readWrite == WRITE)
			iSupplyOffset = *value;
		break;
	case 22:
		// \xml <tmc:axis_parameter type="22" name="I_Supply_Gain" identifier="par::I_Supply_Gain">
		// \xml   <desc>Temporal I_Supply_Gain variable.</desc>
		// \xml   <tmc:value_unsigned min="0" max="#s32_MAX#" default="0" unit="" access="RW"></tmc:value_unsigned>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ)
			*value = iSupplyGain.i;
		else if(readWrite == WRITE)
			iSupplyGain.i = *value;
		break;
	case 24:
		// \xml <tmc:axis_parameter type="24" name="Torque_Offset" identifier="par::Torque_Offset">
		// \xml   <desc>Temporal Torque_Offset variable.</desc>
		// \xml   <tmc:value_signed min="#s32_MIN#" max="#s32_MAX#" default="0" unit="" access="RW"></tmc:value_signed>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ)
			*value = torqueOffset;
		else if(readWrite == WRITE)
			torqueOffset = *value;
		break;
	case 25:
		// \xml <tmc:axis_parameter type="25" name="Torque_Gain" identifier="par::Torque_Gain">
		// \xml   <desc>Temporal Torque_Gain variable.</desc>
		// \xml   <tmc:value_unsigned min="0" max="#s32_MAX#" default="0" unit="" access="RW"></tmc:value_unsigned>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ)
			*value = torqueGain.i;
		else if(readWrite == WRITE)
			torqueGain.i = *value;
		break;
	case 26:
		// \xml <tmc:axis_parameter type="26" name="Efficiency_in_percent_F32" identifier="par::Efficiency_in_percent_F32">
		// \xml <desc>Efficiency shown as a percentage in float 32 format.</desc>
		// \xml   <tmc:value_unsigned min="0" max="#s32_MAX#" default="0" unit="" access="R"></tmc:value_unsigned>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ)
		{
			TMC8690_calcEfficiency();
			*value = efficiencyInPercent.i;
		}
		else if(readWrite == WRITE)
		{
			errors |= ERROR_TYPE;
		}
		break;

	case 27:
		// \xml <tmc:axis_parameter type="27" name="POSITION_ACTUAL_16" identifier="par::POSITION_ACTUAL_16">
		// \xml <desc>16 upper bits of Register 38 with name POSITION_ACTUAL of TMC8690.</desc>
		// \xml   <tmc:value_unsigned min="0" max="#u16_MAX#" default="0" unit="" access="R"></tmc:value_unsigned>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ)
			*value = (s32) tmc8690_spi_readRegister16BitValue(POSITION_ACTUAL, BIT_16_TO_31);
		else if(readWrite == WRITE)
			errors |= ERROR_TYPE;
		break;
#ifdef ENABLE_IC_SCOPE
	case 39:  // OSCI_status
		if(readWrite == READ)
		{
			*value = (s16) tmc8690_spi_readRegister16BitValue(OSCI_STATUS, BIT_0_TO_15);
		}
		else if(readWrite == WRITE)
		{
			errors |= ERROR_TYPE;
		}
		break;
	case 40:  // trigger_channel
		if(readWrite == READ)
		{
			*value = TMC8690_FIELD_READ(motor, OSCI_MAIN_CONFIG_REG, TMC8690_TRG_CHN_MASK, TMC8690_TRG_CHN_SHIFT);
		}
		else if(readWrite == WRITE)
		{
			TMC8690_FIELD_UPDATE(motor, OSCI_MAIN_CONFIG_REG, TMC8690_TRG_CHN_MASK, TMC8690_TRG_CHN_SHIFT, *value);
		}
		break;
	case 41:  // trigger_condition
		if(readWrite == READ)
		{
			*value = TMC8690_FIELD_READ(motor, OSCI_MAIN_CONFIG_REG, TMC8690_TRG_CND_MASK, TMC8690_TRG_CND_SHIFT);
		}
		else if(readWrite == WRITE)
		{
			TMC8690_FIELD_UPDATE(motor, OSCI_MAIN_CONFIG_REG, TMC8690_TRG_CND_MASK, TMC8690_TRG_CND_SHIFT, *value);
		}
		break;
	case 42:  // trigger_value
		if(readWrite == READ)
		{
			*value = TMC8690_FIELD_READ(motor, OSCI_MAIN_CONFIG_REG, TMC8690_TRG_VAL_MASK, TMC8690_TRG_VAL_SHIFT);
		}
		else if(readWrite == WRITE)
		{
			TMC8690_FIELD_UPDATE(motor, OSCI_MAIN_CONFIG_REG, TMC8690_TRG_VAL_MASK, TMC8690_TRG_VAL_SHIFT, *value);
		}
		break;
	case 43:  // OSR
		if(readWrite == READ)
		{
			*value = TMC8690_FIELD_READ(motor, OSCI_MAIN_CONFIG_REG, TMC8690_OSR_MASK, TMC8690_OSR_SHIFT);
		}
		else if(readWrite == WRITE)
		{
			TMC8690_FIELD_UPDATE(motor, OSCI_MAIN_CONFIG_REG, TMC8690_OSR_MASK, TMC8690_OSR_SHIFT, *value);
		}
		break;
	case 44:  // DSR
		if(readWrite == READ)
		{
			*value = TMC8690_FIELD_READ(motor, OSCI_MAIN_CONFIG_REG, TMC8690_DSR_MASK, TMC8690_DSR_SHIFT);
		}
		else if(readWrite == WRITE)
		{
			TMC8690_FIELD_UPDATE(motor, OSCI_MAIN_CONFIG_REG, TMC8690_DSR_MASK, TMC8690_DSR_SHIFT, *value);
		}
		break;
	case 45:  // enable/disable measurement
		if(readWrite == READ)
		{
			*value = TMC8690_FIELD_READ(motor, OSCI_MAIN_CONFIG_REG, TMC8690_MEAS_EN_MASK, TMC8690_MEAS_EN_SHIFT);
		}
		else if(readWrite == WRITE)
		{
			TMC8690_FIELD_UPDATE(motor, OSCI_MAIN_CONFIG_REG, TMC8690_MEAS_EN_MASK, TMC8690_MEAS_EN_SHIFT, *value);
		}
		break;
	case 46:  // clear measurement
		if(readWrite == READ)
		{
			*value = TMC8690_FIELD_READ(motor, OSCI_MAIN_CONFIG_REG, TMC8690_MEAS_CLR_MASK, TMC8690_MEAS_CLR_SHIFT);
		}
		else if(readWrite == WRITE)
		{
			TMC8690_FIELD_UPDATE(motor, OSCI_MAIN_CONFIG_REG, TMC8690_MEAS_CLR_MASK, TMC8690_MEAS_CLR_SHIFT, *value);
		}
		break;
	case 47:  // Pretrigger_conf
		if(readWrite == READ)
			*value = (s16) tmc8690_spi_readRegister16BitValue(OSCI_PRETRIGGER_CONFIG_REG, BIT_0_TO_15);
		else if(readWrite == WRITE)
			tmc8690_spi_writeRegister16BitValue(OSCI_PRETRIGGER_CONFIG_REG, BIT_0_TO_15, *value);
		break;
	case 48:	// data
		if(readWrite == READ)
		{
			*value = tmc8690_readInt(motor, OSCI_OUTPUT_DATA);
		}
		else if(readWrite == WRITE)
			errors |= ERROR_TYPE;
		break;
	case 49:  // data address // todo AP 2: shifte die restlichen Adressen nach oben #1
		if(readWrite == READ)
			*value = (u16) tmc8690_spi_readRegister16BitValue(OSCI_DATA_ADDRESS, BIT_0_TO_15);
		else if(readWrite == WRITE)
			tmc8690_spi_writeRegister16BitValue(OSCI_DATA_ADDRESS, BIT_0_TO_15, *value);
		break;
#endif /* ifdef ENABLE_IC_SCOPE */
	case 51:
		// \xml <tmc:axis_parameter type="51" name="CHIPINFO_DATA" identifier="par::CHIPINFO_DATA">
		// \xml <desc>Register 00 with name CHIPINFO_DATA of TMC8690.</desc>
		// \xml   <tmc:value_unsigned min="0" max="#u16_MAX#" default="0" unit="" access="R"></tmc:value_unsigned>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ)
			*value = (u16) tmc8690_spi_readRegister16BitValue(CHIPINFO_DATA, BIT_0_TO_15);
		else if(readWrite == WRITE)
			errors |= ERROR_TYPE;
		break;
	case 52:
		// \xml <tmc:axis_parameter type="52" name="CHIPINFO_ADDR" identifier="par::CHIPINFO_ADDR">
		// \xml <desc>Register 01 with name CHIPINFO_ADDR of TMC8690.</desc>
		// \xml   <tmc:value_unsigned min="0" max="#u16_MAX#" default="0" unit="" access="RW"></tmc:value_unsigned>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ)
			*value = (u16) tmc8690_spi_readRegister16BitValue(CHIPINFO_ADDR, BIT_0_TO_15);
		else if(readWrite == WRITE)
			tmc8690_spi_writeRegister16BitValue(CHIPINFO_ADDR, BIT_0_TO_15, *value);
		break;
	case 57:
		// \xml <tmc:axis_parameter type="57" name="STATUS_WORD" identifier="par::STATUS_WORD">
		// \xml <desc>Register 02 with name STATUS_WORD of TMC8690.</desc>
		// \xml   <tmc:value_unsigned min="0" max="#u32_MAX#" default="0" unit="" access="R"></tmc:value_unsigned>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ)
			*value = (s32) tmc8690_readInt(motor, STATUS_WORD);
		else if(readWrite == WRITE)
			errors |= ERROR_TYPE;
		break;
	case 58:
		// \xml <tmc:axis_parameter type="58" name="COMMAND_WORD" identifier="par::COMMAND_WORD">
		// \xml <desc>Register 03 with name COMMAND_WORD of TMC8690.</desc>
		// \xml   <tmc:value_unsigned min="0" max="#u32_MAX#" default="0" unit="" access="RW"></tmc:value_unsigned>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ)
			*value = (s32) tmc8690_readInt(motor, COMMAND_WORD);
		else if(readWrite == WRITE)
			tmc8690_writeInt(motor, COMMAND_WORD, *value);
		break;
	case 59:
		// \xml <tmc:axis_parameter type="59" name="P_FAK_BRAKE" identifier="par::P_FAK_BRAKE">
		// \xml <desc>Register 04 with name P_FAK_BRAKE of TMC8690.</desc>
		// \xml   <tmc:value_signed min="#s16_MIN#" max="#s16_MAX#" default="0" unit="" access="RW"></tmc:value_signed>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ)
			*value = (s16) tmc8690_spi_readRegister16BitValue(P_FAK_BRAKE, BIT_0_TO_15);
		else if(readWrite == WRITE)
			tmc8690_spi_writeRegister16BitValue(P_FAK_BRAKE, BIT_0_TO_15, *value);
		break;
	case 60:
		// \xml <tmc:axis_parameter type="60" name="I_FAK_BRAKE" identifier="par::I_FAK_BRAKE">
		// \xml <desc>Register 05 with name I_FAK_BRAKE of TMC8690.</desc>
		// \xml   <tmc:value_signed min="#s16_MIN#" max="#s16_MAX#" default="0" unit="" access="RW"></tmc:value_signed>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ)
			*value = (s16) tmc8690_spi_readRegister16BitValue(I_FAK_BRAKE, BIT_0_TO_15);
		else if(readWrite == WRITE)
			tmc8690_spi_writeRegister16BitValue(I_FAK_BRAKE, BIT_0_TO_15, *value);
		break;
	case 61:
		// \xml <tmc:axis_parameter type="61" name="P_FAK_HEAT" identifier="par::P_FAK_HEAT">
		// \xml <desc>Register 06 with name P_FAK_HEAT of TMC8690.</desc>
		// \xml   <tmc:value_signed min="#s16_MIN#" max="#s16_MAX#" default="0" unit="" access="RW"></tmc:value_signed>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ)
			*value = (s16) tmc8690_spi_readRegister16BitValue(P_FAK_HEAT, BIT_0_TO_15);
		else if(readWrite == WRITE)
			tmc8690_spi_writeRegister16BitValue(P_FAK_HEAT, BIT_0_TO_15, *value);
		break;
	case 62:
		// \xml <tmc:axis_parameter type="62" name="I_FAK_HEAT" identifier="par::I_FAK_HEAT">
		// \xml <desc>Register 07 with name I_FAK_HEAT of TMC8690.</desc>
		// \xml   <tmc:value_signed min="#s16_MIN#" max="#s16_MAX#" default="0" unit="" access="RW"></tmc:value_signed>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ)
			*value = (s16) tmc8690_spi_readRegister16BitValue(I_FAK_HEAT, BIT_0_TO_15);
		else if(readWrite == WRITE)
			tmc8690_spi_writeRegister16BitValue(I_FAK_HEAT, BIT_0_TO_15, *value);
		break;
	case 63:
		// \xml <tmc:axis_parameter type="63" name="P_FAK_VEL" identifier="par::P_FAK_VEL">
		// \xml <desc>Register 08 with name P_FAK_VEL of TMC8690.</desc>
		// \xml   <tmc:value_signed min="#s16_MIN#" max="#s16_MAX#" default="0" unit="" access="RW"></tmc:value_signed>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ)
			*value = (s16) tmc8690_spi_readRegister16BitValue(P_FAK_VEL, BIT_0_TO_15);
		else if(readWrite == WRITE)
			tmc8690_spi_writeRegister16BitValue(P_FAK_VEL, BIT_0_TO_15, *value);
		break;
	case 64:
		// \xml <tmc:axis_parameter type="64" name="I_FAK_VEL" identifier="par::I_FAK_VEL">
		// \xml <desc>Register 09 with name I_FAK_VEL of TMC8690.</desc>
		// \xml   <tmc:value_signed min="#s16_MIN#" max="#s16_MAX#" default="0" unit="" access="RW"></tmc:value_signed>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ)
			*value = (s16) tmc8690_spi_readRegister16BitValue(I_FAK_VEL, BIT_0_TO_15);
		else if(readWrite == WRITE)
			tmc8690_spi_writeRegister16BitValue(I_FAK_VEL, BIT_0_TO_15, *value);
		break;
	case 65:
		// \xml <tmc:axis_parameter type="65" name="P_FAK_POS" identifier="par::P_FAK_POS">
		// \xml <desc>Register 0A with name P_FAK_POS of TMC8690.</desc>
		// \xml   <tmc:value_signed min="#s16_MIN#" max="#s16_MAX#" default="0" unit="" access="RW"></tmc:value_signed>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ)
			*value = (s16) tmc8690_spi_readRegister16BitValue(P_FAK_POS, BIT_0_TO_15);
		else if(readWrite == WRITE)
			tmc8690_spi_writeRegister16BitValue(P_FAK_POS, BIT_0_TO_15, *value);
		break;
	case 66:
		// \xml <tmc:axis_parameter type="66" name="ENC_Z_COMP_VAL" identifier="par::ENC_Z_COMP_VAL">
		// \xml <desc>Register 0B with name ENC_Z_COMP_VAL of TMC8690.</desc>
		// \xml   <tmc:value_signed min="#s16_MIN#" max="#s16_MAX#" default="0" unit="" access="RW"></tmc:value_signed>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ)
			*value = (s16) tmc8690_spi_readRegister16BitValue(ENC_Z_COMP_VAL, BIT_0_TO_15);
		else if(readWrite == WRITE)
			tmc8690_spi_writeRegister16BitValue(ENC_Z_COMP_VAL, BIT_0_TO_15, *value);
		break;
	case 67:
		// \xml <tmc:axis_parameter type="67" name="TORQUE_TARGET" identifier="par::TORQUE_TARGET">
		// \xml <desc>Register 0C with name TORQUE_TARGET of TMC8690.</desc>
		// \xml   <tmc:value_signed min="#s16_MIN#" max="#s16_MAX#" default="0" unit="" access="RW"></tmc:value_signed>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ)
			*value = (s16) tmc8690_spi_readRegister16BitValue(TORQUE_TARGET, BIT_0_TO_15);
		else if(readWrite == WRITE)
			tmc8690_spi_writeRegister16BitValue(TORQUE_TARGET, BIT_0_TO_15, *value);
		break;
	case 68:
		// \xml <tmc:axis_parameter type="68" name="VELOCITY_TARGET" identifier="par::VELOCITY_TARGET">
		// \xml <desc>Register 0D with name VELOCITY_TARGET of TMC8690.</desc>
		// \xml   <tmc:value_signed min="#s32_MIN#" max="#s32_MAX#" default="0" unit="" access="RW"></tmc:value_signed>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ)
			*value = (u32) tmc8690_readInt(motor, VELOCITY_TARGET);
		else if(readWrite == WRITE)
			tmc8690_writeInt(motor, VELOCITY_TARGET, *value);
		break;
	case 69:
		// \xml <tmc:axis_parameter type="69" name="POSITION_TARGET" identifier="par::POSITION_TARGET">
		// \xml <desc>Register 0E with name POSITION_TARGET of TMC8690.</desc>
		// \xml   <tmc:value_signed min="#s32_MIN#" max="#s32_MAX#" default="0" unit="" access="RW"></tmc:value_signed>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ)
			*value = (u32) tmc8690_readInt(motor, POSITION_TARGET);
		else if(readWrite == WRITE)
			tmc8690_writeInt(motor, POSITION_TARGET, *value);
		break;
	case 70:
		// \xml <tmc:axis_parameter type="70" name="TEMPERATURE_TARGET" identifier="par::TEMPERATURE_TARGET">
		// \xml <desc>Register 0F with name TEMPERATURE_TARGET of TMC8690.</desc>
		// \xml   <tmc:value_unsigned min="0" max="#u16_MAX#" default="0" unit="" access="RW"></tmc:value_unsigned>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ)
			*value = (u16) tmc8690_spi_readRegister16BitValue(TEMPERATURE_TARGET, BIT_0_TO_15);
		else if(readWrite == WRITE)
			tmc8690_spi_writeRegister16BitValue(TEMPERATURE_TARGET, BIT_0_TO_15, *value);
		break;
	case 71:
		// \xml <tmc:axis_parameter type="71" name="OSCI_MAIN_CONFIG_REG" identifier="par::OSCI_MAIN_CONFIG_REG">
		// \xml <desc>Register 10 with name OSCI_MAIN_CONFIG_REG of TMC8690.</desc>
		// \xml   <tmc:value_unsigned min="0" max="#u32_MAX#" default="0" unit="" access="RW"></tmc:value_unsigned>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ)
			*value = (s32) tmc8690_readInt(motor, OSCI_MAIN_CONFIG_REG);
		else if(readWrite == WRITE)
			tmc8690_writeInt(motor, OSCI_MAIN_CONFIG_REG, *value);
		break;
	case 72:
		// \xml <tmc:axis_parameter type="72" name="OSCI_PRETRIGGER_CONFIG_REG" identifier="par::OSCI_PRETRIGGER_CONFIG_REG">
		// \xml <desc>Register 11 with name OSCI_PRETRIGGER_CONFIG_REG of TMC8690.</desc>
		// \xml   <tmc:value_unsigned min="0" max="#u16_MAX#" default="0" unit="" access="RW"></tmc:value_unsigned>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ)
			*value = (u16) tmc8690_spi_readRegister16BitValue(OSCI_PRETRIGGER_CONFIG_REG, BIT_0_TO_15);
		else if(readWrite == WRITE)
			tmc8690_spi_writeRegister16BitValue(OSCI_PRETRIGGER_CONFIG_REG, BIT_0_TO_15, *value);
		break;
	case 73:
		// \xml <tmc:axis_parameter type="73" name="OSCI_DATA_ADDRESS" identifier="par::OSCI_DATA_ADDRESS">
		// \xml <desc>Register 12 with name OSCI_DATA_ADDRESS of TMC8690.</desc>
		// \xml   <tmc:value_unsigned min="0" max="#u16_MAX#" default="0" unit="" access="RW"></tmc:value_unsigned>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ)
			*value = (u16) tmc8690_spi_readRegister16BitValue(OSCI_DATA_ADDRESS, BIT_0_TO_15);
		else if(readWrite == WRITE)
			tmc8690_spi_writeRegister16BitValue(OSCI_DATA_ADDRESS, BIT_0_TO_15, *value);
		break;
	case 74:
		// \xml <tmc:axis_parameter type="74" name="OSCI_STATUS" identifier="par::OSCI_STATUS">
		// \xml <desc>Register 13 with name OSCI_STATUS of TMC8690.</desc>
		// \xml   <tmc:value_signed min="#s16_MIN#" max="#s16_MAX#" default="0" unit="" access="R"></tmc:value_signed>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ)
			*value = (s16) tmc8690_spi_readRegister16BitValue(OSCI_STATUS, BIT_0_TO_15);
		else if(readWrite == WRITE)
			errors |= ERROR_TYPE;
		break;
	case 75:
		// \xml <tmc:axis_parameter type="75" name="OSCI_OUTPUT_DATA" identifier="par::OSCI_OUTPUT_DATA">
		// \xml <desc>Register 14 with name OSCI_OUTPUT_DATA of TMC8690.</desc>
		// \xml   <tmc:value_signed min="#s16_MIN#" max="#s16_MAX#" default="0" unit="" access="R"></tmc:value_signed>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ)
			*value = (s16) tmc8690_spi_readRegister16BitValue(OSCI_OUTPUT_DATA, BIT_0_TO_15);
		else if(readWrite == WRITE)
			errors |= ERROR_TYPE;
		break;
	case 76:
		// \xml <tmc:axis_parameter type="76" name="ADC_ENC_TRACK_A" identifier="par::ADC_ENC_TRACK_A">
		// \xml <desc>Register 15 with name ADC_ENC_TRACK_A of TMC8690.</desc>
		// \xml   <tmc:value_signed min="#s16_MIN#" max="#s16_MAX#" default="0" unit="" access="R"></tmc:value_signed>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ)
			*value = (s16) tmc8690_spi_readRegister16BitValue(ADC_ENC_TRACK_A, BIT_0_TO_15);
		else if(readWrite == WRITE)
			errors |= ERROR_TYPE;
		break;
	case 77:
		// \xml <tmc:axis_parameter type="77" name="ADC_ENC_TRACK_B" identifier="par::ADC_ENC_TRACK_B">
		// \xml <desc>Register 16 with name ADC_ENC_TRACK_B of TMC8690.</desc>
		// \xml   <tmc:value_signed min="#s16_MIN#" max="#s16_MAX#" default="0" unit="" access="R"></tmc:value_signed>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ)
			*value = (s16) tmc8690_spi_readRegister16BitValue(ADC_ENC_TRACK_B, BIT_0_TO_15);
		else if(readWrite == WRITE)
			errors |= ERROR_TYPE;
		break;
	case 78:
		// \xml <tmc:axis_parameter type="78" name="ADC_ENC_TRACK_Z" identifier="par::ADC_ENC_TRACK_Z">
		// \xml <desc>Register 17 with name ADC_ENC_TRACK_Z of TMC8690.</desc>
		// \xml   <tmc:value_unsigned min="0" max="#u16_MAX#" default="0" unit="" access="R"></tmc:value_unsigned>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ)
			*value = (u16) tmc8690_spi_readRegister16BitValue(ADC_ENC_TRACK_Z, BIT_0_TO_15);
		else if(readWrite == WRITE)
			errors |= ERROR_TYPE;
		break;
	case 79:
		// \xml <tmc:axis_parameter type="79" name="ADC_I_U" identifier="par::ADC_I_U">
		// \xml <desc>Register 18 with name ADC_I_U of TMC8690.</desc>
		// \xml   <tmc:value_unsigned min="0" max="#u16_MAX#" default="0" unit="" access="R"></tmc:value_unsigned>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ)
			*value = (u16) tmc8690_spi_readRegister16BitValue(ADC_I_U, BIT_0_TO_15);
		else if(readWrite == WRITE)
			errors |= ERROR_TYPE;
		break;
	case 80:
		// \xml <tmc:axis_parameter type="80" name="ADC_I_V" identifier="par::ADC_I_V">
		// \xml <desc>Register 19 with name ADC_I_V of TMC8690.</desc>
		// \xml   <tmc:value_unsigned min="0" max="#u16_MAX#" default="0" unit="" access="R"></tmc:value_unsigned>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ)
			*value = (u16) tmc8690_spi_readRegister16BitValue(ADC_I_V, BIT_0_TO_15);
		else if(readWrite == WRITE)
			errors |= ERROR_TYPE;
		break;
	case 81:
		// \xml <tmc:axis_parameter type="81" name="ADC_I_W" identifier="par::ADC_I_W">
		// \xml <desc>Register 20 with name ADC_I_W of TMC8690.</desc>
		// \xml   <tmc:value_unsigned min="0" max="#u16_MAX#" default="0" unit="" access="R"></tmc:value_unsigned>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ)
			*value = (u16) tmc8690_spi_readRegister16BitValue(ADC_I_W, BIT_0_TO_15);
		else if(readWrite == WRITE)
			errors |= ERROR_TYPE;
		break;
	case 82:
		// \xml <tmc:axis_parameter type="82" name="ADC_I_SUPPLY" identifier="par::ADC_I_SUPPLY">
		// \xml <desc>Register 21 with name ADC_I_SUPPLY of TMC8690.</desc>
		// \xml   <tmc:value_unsigned min="0" max="#u32_MAX#" default="0" unit="" access="R"></tmc:value_unsigned>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ)
			*value = (s32) tmc8690_readInt(motor, ADC_I_SUPPLY);
		else if(readWrite == WRITE)
			errors |= ERROR_TYPE;
		break;
	case 83:
		// \xml <tmc:axis_parameter type="83" name="ADC_I_SUPPLY_FILT" identifier="par::ADC_I_SUPPLY_FILT">
		// \xml <desc>Register 22 with name ADC_I_SUPPLY_FILT of TMC8690.</desc>
		// \xml   <tmc:value_unsigned min="0" max="#u32_MAX#" default="0" unit="" access="R"></tmc:value_unsigned>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ)
			*value = (s32) tmc8690_readInt(motor, ADC_I_SUPPLY_FILT);
		else if(readWrite == WRITE)
			errors |= ERROR_TYPE;
		break;
	case 84:
		// \xml <tmc:axis_parameter type="84" name="ADC_TORQUE" identifier="par::ADC_TORQUE">
		// \xml <desc>Register 23 with name ADC_TORQUE of TMC8690.</desc>
		// \xml   <tmc:value_unsigned min="0" max="#u16_MAX#" default="0" unit="" access="R"></tmc:value_unsigned>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ)
			*value = (u16) tmc8690_spi_readRegister16BitValue(ADC_TORQUE, BIT_0_TO_15);
		else if(readWrite == WRITE)
			errors |= ERROR_TYPE;
		break;
	case 85:
		// \xml <tmc:axis_parameter type="85" name="ADC_TEMP" identifier="par::ADC_TEMP">
		// \xml <desc>Register 24 with name ADC_TEMP of TMC8690.</desc>
		// \xml   <tmc:value_unsigned min="0" max="#u16_MAX#" default="0" unit="" access="R"></tmc:value_unsigned>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ)
			*value = (u16) tmc8690_spi_readRegister16BitValue(ADC_TEMP, BIT_0_TO_15);
		else if(readWrite == WRITE)
			errors |= ERROR_TYPE;
		break;
	case 86:
		// \xml <tmc:axis_parameter type="86" name="ADC_MICROPHONE" identifier="par::ADC_MICROPHONE">
		// \xml <desc>Register 25 with name ADC_MICROPHONE of TMC8690.</desc>
		// \xml   <tmc:value_unsigned min="0" max="#u16_MAX#" default="0" unit="" access="R"></tmc:value_unsigned>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ)
			*value = (u16) tmc8690_spi_readRegister16BitValue(ADC_MICROPHONE, BIT_0_TO_15);
		else if(readWrite == WRITE)
			errors |= ERROR_TYPE;
		break;
	case 87:
		// \xml <tmc:axis_parameter type="87" name="ADC_I_BRAKE" identifier="par::ADC_I_BRAKE">
		// \xml <desc>Register 26 with name ADC_I_BRAKE of TMC8690.</desc>
		// \xml   <tmc:value_unsigned min="0" max="#u16_MAX#" default="0" unit="" access="R"></tmc:value_unsigned>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ)
			*value = (u16) tmc8690_spi_readRegister16BitValue(ADC_I_BRAKE, BIT_0_TO_15);
		else if(readWrite == WRITE)
			errors |= ERROR_TYPE;
		break;
	case 88:
		// \xml <tmc:axis_parameter type="88" name="TRACK_A_OFFSET" identifier="par::TRACK_A_OFFSET">
		// \xml <desc>Register 30 with name TRACK_A_OFFSET of TMC8690.</desc>
		// \xml   <tmc:value_unsigned min="0" max="#u16_MAX#" default="0" unit="" access="RW"></tmc:value_unsigned>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ)
			*value = (u16) tmc8690_spi_readRegister16BitValue(TRACK_A_OFFSET, BIT_0_TO_15);
		else if(readWrite == WRITE)
			tmc8690_spi_writeRegister16BitValue(TRACK_A_OFFSET, BIT_0_TO_15, *value);
		break;
	case 89:
		// \xml <tmc:axis_parameter type="89" name="TRACK_B_OFFSET" identifier="par::TRACK_B_OFFSET">
		// \xml <desc>Register 31 with name TRACK_B_OFFSET of TMC8690.</desc>
		// \xml   <tmc:value_unsigned min="0" max="#u16_MAX#" default="0" unit="" access="RW"></tmc:value_unsigned>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ)
			*value = (u16) tmc8690_spi_readRegister16BitValue(TRACK_B_OFFSET, BIT_0_TO_15);
		else if(readWrite == WRITE)
			tmc8690_spi_writeRegister16BitValue(TRACK_B_OFFSET, BIT_0_TO_15, *value);
		break;
	case 90:
		// \xml <tmc:axis_parameter type="90" name="DAC_V_REF" identifier="par::DAC_V_REF">
		// \xml <desc>Register 32 with name DAC_V_REF of TMC8690.</desc>
		// \xml   <tmc:value_unsigned min="0" max="#u16_MAX#" default="0" unit="" access="RW"></tmc:value_unsigned>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ)
			*value = (u16) tmc8690_spi_readRegister16BitValue(DAC_V_REF, BIT_0_TO_15);
		else if(readWrite == WRITE)
			tmc8690_spi_writeRegister16BitValue(DAC_V_REF, BIT_0_TO_15, *value);
		break;
	case 91:
		// \xml <tmc:axis_parameter type="91" name="T_PT1_FILTER_VEL" identifier="par::T_PT1_FILTER_VEL">
		// \xml <desc>Register 33 with name T_PT1_FILTER_VEL of TMC8690.</desc>
		// \xml   <tmc:value_unsigned min="0" max="#u16_MAX#" default="0" unit="" access="RW"></tmc:value_unsigned>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ)
			*value = (u16) tmc8690_spi_readRegister16BitValue(T_PT1_FILTER_VEL, BIT_0_TO_15);
		else if(readWrite == WRITE)
			tmc8690_spi_writeRegister16BitValue(T_PT1_FILTER_VEL, BIT_0_TO_15, *value);
		break;
	case 92:
		// \xml <tmc:axis_parameter type="92" name="T_PT1_FILTER_I_SUPPLY" identifier="par::T_PT1_FILTER_I_SUPPLY">
		// \xml <desc>Register 34 with name T_PT1_FILTER_I_SUPPLY of TMC8690.</desc>
		// \xml   <tmc:value_unsigned min="0" max="#u16_MAX#" default="0" unit="" access="RW"></tmc:value_unsigned>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ)
			*value = (u16) tmc8690_spi_readRegister16BitValue(T_PT1_FILTER_I_SUPPLY, BIT_0_TO_15);
		else if(readWrite == WRITE)
			tmc8690_spi_writeRegister16BitValue(T_PT1_FILTER_I_SUPPLY, BIT_0_TO_15, *value);
		break;
	case 93:
		// \xml <tmc:axis_parameter type="93" name="DAC_BRAKE" identifier="par::DAC_BRAKE">
		// \xml <desc>Register 35 with name DAC_BRAKE of TMC8690.</desc>
		// \xml   <tmc:value_signed min="#s32_MIN#" max="#s32_MAX#" default="0" unit="" access="R"></tmc:value_signed>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ)
			*value = (u32) tmc8690_readInt(motor, DAC_BRAKE);
		else if(readWrite == WRITE)
			errors |= ERROR_TYPE;
		break;
	case 94:
		// \xml <tmc:axis_parameter type="94" name="DAC_BRAKE_TARGET" identifier="par::DAC_BRAKE_TARGET">
		// \xml <desc>Register 36 with name DAC_BRAKE_TARGET of TMC8690.</desc>
		// \xml   <tmc:value_signed min="#s32_MIN#" max="#s32_MAX#" default="0" unit="" access="RW"></tmc:value_signed>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ)
			*value = (u32) tmc8690_readInt(motor, DAC_BRAKE_TARGET);
		else if(readWrite == WRITE)
			tmc8690_writeInt(motor, DAC_BRAKE_TARGET, *value);
		break;
	case 95:
		// \xml <tmc:axis_parameter type="95" name="VELOCITY_ACTUAL" identifier="par::VELOCITY_ACTUAL">
		// \xml <desc>Register 37 with name VELOCITY_ACTUAL of TMC8690.</desc>
		// \xml   <tmc:value_signed min="#s32_MIN#" max="#s32_MAX#" default="0" unit="" access="R"></tmc:value_signed>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ)
			*value = (u32) tmc8690_readInt(motor, VELOCITY_ACTUAL);
		else if(readWrite == WRITE)
			errors |= ERROR_TYPE;
		break;
	case 96:
		// \xml <tmc:axis_parameter type="96" name="VELOCITY_IN_RPM_ACTUAL" identifier="par::VELOCITY_IN_RPM_ACTUAL">
		// \xml <desc>Register 38 with name VELOCITY_IN_RPM_ACTUAL of TMC8690.</desc>
		// \xml   <tmc:value_signed min="#s32_MIN#" max="#s32_MAX#" default="0" unit="" access="R"></tmc:value_signed>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ)
			*value = (u32) tmc8690_readInt(motor, VELOCITY_IN_RPM_ACTUAL);
		else if(readWrite == WRITE)
			errors |= ERROR_TYPE;
		break;
	case 97:
		// \xml <tmc:axis_parameter type="97" name="VELOCITY_IN_RPM_ACTUAL_PT1" identifier="par::VELOCITY_IN_RPM_ACTUAL_PT1">
		// \xml <desc>Register 39 with name VELOCITY_IN_RPM_ACTUAL_PT1 of TMC8690.</desc>
		// \xml   <tmc:value_signed min="#s32_MIN#" max="#s32_MAX#" default="0" unit="" access="R"></tmc:value_signed>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ)
			*value = (u32) tmc8690_readInt(motor, VELOCITY_IN_RPM_ACTUAL_PT1);
		else if(readWrite == WRITE)
			errors |= ERROR_TYPE;
		break;
	case 98:
		// \xml <tmc:axis_parameter type="98" name="VELOCITY_IN_RPM_ACTUAL_MAV" identifier="par::VELOCITY_IN_RPM_ACTUAL_MAV">
		// \xml <desc>Register 3A with name VELOCITY_IN_RPM_ACTUAL_MAV of TMC8690.</desc>
		// \xml   <tmc:value_signed min="#s32_MIN#" max="#s32_MAX#" default="0" unit="" access="R"></tmc:value_signed>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ)
			*value = (u32) tmc8690_readInt(motor, VELOCITY_IN_RPM_ACTUAL_MAV);
		else if(readWrite == WRITE)
			errors |= ERROR_TYPE;
		break;
	case 99:
		// \xml <tmc:axis_parameter type="99" name="POSITION_ACTUAL" identifier="par::POSITION_ACTUAL">
		// \xml <desc>Register 3B with name POSITION_ACTUAL of TMC8690.</desc>
		// \xml   <tmc:value_signed min="#s32_MIN#" max="#s32_MAX#" default="0" unit="" access="R"></tmc:value_signed>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ)
			*value = (u32) tmc8690_readInt(motor, POSITION_ACTUAL);
		else if(readWrite == WRITE)
			errors |= ERROR_TYPE;
		break;
	case 100:
		// \xml <tmc:axis_parameter type="100" name="VEL_CTRL_TORQUE_TARGET" identifier="par::VEL_CTRL_TORQUE_TARGET">
		// \xml <desc>Register 3C with name VEL_CTRL_TORQUE_TARGET of TMC8690.</desc>
		// \xml   <tmc:value_unsigned min="0" max="#u16_MAX#" default="0" unit="" access="R"></tmc:value_unsigned>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ)
			*value = (u16) tmc8690_spi_readRegister16BitValue(VEL_CTRL_TORQUE_TARGET, BIT_0_TO_15);
		else if(readWrite == WRITE)
			errors |= ERROR_TYPE;
		break;
	case 101:
		// \xml <tmc:axis_parameter type="101" name="POS_CTRL_VEL_TARGET" identifier="par::POS_CTRL_VEL_TARGET">
		// \xml <desc>Register 3D with name POS_CTRL_VEL_TARGET of TMC8690.</desc>
		// \xml   <tmc:value_unsigned min="0" max="#u16_MAX#" default="0" unit="" access="R"></tmc:value_unsigned>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ)
			*value = (u16) tmc8690_spi_readRegister16BitValue(POS_CTRL_VEL_TARGET, BIT_0_TO_15);
		else if(readWrite == WRITE)
			errors |= ERROR_TYPE;
		break;
	case 102:
		// \xml <tmc:axis_parameter type="102" name="PWM_HEATER" identifier="par::PWM_HEATER">
		// \xml <desc>Register 3E with name PWM_HEATER of TMC8690.</desc>
		// \xml   <tmc:value_unsigned min="0" max="#u16_MAX#" default="0" unit="" access="R"></tmc:value_unsigned>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ)
			*value = (u16) tmc8690_spi_readRegister16BitValue(PWM_HEATER, BIT_0_TO_15);
		else if(readWrite == WRITE)
			errors |= ERROR_TYPE;
		break;
	case 103:
		// \xml <tmc:axis_parameter type="103" name="U_BRAKE" identifier="par::U_BRAKE">
		// \xml <desc>Register 3F with name U_BRAKE of TMC8690.</desc>
		// \xml   <tmc:value_unsigned min="0" max="#u16_MAX#" default="0" unit="" access="R"></tmc:value_unsigned>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ)
			*value = (u16) tmc8690_spi_readRegister16BitValue(U_BRAKE, BIT_0_TO_15);
		else if(readWrite == WRITE)
			errors |= ERROR_TYPE;
		break;
	case 104:
		// \xml <tmc:axis_parameter type="104" name="DEBUG_AXIS_PARAM_0" identifier="par::DEBUG_AXIS_PARAM_0">
		// \xml <desc>Register 50 with name DEBUG_AXIS_PARAM_0 of TMC8690.</desc>
		// \xml   <tmc:value_unsigned min="0" max="#u32_MAX#" default="0" unit="" access="RW"></tmc:value_unsigned>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ)
			*value = (s32) tmc8690_readInt(motor, DEBUG_AXIS_PARAM_0);
		else if(readWrite == WRITE)
			tmc8690_writeInt(motor, DEBUG_AXIS_PARAM_0, *value);
		break;
	case 105:
		// \xml <tmc:axis_parameter type="105" name="DEBUG_AXIS_PARAM_1" identifier="par::DEBUG_AXIS_PARAM_1">
		// \xml <desc>Register 51 with name DEBUG_AXIS_PARAM_1 of TMC8690.</desc>
		// \xml   <tmc:value_unsigned min="0" max="#u32_MAX#" default="0" unit="" access="RW"></tmc:value_unsigned>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ)
			*value = (s32) tmc8690_readInt(motor, DEBUG_AXIS_PARAM_1);
		else if(readWrite == WRITE)
			tmc8690_writeInt(motor, DEBUG_AXIS_PARAM_1, *value);
		break;
	case 106:
		// \xml <tmc:axis_parameter type="106" name="DEBUG_AXIS_PARAM_2" identifier="par::DEBUG_AXIS_PARAM_2">
		// \xml <desc>Register 52 with name DEBUG_AXIS_PARAM_2 of TMC8690.</desc>
		// \xml   <tmc:value_unsigned min="0" max="#u32_MAX#" default="0" unit="" access="RW"></tmc:value_unsigned>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ)
			*value = (s32) tmc8690_readInt(motor, DEBUG_AXIS_PARAM_2);
		else if(readWrite == WRITE)
			tmc8690_writeInt(motor, DEBUG_AXIS_PARAM_2, *value);
		break;
	case 107:
		// \xml <tmc:axis_parameter type="107" name="DEBUG_AXIS_PARAM_3" identifier="par::DEBUG_AXIS_PARAM_3">
		// \xml <desc>Register 53 with name DEBUG_AXIS_PARAM_3 of TMC8690.</desc>
		// \xml   <tmc:value_unsigned min="0" max="#u32_MAX#" default="0" unit="" access="RW"></tmc:value_unsigned>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ)
			*value = (s32) tmc8690_readInt(motor, DEBUG_AXIS_PARAM_3);
		else if(readWrite == WRITE)
			tmc8690_writeInt(motor, DEBUG_AXIS_PARAM_3, *value);
		break;
	case 108:
		// \xml <tmc:axis_parameter type="108" name="DEBUG_AXIS_PARAM_4" identifier="par::DEBUG_AXIS_PARAM_4">
		// \xml <desc>Register 54 with name DEBUG_AXIS_PARAM_4 of TMC8690.</desc>
		// \xml   <tmc:value_unsigned min="0" max="#u32_MAX#" default="0" unit="" access="RW"></tmc:value_unsigned>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ)
			*value = (s32) tmc8690_readInt(motor, DEBUG_AXIS_PARAM_4);
		else if(readWrite == WRITE)
			tmc8690_writeInt(motor, DEBUG_AXIS_PARAM_4, *value);
		break;
	case 109:
		// \xml <tmc:axis_parameter type="109" name="DEBUG_AXIS_PARAM_5" identifier="par::DEBUG_AXIS_PARAM_5">
		// \xml <desc>Register 55 with name DEBUG_AXIS_PARAM_5 of TMC8690.</desc>
		// \xml   <tmc:value_unsigned min="0" max="#u32_MAX#" default="0" unit="" access="RW"></tmc:value_unsigned>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ)
			*value = (s32) tmc8690_readInt(motor, DEBUG_AXIS_PARAM_5);
		else if(readWrite == WRITE)
			tmc8690_writeInt(motor, DEBUG_AXIS_PARAM_5, *value);
		break;
	case 110:
		// \xml <tmc:axis_parameter type="110" name="DEBUG_AXIS_PARAM_6" identifier="par::DEBUG_AXIS_PARAM_6">
		// \xml <desc>Register 56 with name DEBUG_AXIS_PARAM_6 of TMC8690.</desc>
		// \xml   <tmc:value_unsigned min="0" max="#u32_MAX#" default="0" unit="" access="RW"></tmc:value_unsigned>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ)
			*value = (s32) tmc8690_readInt(motor, DEBUG_AXIS_PARAM_6);
		else if(readWrite == WRITE)
			tmc8690_writeInt(motor, DEBUG_AXIS_PARAM_6, *value);
		break;
	case 111:
		// \xml <tmc:axis_parameter type="111" name="DEBUG_AXIS_PARAM_7" identifier="par::DEBUG_AXIS_PARAM_7">
		// \xml <desc>Register 57 with name DEBUG_AXIS_PARAM_7 of TMC8690.</desc>
		// \xml   <tmc:value_unsigned min="0" max="#u32_MAX#" default="0" unit="" access="RW"></tmc:value_unsigned>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ)
			*value = (s32) tmc8690_readInt(motor, DEBUG_AXIS_PARAM_7);
		else if(readWrite == WRITE)
			tmc8690_writeInt(motor, DEBUG_AXIS_PARAM_7, *value);
		break;
	case 112:
		// \xml <tmc:axis_parameter type="112" name="STATE_VAR_PARAM_0" identifier="par::STATE_VAR_PARAM_0">
		// \xml <desc>Register 58 with name STATE_VAR_PARAM_0 of TMC8690.</desc>
		// \xml   <tmc:value_unsigned min="0" max="#u32_MAX#" default="0" unit="" access="R"></tmc:value_unsigned>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ)
			*value = (s32) tmc8690_readInt(motor, STATE_VAR_PARAM_0);
		else if(readWrite == WRITE)
			errors |= ERROR_TYPE;
		break;
	case 113:
		// \xml <tmc:axis_parameter type="113" name="STATE_VAR_PARAM_1" identifier="par::STATE_VAR_PARAM_1">
		// \xml <desc>Register 59 with name STATE_VAR_PARAM_1 of TMC8690.</desc>
		// \xml   <tmc:value_unsigned min="0" max="#u32_MAX#" default="0" unit="" access="R"></tmc:value_unsigned>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ)
			*value = (s32) tmc8690_readInt(motor, STATE_VAR_PARAM_1);
		else if(readWrite == WRITE)
			errors |= ERROR_TYPE;
		break;
	case 114:
		// \xml <tmc:axis_parameter type="114" name="STATE_VAR_PARAM_2" identifier="par::STATE_VAR_PARAM_2">
		// \xml <desc>Register 5A with name STATE_VAR_PARAM_2 of TMC8690.</desc>
		// \xml   <tmc:value_unsigned min="0" max="#u32_MAX#" default="0" unit="" access="R"></tmc:value_unsigned>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ)
			*value = (s32) tmc8690_readInt(motor, STATE_VAR_PARAM_2);
		else if(readWrite == WRITE)
			errors |= ERROR_TYPE;
		break;
	case 115:
		// \xml <tmc:axis_parameter type="115" name="STATE_VAR_PARAM_3" identifier="par::STATE_VAR_PARAM_3">
		// \xml <desc>Register 5B with name STATE_VAR_PARAM_3 of TMC8690.</desc>
		// \xml   <tmc:value_unsigned min="0" max="#u32_MAX#" default="0" unit="" access="R"></tmc:value_unsigned>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ)
			*value = (s32) tmc8690_readInt(motor, STATE_VAR_PARAM_3);
		else if(readWrite == WRITE)
			errors |= ERROR_TYPE;
		break;
	case 116:
		// \xml <tmc:axis_parameter type="116" name="STATE_VAR_PARAM_4" identifier="par::STATE_VAR_PARAM_4">
		// \xml <desc>Register 5C with name STATE_VAR_PARAM_4 of TMC8690.</desc>
		// \xml   <tmc:value_unsigned min="0" max="#u32_MAX#" default="0" unit="" access="R"></tmc:value_unsigned>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ)
			*value = (s32) tmc8690_readInt(motor, STATE_VAR_PARAM_4);
		else if(readWrite == WRITE)
			errors |= ERROR_TYPE;
		break;
	case 117:
		// \xml <tmc:axis_parameter type="117" name="STATE_VAR_PARAM_5" identifier="par::STATE_VAR_PARAM_5">
		// \xml <desc>Register 5D with name STATE_VAR_PARAM_5 of TMC8690.</desc>
		// \xml   <tmc:value_unsigned min="0" max="#u32_MAX#" default="0" unit="" access="R"></tmc:value_unsigned>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ)
			*value = (s32) tmc8690_readInt(motor, STATE_VAR_PARAM_5);
		else if(readWrite == WRITE)
			errors |= ERROR_TYPE;
		break;
	case 118:
		// \xml <tmc:axis_parameter type="118" name="STATE_VAR_PARAM_6" identifier="par::STATE_VAR_PARAM_6">
		// \xml <desc>Register 5E with name STATE_VAR_PARAM_6 of TMC8690.</desc>
		// \xml   <tmc:value_unsigned min="0" max="#u32_MAX#" default="0" unit="" access="R"></tmc:value_unsigned>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ)
			*value = (s32) tmc8690_readInt(motor, STATE_VAR_PARAM_6);
		else if(readWrite == WRITE)
			errors |= ERROR_TYPE;
		break;
	case 119:
		// \xml <tmc:axis_parameter type="119" name="STATE_VAR_PARAM_7" identifier="par::STATE_VAR_PARAM_7">
		// \xml <desc>Register 5F with name STATE_VAR_PARAM_7 of TMC8690.</desc>
		// \xml   <tmc:value_unsigned min="0" max="#u32_MAX#" default="0" unit="" access="R"></tmc:value_unsigned>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ)
			*value = (s32) tmc8690_readInt(motor, STATE_VAR_PARAM_7);
		else if(readWrite == WRITE)
			errors |= ERROR_TYPE;
		break;
	case 120:
		// \xml <tmc:axis_parameter type="120" name="SAMP_INPUTS_RAW" identifier="par::SAMP_INPUTS_RAW">
		// \xml <desc>Register 76 with name SAMP_INPUTS_RAW of TMC8690.</desc>
		// \xml   <tmc:value_unsigned min="0" max="#u32_MAX#" default="0" unit="" access="R"></tmc:value_unsigned>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ)
			*value = (s32) tmc8690_readInt(motor, SAMP_INPUTS_RAW);
		else if(readWrite == WRITE)
			errors |= ERROR_TYPE;
		break;
	case 121:
		// \xml <tmc:axis_parameter type="121" name="SAMP_OUTPUTS_RAW" identifier="par::SAMP_OUTPUTS_RAW">
		// \xml <desc>Register 77 with name SAMP_OUTPUTS_RAW of TMC8690.</desc>
		// \xml   <tmc:value_unsigned min="0" max="#u32_MAX#" default="0" unit="" access="R"></tmc:value_unsigned>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ)
			*value = (s32) tmc8690_readInt(motor, SAMP_OUTPUTS_RAW);
		else if(readWrite == WRITE)
			errors |= ERROR_TYPE;
		break;
	case 122:
		// \xml <tmc:axis_parameter type="122" name="STATUS_FLAGS" identifier="par::STATUS_FLAGS">
		// \xml <desc>Register 7C with name STATUS_FLAGS of TMC8690.</desc>
		// \xml   <tmc:value_unsigned min="0" max="#u32_MAX#" default="0" unit="" access="RW"></tmc:value_unsigned>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ)
			*value = (s32) tmc8690_readInt(motor, STATUS_FLAGS);
		else if(readWrite == WRITE)
			tmc8690_writeInt(motor, STATUS_FLAGS, *value);
		break;
	case 123:
		// \xml <tmc:axis_parameter type="123" name="WARNING_MASK" identifier="par::WARNING_MASK">
		// \xml <desc>Register 7D with name WARNING_MASK of TMC8690.</desc>
		// \xml   <tmc:value_unsigned min="0" max="#u32_MAX#" default="0" unit="" access="RW"></tmc:value_unsigned>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ)
			*value = (s32) tmc8690_readInt(motor, WARNING_MASK);
		else if(readWrite == WRITE)
			tmc8690_writeInt(motor, WARNING_MASK, *value);
		break;
	case 124:
		// \xml <tmc:axis_parameter type="124" name="ERROR_MASK" identifier="par::ERROR_MASK">
		// \xml <desc>Register 7E with name ERROR_MASK of TMC8690.</desc>
		// \xml   <tmc:value_unsigned min="0" max="#u32_MAX#" default="0" unit="" access="RW"></tmc:value_unsigned>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ)
			*value = (s32) tmc8690_readInt(motor, ERROR_MASK);
		else if(readWrite == WRITE)
			tmc8690_writeInt(motor, ERROR_MASK, *value);
		break;
	case 255:
		// \xml <tmc:axis_parameter type="255" name="Main loop counter" identifier="par::MainLoopCounter">
		// \xml    <desc></desc>
		// \xml    <tmc:value_unsigned min="0" max="#u32_MAX#" default="0" unit="" access="R"></tmc:value_unsigned>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			*value = mainLoopCount;
		} else if(readWrite == WRITE) {
			errors |= ERROR_TYPE;
		}
		break;
	default:
		errors |= ERROR_TYPE;
		break;
	}
	// \xml </axis_parameters>
	return errors;
}
// \xml </tmc:product_module>

static void periodicJob(uint32 actualSystick)
{
	static u32 loopCounterCheckTime;
	static u32 loopCounter = 0;

	if((actualSystick - loopCounterCheckTime) >= 1000)
	{
		mainLoopCount = loopCounter;
		loopCounter = 0;

		loopCounterCheckTime = actualSystick;
	}
	loopCounter++;
}

static void writeRegister(uint8 address, int32 value)
{
	tmc8690_writeInt(0, address, value);
}

static void readRegister(uint8 address, int32 *value)
{
	*value = tmc8690_readInt(0, address);
}

static uint32 SAP(uint8 type, uint8 motor, int32 value)
{
	return handleParameter(WRITE, motor, type, &value);
}

static uint32 GAP(uint8 type, uint8 motor, int32 *value)
{
	return handleParameter(READ, motor, type, value);
}

static uint32 userFunction(uint8 type, uint8 motor, int32 *value)
{
	UNUSED(type);
	UNUSED(motor);
	UNUSED(value);
	return 0;
}

static void enableDriver(DriverState state)
{
	if(state == DRIVER_USE_GLOBAL_ENABLE)
		state = Evalboards.driverEnable;

	if(state == DRIVER_DISABLE)
		HAL.IOs->config->setLow(PIN_DRV_ENN);
	else if((state == DRIVER_ENABLE) && (Evalboards.driverEnable == DRIVER_ENABLE))
		HAL.IOs->config->setHigh(PIN_DRV_ENN);
}

static void deInit(void)
{
	enableDriver(DRIVER_DISABLE);
	HAL.IOs->config->reset(PIN_DRV_ENN);
};

static uint8 reset()
{
	return 1;
}

static uint8 restore()
{
	return 1;
}

static void checkErrors(uint32 tick)
{
	UNUSED(tick);
	Evalboards.ch1.errors = 0;
}

void TMC8690_init(void)
{
	// configure ENABLE-PIN for TMC8690
	PIN_DRV_ENN = &HAL.IOs->pins->DIO0;
	HAL.IOs->config->toOutput(PIN_DRV_ENN);
	HAL.IOs->config->setHigh(PIN_DRV_ENN);

	TMC8690_config = Evalboards.ch1.config;

	// connect evalboard functions
	Evalboards.ch1.config->reset        = reset;
	Evalboards.ch1.config->restore      = restore;
	Evalboards.ch1.config->state        = CONFIG_RESET;
	Evalboards.ch1.config->configIndex  = 0;
//// todo ADD 3: erweitern! (ED) => #2
//	Evalboards.ch1.rotate               = rotate;
//	Evalboards.ch1.right                = right;
//	Evalboards.ch1.left                 = left;
//	Evalboards.ch1.stop                 = stop;
//	Evalboards.ch1.getMeasuredSpeed     = getMeasuredSpeed;
//// <=
	Evalboards.ch1.GAP                  = GAP;
	Evalboards.ch1.SAP                  = SAP;
//	Evalboards.ch1.moveTo               = moveTo;
//	Evalboards.ch1.moveBy               = moveBy;
	Evalboards.ch1.writeRegister        = writeRegister;
	Evalboards.ch1.readRegister         = readRegister;
	Evalboards.ch1.periodicJob          = periodicJob;
	Evalboards.ch1.userFunction         = userFunction;
	Evalboards.ch1.enableDriver         = enableDriver;
	Evalboards.ch1.checkErrors          = checkErrors;
	Evalboards.ch1.numberOfMotors       = MOTORS;
	Evalboards.ch1.deInit               = deInit;
	Evalboards.ch1.VMMin                = VM_MIN;
	Evalboards.ch1.VMMax                = VM_MAX;

	// init used API functions
//	tmc8690_init(); // no init function yet

	// enable the driver
	enableDriver(DRIVER_ENABLE);
};

